<?php
/**
 * @file
 * Add geodata.
 */

define('GEODATA_KEEP_PK_ADD', 0);
define('GEODATA_REPLACE_PK_ADD', 1);
define('GEODATA_IGNORE_PK_ADD', 2);

function geodata_add_form($form_state, &$node) {
  $form = array();
  
  $form['import'] = array('#type' => 'fieldset'
    , '#title' => t('Import')
    , '#collapsible' => TRUE
    );
  
  // Drupal form api: File fields are NOT allowed to be required.
  $form['import']['data_file'] = array('#type' => 'file'
    , '#title' => t('File')
    , '#default_value' => ''
    , '#description' => t('Select the file to add to this data set.')
    );
  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  
  $form['import']['key_option'] = array('#type' => 'radios'
    , '#title' => t('Key options')
    , '#options' => array(GEODATA_KEEP_PK_ADD => t('Keep present data'), GEODATA_REPLACE_PK_ADD => t('Replace with new data'), GEODATA_IGNORE_PK_ADD => t('Ignore'))
    , '#default_value' => GEODATA_KEEP_PK_ADD
    , '#description' => t('How to handle duplicate values of the primary key fields.')
    );
  $form['import']['srid'] = array('#type' => 'textfield'
    , '#title' => t('SRS')
    , '#default_value' => isset($node->srid) ? $node->srid : '4326'
    , '#description' => t('Enter SRS of the import file, without the EPSG: part.')
    , '#size' => 10
    , '#maxlength' => 6
    );
  
  if (module_exists('geometadata')) {
    if ($node->metadata_nid > 0) {
      $form['metadata'] = array('#type' => 'fieldset'
        , '#title' => t('Metadata')
        , '#collapsible' => TRUE
        );
      $form['metadata']['metadata-file'] = array('#type' => 'file'
        , '#title' => t('Metadata file')
        , '#required' => $node->metadata_required == 1
        );
    }
  }
  
  $form['nid'] = array('#type' => 'value', '#value' => $node->nid);
  
  $form['submit'] = array('#type' => 'submit'
    , '#value' => t('Add')
    );
  
  $form['#validate'][] = 'geodata_add_validate';
  $form['#submit'][] = 'geodata_add_submit';
  
  return $form;
}

/**
 *
 */
function geodata_add_validate(&$form, &$form_state) {
  drupal_set_message('geodata_add_validate '. var_export($form_state, TRUE));
  
  $validators = array('geodata_upload_file_validate' => array());
  if ($file = file_save_upload('data_file', $validators)) {
    // do something?
    form_set_value($form['import']['data_file'], $file->filename, $form_state);
  }
  else {
    form_set_error('data_file', t('Not a valid data file.'));
  }
  drupal_set_message('upload file: '. var_export($file, TRUE));
}

/**
 *
 */
function geodata_add_submit(&$form, &$form_state) {
  //drupal_set_message('geodata_add_submit:'. var_export(array('state'=>$form_state), TRUE));
  
  $node = node_load(array('nid' => $form_state['values']['nid']));
  if (!$node) {
    return;
  }
  
  $validators = array('geodata_upload_file_validate' => array());
  if ($file = file_save_upload('data_file', $validators)) {
    // unzip archive into temporary folder
    $shape_dir = file_directory_temp() .'/'. uniqid();
    $cmd = strtr(variable_get('mappingkit_unzip_command', DEFAULT_MAPPINGKIT_UNZIP_COMMAND)
      , array('%archive' => $file->filepath
        , '%tempdir' => $shape_dir
        )
      );
    $results = mappingkit_proc($cmd);
    drupal_set_message('unzip results:'. var_export($results, TRUE));
    
    $table_names = array();
    global $db_url;
    if (isset($db_url[$node->database_name])) {
      $geodata = parse_url($db_url[$node->database_name]);
      $geodata['database'] = substr($geodata['path'], 1);
      
      foreach (glob($shape_dir .'/*.shp') as $filename) {
        // import shapes into temporary table
        drupal_set_message('shape file:'.$filename);
        $table_names[] = $table_name = 'tmp_'. uniqid();
        
        // create new table (-c), or drop tabel and recreate it (-d)
        $cmd = strtr('"%postgresql_bin/shp2pgsql%exe" -s %srid -W ISO-8859-1 -g geom -I -c %shapefile %table | %postgresql_bin/psql%exe -h %host %port_option -U %user %database'
          , array('%postgresql_bin' => variable_get('mappingkit_postgresql_bin_path', DEFAULT_MAPPINGKIT_POSTGRESQL_BIN_PATH)
            , '%srid' => $form_state['values']['srid']
            , '%shapefile' => $filename
            , '%table' => $table_name
            , '%host' => $geodata['host']
            , '%port_option' => isset($geodata['port']) ? '-p '. $geodata['port'] : ''
            , '%user' => $geodata['user']
            , '%database' => $geodata['database']
            , '%exe' => MAPPINGKIT_EXE_TYPE
            )
          );
        
        $results = mappingkit_proc($cmd);
        
        // stdout
        //$count = preg_match('/INSERT(.*)\n/m', $results['stdout'], $matches);
        
        drupal_set_message('import results:'. var_export($results, TRUE));
        //drupal_set_message(var_export($matches, TRUE));
        
        drupal_set_message(preg_replace('/\n/', '<br/>', $results['stderr']));
      }
      // delay so that postgresql 'knows' the new tables
      //sleep(10);
      
      $prev_db = db_set_active($node->database_name);
      
      // import and cleanup temporary tables into node
      $dest_table = $node->data_path;
      foreach ($table_names as $src_table) {
        $query = '';
        //$query .= "BEGIN;";
        
        // add uid to src_table
        global $user;
        db_query("alter table $src_table add column _importuid int; update $src_table set _importuid = {$user->uid};");
        
        // import based on corresponding field names in import table and
        // destination table
        $dest_fields = geodata_describe_table($src_table, $node->database_name);
        $common_fields = implode(',', array_intersect($dest_fields, $node->fields));
        
        // key_option
        if ($form_state['values']['key_option'] != GEODATA_IGNORE_PK_ADD && count($node->keyfields) > 0) {
          $key_clause = '';
          $op = '';
          foreach ($node->keyfields as $key) {
            $key_clause .= $op .'s.'. $key .' = d.'. $key;
            $op = ' and ';
          }
          
          if ($form_state['values']['key_option'] == GEODATA_REPLACE_PK_ADD) {
            // delete records from destination if same key is present in new
            $query .= "delete from $dest_table as d where exists(select 1 from $src_table as s where $key_clause);";
          }
          else if ($form_state['values']['key_option'] == GEODATA_KEEP_PK_ADD) {
            // delete records from new if same key is present in old
            $query .= "delete from $src_table as s where exists(select 1 from $dest_table as d where $key_clause);";
          }
        }
        
        // import from src to dest
        $query .= "insert into $dest_table($common_fields) select $common_fields from $src_table;";
        
        //$query .= "COMMIT;";
        drupal_set_message('query:'.$query);
        db_query($query);
        
        // clean up
        //drupal_set_message("select dropgeometrytable('$src_table');");
        db_query("select dropgeometrytable('$src_table')");
      }
      
      db_set_active($prev_db);
    }
    
    // Remove the shape dir; note that the uploaded zip files are automatically
    // deleted by cron
    mk_remove_files($shape_dir);
  }
  
  // redirect to node view
  $parameters = $form['#parameters'];
  $form_state['redirect'] = 'node/'. $parameters[2]->nid;
}

