<?php
/**
 * @file
 * Export geodata.
 */

function geodata_export_form($form_state, &$node) {
  $form = array();
  
  //drupal_set_message('geodata_export_form: '. var_export(array('state'=>$form_state,'node'=>$node), TRUE));
  
  $form['output'] = array('#type' => 'fieldset'
    , '#title' => t('Output')
    );
  $form['output']['shapefile'] = array('#type' => 'textfield'
    , '#title' => t('Shapefile name')
    , '#default_value' => $node->data_path
    );
  
  $form['source']['geodata_type'] = array('#type' => 'value'
    , '#value' => $node->geodata_type
    );
  $form['source']['data_path'] = array('#type' => 'value'
    , '#value' => $node->data_path
    );
  $form['source']['nid'] = array('#type' => 'value'
    , '#value' => $node->nid
    );
  $form['export_web_path'] = array('#type' => 'value'
    , '#value' => ''
    );
  /*
  $form['output']['archive_type'] = array('#type' => 'select'
    , '#options' => array('7z' => '7z', 'zip' => 'zip', 'gzip' => 'gzip', 'bzip2' => 'bzip2', 'tar' => 'tar')
    , '#default_value' => 'zip'
    );
  */
  
  $form['submit'] = array('#type' => 'submit'
    , '#value' => t('Download')
    );
  
  $form['#validate'][] = 'geodata_export_validate';
  $form['#submit'][] = 'geodata_export_submit';
  
  return $form;
}

function geodata_export_validate($form, &$form_state) {
  global $db_url;
  global $_mappingkit_logger;
  
  $_mappingkit_logger->debug('current working directory: ' . getcwd());
  
  if ($form_state['values']['geodata_type'] == GEODATA_IMPORT_POSTGIS) {
    
    $nid = (int)$form_state['values']['nid'];
    $node = node_load(array('nid' => $nid));
    
    //$url = $db_url[variable_get('geodata_database', DEFAULT_GEODATA_DATABASE)];
    $url = $db_url[$node->database_name];
    $geodata = parse_url($url);
    $geodata['database'] = substr($geodata['path'], 1);
    
    $_mappingkit_logger->debug('databases: ', array('db_url' => $db_url, 'geodata' => $geodata, 'values' => $form_state['values']));
    
    $table_name = $form_state['values']['data_path'];
    $shapefile = $form_state['values']['shapefile'];
    $shapepath = file_directory_temp() .'/'. $shapefile;
    // pgsql2shp
    $cmd = strtr('"%postgresql_bin/pgsql2shp%exe" -h %host %port_option -u %user -g %geom -f %shapefile %database %table'
      , array('%postgresql_bin' => variable_get('mappingkit_postgresql_bin_path', DEFAULT_MAPPINGKIT_POSTGRESQL_BIN_PATH)
        , '%shapefile' => $shapepath
        , '%table' => $table_name
        , '%host' => $geodata['host']
        , '%port_option' => isset($geodata['port']) ? '-p '. $geodata['port'] : ''
        , '%user' => $geodata['user']
        , '%database' => $geodata['database']
        , '%geom' => $node->geometry_column
        , '%exe' => MAPPINGKIT_EXE_TYPE
        )
      );
    
    $_mappingkit_logger->info($cmd);
    $results = mappingkit_proc($cmd);
    $_mappingkit_logger->debug('pgsql2shp results: ', $results);
    
    // zip
    $tempdir = file_directory_temp();
    $tempfile = 'mk_' . uniqid() .'.zip';
    $cmd = strtr(variable_get('mappingkit_zip_command', DEFAULT_MAPPINGKIT_ZIP_COMMAND)
      , array('%archive' => $tempfile
        , '%tempdir' => $tempdir
        )
      );
    $cmd .= " \"$shapefile.shp\" \"$shapefile.shx\" \"$shapefile.dbf\""; //\"
    
    $_mappingkit_logger->info($cmd);
    $results = mappingkit_proc($cmd, array('cwd' => file_directory_temp()));
    if ($results['return_value'] != 0) {
      // error
      $_mappingkit_logger->debug('zip results:', $results);
      
      form_set_error('', t('The archive could not be created. Please contact your system administrator.<br/>The command %cmd returned the following error output: %msg', array('%cmd' => $cmd, '%msg' => $results['stderr'])));
      return;
    }
    
    // remove shape files in $tempdir
    mk_remove_files(array("$tempdir/$shapefile.shp", "$tempdir/$shapefile.shx", "$tempdir/$shapefile.dbf"));
    
    // return link to zip file
    $export_web_path = mapclient_get_url(file_directory_temp() .'/'. $tempfile, TRUE);
    form_set_value($form['export_web_path'], $export_web_path, $form_state);
  }
  else {
    // shapefile
    
    //$folder = realpath(dirname($form_state['values']['data_path']));
    
    // todo: use caching for shapefile export?
    
    // check if zip exists for last part of shapefile's dirname
    $dir = dirname($form_state['values']['data_path']);
    $base = file_directory_path() .'/geodata/shapes/';
    $tempfile = NULL;
    if (($pos = strpos($dir, $base)) !== FALSE) {
      $uniqid = substr($dir, $pos + strlen($base));
      if (strlen($uniqid) == 13) {
        $tempfile = $uniqid .'.zip';
      }
    }
    //drupal_set_message('uniqid:'. var_export(array('dir'=>$dir,'base'=>$base,'uniqid'=>$uniqid,'tempfile'=>$tempfile), TRUE));
    if (is_null($tempfile)) {
      $tempfile = uniqid() .'.zip';
    }
    
    $tempdir = file_directory_temp();
    $export_web_path = mapclient_get_url($tempdir .'/'. $tempfile, TRUE);
    form_set_value($form['export_web_path'], $export_web_path, $form_state);
    //form_set_error('','error');
    //return;
    
    if (file_exists($tempdir .'/'. $tempfile)) {
      drupal_set_message('already exists');
    }
    else {
      // zip
      //$tempfile = uniqid() .'.zip';
      $cmd = strtr(variable_get('mappingkit_zip_command', DEFAULT_MAPPINGKIT_ZIP_COMMAND)
        , array('%archive' => $tempfile
          , '%tempdir' => $tempdir
          )
        );
      $cmd .= " {$folder}/*";
      
      $results = mappingkit_proc($cmd, array('cwd' => file_directory_temp()));
      drupal_set_message('results:'. var_export($results, TRUE));
    }
  }
  $_mappingkit_logger->debug('current working directory: ' . getcwd());
}

function geodata_export_submit($form, &$form_state) {
  drupal_set_message(t('The exported archive can be downloaded from this !link.'
    , array('!link' => l('link', $form_state['values']['export_web_path']))));
  
  drupal_goto('node/'. $form_state['values']['nid']);
  
  // Even though session_write_close() is registered as a shutdown function, we
  // need all session data written to the database before redirecting.
  /*
  session_write_close();
  
  header('Location: '. $form_state['values']['export_web_path'], TRUE, 302);
  
  // The "Location" header sends a redirect status code to the HTTP daemon. In
  // some cases this can be wrong, so we make sure none of the code below the
  // drupal_goto() call gets executed upon redirection.
  exit();
  */
}

