<?php

/**
 * @file
 * These functions can only be used when the php_dbase extension is loaded.
 */

/**
 * Describe dbase table (file).
 */
function dbase_describe_table($table_name) {
  $fields = array();
  $handle = dbase_open($table_name, 0);
  if (FALSE !== ($header = dbase_get_header_info($handle))) {
    // ok
    // fields: name, type (character, number), length, precision, format (string), offset
    foreach ($header as $field) {
      $fields[] = $field['name'];
    }
  }
  /*
  $rec = dbase_get_record_with_names($handle, 1);
  drupal_set_message('rec #1: '.var_export($rec, true));
  $recnum = 253; //$handle->header['# of Records']
  $rec = dbase_get_record_with_names($handle, $recnum);
  drupal_set_message("rec #{$recnum}: ".var_export($rec, true));
  */
  dbase_close($handle);
  return $fields;
}

if (!function_exists('dbase_open')) {
  
  /**
   * Dbase file class for functions in case the dbase php extension is not
   * available.
   */
  class dbf_class {
    
    var $handle;
    var $filename;
    var $header;
    var $fields;
    
    function __construct($filename = NULL) {
      if ($filename) {
        $this->open($filename, 0);
      }
    }
    
    function __destruct() {
      $this->close();
    }
    
    function open($filename, $mode = 2) {
      $this->handle = fopen($this->filename = $filename, $mode == 0 ? 'rb' : 'r+b');
      if ($this->handle) {
        $this->init();
      }
    }
    
    function close() {
      if ($this->handle) {
        $ret = fclose($this->handle);
        $this->handle = NULL;
      }
      return $ret;
    }
    
    /**
     * @return Nothing.
     */
    function init() {
      if (-1 == fseek($this->handle, 0)) {
        //echo 'seek failed';
        return;
      }
      if (($data = fread($this->handle, 32)) === false) {
        //echo 'read failed';
        return;
      }
      // Create the format for unpacking the header data
      $header_format = 
        'H2id/' .           # Grab two Big-Endian hex digits ('03')
        'Cyear/' .          # Grab an unsigned bit
        'Cmonth/' .         # Grab an unsigned bit
        'Cday/' .           # Grab an unsigned bit
        'L# of Records/' .  # Grab an unsigned long (32 bit number)
        'SHeader Size/' .   # Grab an unsigned short (16 bit number)
        'SRecord Size';     # Grab an unsigned short (16 bit number)
      // unpack the header
      $this->header = unpack ($header_format, $data);
      $this->header['year'] += 1900;
      
      // Make sure that the file is the right size
      if (filesize($this->filename) < ($size = $this->header['Header Size'] + ($this->header['Record Size'] * $this->header['# of Records']))) {
        //drupal_set_message(t('File %file is not a valid dbf file.', array('%file' => $this->filename)), 'error');
        //echo 'filesize:'.filesize($this->filename);
        return false;
      }
      
      // Get the rest of the dbf file header
      $data = fread($this->handle, $this->header['Header Size'] - 34);
      // Create the format for unpacking the data that describes the format of the 
      // records in the file
      $record_format = 
        'a11name/' .  # Grab 11 alphanumeric characters
        'Atype/' .    # Grab a single alphanumeric character
        'x4/' .       # Skip 4 bytes forward
        'Clength/' .  # Grab an unsigned bit
        'Cprecision'; # Grab an unsigned bit
  
      $this->fields = array();
      $type_names = array('C' => 'character', 'D' => 'date', 'F' => 'float', 'I' => 'integer', 'L' => 'boolean', 'M' => 'memo', 'N' => 'number');
      for ($offset = 0; $offset < strlen ($data); $offset += 32) {
        // C (stands for character), D (date) , I (integer), N (number), L (boolean), M (memo). All other symbols like B, @, l, +, F, O and G are defined as unknown.
        $field = unpack("@$offset/$record_format", $data);
        switch ($field['type']) {
        case 'L':
          $field['format'] = 'A1';
          break;
        case 'D':
        case 'C':
        case 'N':
        default:
          $field['format'] = "A{$field['length']}";
          break;
        }
        $field['type'] = $type_names[ $field['type'] ];
        $this->fields[] = $field;
      }
    }
    
    /**
     * Seek start of record.
     * @param $record_number >= 1
     * @return TRUE or FALSE
     */
    function seek_record($record_number) {
      return 0 == fseek($this->handle, $this->header['Header Size'] + $this->header['Record Size'] * ($record_number - 1));
    }
    
    /**
     * Write delete flag at current position.
     * @return TRUE of FALSE
     */
    function flag_delete($delete = TRUE) {
      return fwrite($this->handle, $delete ? '*' : ' ', 1) !== false;
    }
    
    /**
     * Write record at current position.
     */
    function write_record($record) {
      $record_format = 'A1';
      $field_data = array(' '); // deleted flag
      $num_fields = count($this->fields);
      for ($i = 0; $i < $num_fields; $i++) {
        $field = $this->fields[$i];
        $record_format .= $field['format'];
        
        // check $record values
        switch ($field['type']) {
        case 'number':
          $value = sprintf("%{$field['length']}.{$field['precision']}f", (float)$record[$i]);
          break;
        case 'boolean':
          // check content of string or cast to boolean
          if (is_string($record[$i]) ? strpos('yYtT', substr($record[$i],0,1)) !== false : (bool)$record[$i]) {
            $value = 'T';
          }
          else {
            $value = 'F';
          }
          break;
        default:
          $value = $record[$i];
          break;
        }
        $field_data[] = pack($field['format'], $value);
      }
      
      // call pack with record_format and $record
      array_unshift($field_data, $record_format);
      //echo $record_format."\n";
      //echo var_export($field_data, true)."\n";
      $record_data = call_user_func_array('pack', $field_data);
      
      fwrite($this->handle, $record_data);
    }
  }
  
  /**
   * @param $mode Open mode: 0 = read, 1 = write, 2 = read/write.
   * @return FALSE in case of an error.
   */
  function dbase_open($filename, $mode) {
    // Ensure that the file is big enough to be a db file
    if (($filesize = filesize ($filename)) <= 68) {
      //drupal_set_message(t("File %file is not large enough to be a dbf file.", array('%file' => $filename)), 'error');
      return FALSE;
    }
    
    $dbf = new dbf_class($filename);
    if ($dbf->handle) {
      return $dbf;
    }
    else {
      return FALSE;
    }
  }
  
  /**
   * @return Returns TRUE on success or FALSE on failure.
   */
  function dbase_close($dbf) {
    return ($dbf) ? $dbf->close() : FALSE;
  }
  
  /**
   * Difference with the php extension function is that a) format is not the 
   * suggested printf format but the unpack format, b) offset is missing.
   * @return An indexed array with an entry for each column in the database. The 
   * array index starts at 0.
   */
  function dbase_get_header_info($dbf) {
    // see http://www.clicketyclick.dk/databases/xbase/format/dbf.html#DBF_STRUCT
    // and http://www.phpdig.net/ref/rn45re878.html
    return $dbf->fields;
  }
  
  /**
   * @return The number of records in the database, or FALSE if an error occurs.
   */
  function dbase_numrecords($dbf) {
    return $dbf->header['# of Records'];
  }
  
  /**
   * dbase_numfields — Gets the number of fields of a database
   * @return The number of fields in the database, or FALSE if an error occurs.
   */
  function dbase_numfields($dbf) {
    return count($dbf->fields);
  }
  
  /**
   * Difference with the dbase function is that this function trims all 
   * character fields and returns deleted as 0/1.
   * @param $record_number Record number >= 1.
   * @return Associative array of fields and values. In addition a deleted flag.
   */
  function dbase_get_record_with_names($dbf, $record_number) {
    
    if ($record_number < 1 || $record_number > $dbf->header['# of Records']) {
      return FALSE;
    }
    
    $record_format = 'Cdeleted';
    foreach ($dbf->fields as $field) {
      $record_format .= '/'. $field['format'] . $field['name'];
    }
    
    fseek($dbf->handle, $dbf->header['Header Size'] + $dbf->header['Record Size'] * ($record_number - 1));
    $result = unpack($record_format, fread($dbf->handle, $dbf->header['Record Size']));
    
    foreach ($dbf->fields as $field) {
      $value = $result[ $field['name'] ];
      switch ($field['type']) {
      case 'number':
        $result[ $field['name'] ] = (float) $value;
        break;
      case 'boolean': // true for YTty
        $result[ $field['name'] ] = strpos('YyTt', $value) !== FALSE ? TRUE : (strpos('NnFf', $value) !== FALSE ? FALSE : NULL);
        break;
      }
    }
    // deleted flag = '*' (#2A) or ' ' (#20)
    $result['deleted'] = $result['deleted'] == 32 ? FALSE : TRUE;
    
    return $result;
  }
  
  /**
   * @return todo: determine if an open dbf_class is returned or not
   */
  function dbase_create($filename, $def) {
    // header format
    $num_fields = count($def);
    // get date
    
    $rec_size = 1; // deleted flag
    $hdr_size = 32;
    foreach ($def as &$field) {
      $dec = 0;
      switch ($field[1]) {
      case 'D':
        $len = 8;
        break;
      case 'C':
        $len = $field[2];
        break;
      case 'N':
        $len = $field[2];
        $dec = isset($field[3]) ? $field[3] : 0;
        break;
      case 'L':
        $len = 1;
        break;
      default:
        trigger_error('not supported');
        break;
      }
      $field['data'] = pack('a10xAV1C2@32'
        , strtoupper($field[0]) // convert field name to uppercase
        , $field[1]
        , 0
        , $len
        , $dec
        );
      $rec_size += $len;
      $hdr_size += 32;
    }
    $hdr_size += 1;
    
    $date = localtime(time(), true);
    //print(var_export($date, true)."\n");
    
    $header = pack('hC3Vv2@32'
      , 3 // id
      , $date['tm_year'] // year since 1900
      , $date['tm_mon'] + 1 // month >= 0
      , $date['tm_mday'] // day of the month > 0
      , 0 // # of Records
      , $hdr_size // header size
      , $rec_size // record size
      );
    
    $handle = fopen($filename, 'w+');
    // write file header
    fwrite($handle, $header);
    
    //print(var_export($def, true)."\n");
    
    // write field defs
    foreach ($def as &$field) {
      //print(var_export($field['data'], true)."\n");
      fwrite($handle, $field['data']);
    }
    //print(var_export($def, true)."\n");
    
    // closing byte for header
    fwrite($handle, "\r", 1);
    
    $dbf = new dbf_class();
    $dbf->handle = $handle;
    $dbf->filename = $filename;
    $dbf->init();
    return $dbf;
  }
  
  /**
   * Adds a record to a database
   * @param $record
   *   An indexed array of data. The number of items must be equal to the 
   *   number of fields in the database, otherwise dbase_add_record() will fail.
   */
  function dbase_add_record($dbf, $record) {
    if (!$dbf) {
      // not a dbf_class
      //echo 'not a dbf';
      return false;
    }
    
    if (count($record) != count($dbf->fields)) {
      // wrong number of fields
      //echo 'wrong # of fields:' . count($dbf->fields);
      return false;
    }
    
    // seek start position of record
    $dbf->seek_record($dbf->header['# of Records'] + 1);
    
    // write record
    $dbf->write_record($record);
    fwrite($dbf->handle, chr(26), 1); // closing ^Z (#1A)
    
    // increment # of records in header
    fseek($dbf->handle, 4);
    $dbf->header['# of Records'] += 1;
    fwrite($dbf->handle, pack('V1', $dbf->header['# of Records']));
    
    return true;
  }
  
  /**
   * Replaces the given record in the database with the given data.
   * @param $record_number
   *   An integer which spans from 1 to the number of records in the database
   *   (as returned by dbase_numrecords()).
   * @return Returns TRUE on success or FALSE on failure.
   */
  function dbase_replace_record($dbf, $record, $record_number) {
    
    return ($dbf) && $dbf->seek_record($record_number) && $dbf->write_record($record);
  }
   
  /**
   * @return Returns TRUE on success or FALSE on failure.
   */
  function dbase_delete_record($dbf, $record_number) {
    return ($dbf) && $dbf->seek_record($record_number) && $dbf->flag_delete();
  }
}

