<?php

include_once('fpdf/fpdf.php');

class Map2PDF extends FPDF {
  public $lastError;
  public $title;
  public $encoding = 'ISO-8859-1';
  public $image;
  public $px;
  public $py;
  public $extent;
  
  function __construct($title = 'Title', $options = array()) {
    
    // format: A3, A4, A5, letter, legal (add size A0, A1, A2 to fpdf
    // by specifying format as wxh)
    // a3 = 297 x 420, a2 = 420 x 594, a1 = 594 x 841, a0 = 841 x 1189
    // a4 = 210 x 297, a5 = 148 x 210
    // orientation: P, L
    $orientation = isset($options['orientation']) ? $options['orientation'] : 'P';
    $unit = isset($options['unit']) ? $options['unit'] : 'mm';
    $format = isset($options['format']) ? $options['format'] : 'A4';
    parent::__construct($orientation, $unit, $format);
    
    $this->SetTitle($title);
  }
  
  function SetTitle($title) {
    // convert to encoding used by pdf
    $this->title = mb_convert_encoding($title, $this->encoding);
  }
  
  function PrepareImage($w, $h, $extent) {
    $this->extent = $extent;
    $this->image = imagecreatetruecolor($this->width = $w, $this->height = $h);
    $bg = imagecolorallocate($this->image, 255, 255, 255);
    imagefilledrectangle($this->image, 0, 0, $this->width, $this->height, $bg);
    imagealphablending($this->image, true);
  }
  
  //Page header
  function Header() {
    //Logo
    //$this->Image('logo_pb.png',10,8,33);
    //Arial bold 15
    $this->SetFont('Arial', 'B', 15);
    //Move to the right
    $this->Cell(80);
    //Title
    $this->Cell(30, 10, $this->title, 0, 0, 'C');
    //Line break
    $this->Ln(20);
  }
  
  //Page footer
  function Footer() {
    //Position at 1.5 cm from bottom
    $this->SetY(-15);
    //Arial italic 8
    $this->SetFont('Arial', 'I', 8);
    //Page number
    $this->Cell(0, 10, t('Page') .' '. $this->PageNo(), 0, 0, 'C');
  }
  
  function Error($msg) {
    $this->lastError = $msg;
  }
  
  function getLastError() {
    return $this->lastError;
  }
}

