<?php

include_once('map2pdf.class.inc');

/**
 * Add image from url.
 *
 * @param $url
 *   Web path to image.
 * @return
 *   The image object or NULL on failure.
 */
function map2pdf_add_image($url = NULL) {
  static $image = NULL, $width, $height;
  
  if (!$url) {
    return $image;
  }
  
  $im = @imagecreatefrompng($url);
  if (!$im) {
    // failure
    return NULL;
  }
  
  if (!$image) {
    // create image
    $image = imagecreatetruecolor($width = imagesx($im), $height = imagesy($im));
    $bg = imagecolorallocate($image, 255, 255, 255);
    imagefilledrectangle($image, 0, 0, $width, $height, $bg);
    imagealphablending($image, true);
  }
  imagecopy($image, $im, 0, 0, 0, 0, $width, $height);
  return $image;
}

/**
 * Print the map context that is received in the postBody parameter to pdf.
 */
function map2pdf_print_file($xml = NULL) {
  
  $result = array();
  
  if (! file_exists('./' . file_directory_temp())) {
    drupal_json("The temporary directory of this site is not accessible.");
    return;
  }
  
  $temp_webpath = base_path() . file_directory_temp() .'/';
  
  if (empty($xml)) {
    if (!isset($_POST['postBody'])) {
      // no xml
      $result['errors'][] = t('Parameters are missing.');
    }
    $xml = $_POST['postBody'];
  }
  
  //ini_set('max_execution_time', 300); // todo: check present value
  $output = '';
  
  // load wmc parser to parse context
  module_load_include('inc', 'mappingkit', 'mapcontext/wmc.parser');
  
  $wmc = new WMC_Parser();
  $context = $wmc->load($xml);

  $image_urls = array();
  if (empty($context['width'])) {
    //  not a valid context
    $result['errors'][] = t('Invalid web map context.');
    return $result;
  }
  else {
    // double resolution for print! Note that this may return more
    // detailed images.
    $context['width'] *= 2;
    $context['height'] *= 2;
    
    $pdf = new Map2PDF($context['Title'], array(
      'orientation' => variable_get('map2pdf_paper_orientation', DEFAULT_MAP2PDF_PAPER_ORIENTATION)
      , 'format' => variable_get('map2pdf_paper_format', DEFAULT_MAP2PDF_PAPER_FORMAT))
      );
    $pdf->PrepareImage($context['width'], $context['height'], $context['bbox']);
    
    // build wms url's for layers
    foreach ($context['layers'] as $layer) {
      // $context['bbox'] intersects $layer['extension']['maxExtent'] ?
      
      if ($layer['options']['visibility']) {
        // only print visible layers
        
        // layer params
        $params = $layer['params'];
        // other params
        $params['request'] = 'GetMap';
        $params['width'] = $context['width'];
        $params['height'] = $context['height'];
        $params['layers'] = $layer['Name'];
        $params['bbox'] = implode(',', array_values($context['bbox']));
        $params['srs'] = $context['srs'];
        
        // default params
        $params['transparent'] = 'TRUE';
        $params['service'] = 'WMS';
        
        // build url
        if (strpos($layer['href'], '?') === FALSE) {
          $layer['href'] .= '?';
        }
        $image_url = $layer['href'] .'&'. drupal_query_string_encode($params);
        
        $image_urls[] = $image_url;
      }
    }
  }
  
  if (false) {
    $output_path = mappingkit_tempnam('.txt');
    
    $output .= var_export(array('context' => $context, 'image_urls' => $image_urls), true);
    
    foreach ($image_urls as $image_url) {
      $image_response = drupal_http_request($image_url);
      
      $content_type = explode('/', $image_response->headers['Content-Type']);
      $image_path = mappingkit_tempnam('.'. $content_type[1]);
      
      unset($image_response->data);
      $output .= var_export(array('image path' => $image_path, 'response' => $image_response), true);
    }
    
    $handle = fopen($output_path, "w");
    fwrite($handle, $output);
    fclose($handle);
  }
  else {
    
    foreach ($image_urls as $image_url) {
      map2pdf_add_image($image_url);
    }
    $error = NULL;
    $image = map2pdf_add_image();
    if ($image) {
      $image_path = mappingkit_tempnam('.png');
      imagepng($image, $image_path);
      
      $width = 210 - 40;
      //$px = $width/25.4 * 72;
      $height = $width * $context['height']/$context['width'];
      
      $pdf->SetTitle($context['Title']);
      $pdf->SetCreator('Mapping kit for Drupal');
      $pdf->AddPage();
      //$pdf->Rect(20-1, 20-1, $width+2, $height+2);
      $pdf->Image($image_path, 20, 20, $width, $height);
      
      // images in cell?
      //$this->Cell($w[4],6,$this->Image('img/lp_ikona_up.png',$this->GetX(),$this->GetY()),'LR',0,'R');
      
      $pdf->Rect(20, 20, $width, $height);
    }
    else if (count($image_urls) == 0) {
      $result['errors'][] = t('No WMS layers available');
    }
    else {
      $result['errors'][] = t('No image available');
    }
    
    if ($error || $error = $pdf->getLastError()) {
      $output_path = mappingkit_tempnam('.txt');
      
      $handle = fopen($output_path, "w");
      fwrite($handle, $error);
      fclose($handle);
    }
    else {
      $output_path = mappingkit_tempnam('.pdf');
      try {
        $pdf->Output($output_path);
        
        // set file mode (linux)
        $mode = variable_get('mappingkit_chmod_files', DEFAULT_MAPPINGKIT_CHMOD_FILES);
        if (!empty($mode)) {
          chmod($output_path, intval($mode, 8));
        }
      }
      catch(Exception $e) {
      }
    }
  }
  $result['fileUrl'] = $temp_webpath . basename($output_path);
  return $result;
  //drupal_json(array('fileUrl' => $temp_webpath . basename($output_path)));
}

/**
 * Print the map context that is received in the postBody parameter to pdf.
 */
function map2pdf_print($xml = NULL) {
  $retval = map2pdf_print_file($xml);
  drupal_json($retval);
}

