<?php

function map_assist_popup() {
  $output = '';
  
  drupal_add_js(drupal_get_path('module', 'map_assist') .'/map_assist.js');
  
  $output .= drupal_get_form('map_assist_form');
  
  echo theme('popup', $output);
}

function map_assist_form(&$form_state) {
  $form = array();
  
  $form['context'] = array('#type' => 'fieldset'
    , '#title' => t('Map contexts')
    , '#collapsible' => TRUE
    );
  
  //drupal_set_message(var_export(array($form_state), true));
  
  $result = db_query("SELECT DISTINCT nid, title FROM {node} WHERE type = 'mapcontext' ORDER BY title");
  $contexts = array('' => t('No map context'));
  while ($rec = db_fetch_object($result)) {
    $contexts['node/'. $rec->nid .'/context'] = $rec->title .' ('. $rec->nid .')';
  }
  
  $form['context']['contexts'] = array('#type' => 'select'
    , '#title' => t('Map context')
    , '#options' => $contexts
    , '#default_value' => ''
    );
  
  $form['size'] = array('#type' => 'fieldset'
    , '#title' => t('Size')
    , '#collapsible' => TRUE
    , '#collapsed' => TRUE
    , '#description' => t('Size of map window. Leave fields empty to use a default size.')
    );
  $form['size']['width'] = array('#type' => 'textfield'
    , '#title' => t('Width')
    , '#default_value' => ''
    , '#length' => 5
    , '#size' => 10
    , '#prefix' => '<div class="form-inline">'
    );
  $form['size']['height'] = array('#type' => 'textfield'
    , '#title' => t('Height')
    , '#default_value' => ''
    , '#length' => 5
    , '#size' => 10
    , '#suffix' => '</div>'
    );
  
  $form['extent'] = array('#type' => 'fieldset'
    , '#title' => t('Extent')
    , '#collapsible' => TRUE
    , '#collapsed' => TRUE
    , '#theme' => 'extent'
    , '#description' => t('Extent of map.')
    );
  mappingkit_add_extent_fields($form['extent']
    , array('minx' => '', 'miny' => '', 'maxx' => '', 'maxy' => ''));
  
  $form['baselayers'] = array('#type' => 'fieldset'
    , '#title' => t('Base layers')
    , '#collapsible' => TRUE
    , '#collapsed' => TRUE
    );
  $form['baselayers']['base_layers'] = array('#type' => 'checkboxes'
    , '#title' => t('Standard base layers')
    , '#options' => array('google' => 'Google'
      , 've' => 'Virtual Earth'
      , 'yahoo' => 'Yahoo'
      , 'osm' => 'OpenStreetMap'
      , 'oam' => 'OpenAerialMap'
      )
    , '#default_value' => array()
    , '#description' => t('Select the base layers of the map. Note that using a standard base layer will set the projection to spherical mercator.')
    );
  
  $form['layers'] = array('#type' => 'fieldset'
    , '#title' => t('Layers')
    , '#collapsible' => TRUE
    , '#collapsed' => TRUE
    );
  $form['layers']['type'] = array('#type' => 'select'
    , '#title' => t('Type')
    , '#options' => array('' => t('Select type'), 'georss' => 'GeoRSS', 'gpx' => 'GPX', 'image' => 'Image', 'kml' => 'KML', 'wfs' => 'WFS', 'wms' => 'WMS')
    , '#default_value' => ''
    , '#required' => TRUE
    );
  $form['layers']['url'] = array('#type' => 'textfield'
    , '#title' => t('URL')
    , '#default_value' => ''
    , '#size' => 50
    , '#maxlength' => 255
    , '#required' => TRUE
    , '#description' => t('URL to local (relative path) or remote (absolute path) data source, e.g. this %rss or %feed.', array('%rss' => url('rss.xml'), '%feed' => url('taxonomy/term/1')))
    );
  $form['layers']['title'] = array('#type' => 'textfield'
    , '#title' => t('Title')
    , '#default_value' => ''
    , '#size' => 50
    , '#description' => t('Title of layer in layer swicher control.')
    );
  $form['layers']['add'] = array('#type' => 'button'
    , '#value' => t('Add')
    , '#attributes' => array('onclick' => "MappingKit.addLayer(); return false;")
    );
  $form['layers']['layers'] = array('#type' => 'textarea'
    , '#title' => t('Layers')
    , '#default_value' => ''
    , '#rows' => 3
    );
  
  $form['controls'] = array('#type' => 'fieldset'
    , '#title' => t('Controls')
    , '#collapsible' => TRUE
    , '#collapsed' => TRUE
    );
  // none, toolbar, statusbar, layerswitcher, scalebar
  $form['controls']['controls'] = array('#type' => 'checkboxes'
    , '#title' => t('Available controls')
    , '#options' => array('' => 'Default'
      ,'none' => 'None'
      , 'toolbar' => 'Toolbar'
      , 'statusbar' => 'Statusbar'
      , 'layerswitcher' => 'LayerSwitcher'
      , 'scalebar' => 'Scalebar'
      )
    , '#default_value' => array()
    , '#description' => t('Select the controls that will be available.')
    );
  
  $form['output'] = array('#type' => 'fieldset'
    , '#title' => t('Output')
    , '#collapsible' => TRUE
    );
  $form['output']['refresh'] = array('#type' => 'button'
    , '#value' => t('Refresh')
    , '#attributes' => array('onclick' => "MappingKit.buildMapTag('[', ']'); return false;")
    );
  
  $form['output']['maptag'] = array('#type' => 'textarea'
    , '#title' => t('Map tag')
    , '#default_value' => mapclient_build_map_tag('')
    , '#rows' => 3
    );
  
  $form['ok'] = array('#type' => 'submit'
    , '#value' => t('Ok')
    , '#attributes' => array('onclick' => "MappingKit.buildMapTag('[', ']'); window.opener.document.getElementById('edit-body').value += document.getElementById('edit-maptag').value; window.close();")
    );
  $form['cancel'] = array('#type' => 'submit'
    , '#value' => t('Cancel')
    , '#attributes' => array('onclick' => 'window.close();')
    );
  
  $form['#validate'][] = 'map_assist_form_validate';
  $form['#submit'][] = 'map_assist_form_submit';
  
  return $form;
}

function map_assist_form_validate(&$form, &$form_state) {
  if ($form_state['values']['op'] == $form_state['values']['refresh']) {
    if ($form_state['values']['use_context'] == 1) {
      form_set_value($form['output']['maptag'], mapclient_build_map_tag('context="'. $form_state['values']['contexts'] .'"'), $form_state);
      drupal_set_message(htmlentities(var_export($form['output']['maptag'], true)));
    }
  }
  //drupal_set_message('validate '. var_export($form_state['values'], true));
}

function map_assist_form_submit(&$form, &$form_state) {
  drupal_set_message('submit '. var_export($form_state, true));
  unset($_SESSION['layers']);
}

/**
 * theme_popup is derived from theme_page, but adjusted to show a
 * popup without menu/breadcrumb/blocks/footer.
 */
function theme_popup($content) {
    //$output = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
    $output = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
    $output .= '<html xmlns="http://www.w3.org/1999/xhtml">';
    $output .= '<head>';
    $output .= '<title>'. (drupal_get_title() ? strip_tags(drupal_get_title()) : variable_get('site_name', 'drupal')) .'</title>';
    $output .= drupal_get_html_head();
    $output .= drupal_get_css();
    $output .= drupal_get_js();
    
    $output .= '</head>';
    $output .= '<body style="background-color: #fff; color: #000; padding: 4px;">';
//    $output .= '<table border="0" cellspacing="4" cellpadding="4"><tr><td style="vertical-align: top; width: 170px;">';
    
//    $output .= theme('blocks', 'all');
//    $output .= '</td><td style="vertical-align: top;">';
    
//    $output .= theme('breadcrumb', drupal_get_breadcrumb());
//    $output .= '<h1>' . drupal_get_title() . '</h1>';
    
//    if ($tabs = theme('menu_local_tasks')) {
//        $output .= $tabs;
//    }
    
    $output .= theme('help');
    
    $output .= theme('status_messages');
    
    $output .= "\n<!-- begin content -->\n";
    $output .= $content;
    $output .= "\n<!-- end content -->\n";
    
    //$output .= '</td></tr></table>';
    $output .= theme('closure');
    $output .= '</body></html>';
    
    return $output;
}

