<?php
/**
 * @file
 * Administer mapclient module
 */

/**
 * Generate form for map client settings.
 * @return Form.
 */
function mapclient_settings_form() {
  
  $mapclients = module_invoke_all('mapclient');
  $options = array();
  foreach ($mapclients as $client => $info) {
    if ($client == DEFAULT_MAPCLIENT) {
      $info['name'] .= ' (default)';
    }
    $options[$client] = $info['name'];
  }
  //drupal_set_message(var_export($mapclients,TRUE));
  
  $form['mapclient'] = array(
    '#type' => 'fieldset'
    , '#title' => t('Map client')
    , '#collapsible' => TRUE
    );
  $form['mapclient']['mapclient'] = array(
    '#type' => 'select'
    , '#title' => t('Map client')
    , '#options' => $options
    , '#default_value' => variable_get('mapclient', DEFAULT_MAPCLIENT)
    );
  $form['mapclient']['mapclient_load_errors'] = array(
    '#type' => 'checkbox'
    , '#title' => t('Show image load errors')
    , '#default_value' => variable_get('mapclient_load_errors', DEFAULT_MAPCLIENT_LOAD_ERRORS)
    );
  $form['mapclient']['mapclient_proxy'] = array('#type' => 'textfield'
    , '#title' => t('Proxy')
    , '#description' => t('Relative path to proxy script. This script is used to fetch feature info, legends, etc.')
    , '#default_value' => variable_get('mapclient_proxy', DEFAULT_MAPCLIENT_PROXY)
    );
  $form['mapclient']['mapclient_baselayers_separator'] = array('#type' => 'textfield'
    , '#title' => t('Baselayers separator')
    , '#description' => t('Separator for baselayers attribute.')
    , '#default_value' => variable_get('mapclient_baselayers_separator', ',')
    );
  $form['mapclient']['mapclient_baselayer_options_separator'] = array('#type' => 'textfield'
    , '#title' => t('Baselayer options separator')
    , '#description' => t('Separator for baselayer options. Must be different from the baselayers separator!')
    , '#default_value' => variable_get('mapclient_baselayer_options_separator', '|')
    );
  $form['mapclient']['mapclient_layers_separator'] = array('#type' => 'textfield'
    , '#title' => t('Layers separator')
    , '#description' => t('Separator for layers attribute.')
    , '#default_value' => variable_get('mapclient_layers_separator', ',')
    );
  
  $form['google'] = array('#type' => 'fieldset'
    , '#title' => t('Google Maps')
    , '#collapsible' => TRUE
    , '#description' => t('Google Maps can be used as a background layer.')
    );
  $form['google']['google_maps_key'] = array('#type' => 'textfield'
    , '#title' => t('Google Maps Key')
    , '#size' => 100
    , '#default_value' => variable_get('google_maps_key', DEFAULT_GOOGLE_MAPS_KEY)
    , '#description' => t('Google Maps key for your domain. The default key is valid for localhost.')
    );
  
  $form['#validate'][] = 'mapclient_settings_form_validate';
  //$form['#submit'][] = 'mapclient_settings_form_submit';
  
  return system_settings_form($form);
}

function mapclient_settings_form_validate(&$form, &$form_state) {
  
  if ($form_values['op'] == $form_values['submit']) {
    // check that proxy file exists
    $form_values = $form_state['values'];
    
    $file_path = realpath($form_values['mapclient_proxy']);
    $web_path = mapclient_get_url($form_values['mapclient_proxy'], TRUE);
    // 1) check file path to proxy
    if (!file_exists($file_path)) {
      // file does not exist
      form_set_error('mapclient_proxy', t("A proxy script %path cannot be found", array('%path' => $form_values['mapclient_proxy'])));
    }
    else {
      drupal_set_message(t('The proxy script was found at %file_path (file permissions %perms)', array('%file_path' => $file_path, '%perms' => substr(decoct(fileperms($file_path)), -3))));
      
      // 2) check valid parameters (GET) itself
      global $base_url;
      $url = $base_url .'/update.php';
      $response = drupal_http_request($web_path .'?url='. urlencode($url));
      if ($response->code != 200) {
        drupal_set_message(t('The proxy script returned %code (%error) when fetching %url', array('%code' => $response->code, '%error' => $response->error, '%url' => $url)), 'warning');
      }
      else {
        drupal_set_message(t('The proxy script successfully fetched information from %url', array('%url' => $url)));
      }
      
      if (parse_url($url, PHP_URL_HOST) != 'localhost') {
        // 3) check that get request to localhost is secured
        $url = 'http://localhost/';
        $response = drupal_http_request($web_path .'?url='. urlencode($url));
        if ($response->code == 200) {
          drupal_set_message(t("The proxy script can fetch information from %url. You may want to secure this proxy script by setting the \$forbidden_targets variable to /^localhost\\\$/ or limit the \$permitted_targets to whatever you need.", array('%url' => $url)), 'warning');
        }
        else {
          drupal_set_message(t("The proxy script cannot fetch from %url: %code (%error)", array('%code' => $response->code, '%error' => $response->error, '%url' => $url)));
        }
      }
      
      // todo: 4) check post request?
      
      if ($form_values['mapclient_proxy'] == DEFAULT_MAPCLIENT_PROXY) {
        // include mapping kit proxy script and return parameters?
        $proxy_inclusion = TRUE;
        $cwd = getcwd();
        chdir(drupal_get_path('module', 'mappingkit'));
        
        // call proxy
        include_once('proxy.php');
  
        // reset current working dir
        chdir($cwd);
        
        drupal_set_message(t("A proxy script can be secured by limiting target hosts and/or by limiting the content types that can be returned. Current settings in this proxy file are:<br/>permitted target hosts = %permitted_targets<br/>forbidden target hosts = %forbidden_targets<br/>permitted content types = %permitted_content_types"
          , array('%permitted_targets' => empty($proxy_settings['permitted_hosts_regex']) ? '(not used)' : $proxy_settings['permitted_hosts_regex']
            , '%permitted_urls' => empty($proxy_settings['permitted_urls']) ? '(not used)' : $proxy_settings['permitted_urls']
            , '%forbidden_targets' => empty($proxy_settings['forbidden_hosts_regex']) ? '(not used)' : $proxy_settings['forbidden_hosts_regex']
            , '%permitted_content_types' => $proxy_settings['permitted_content_types_regex']
            )));
      }
    }
  }
}

