<?php
/**
 * @file
 * Internal mapclient.
 */

/**
 * Initialize header.
 * This mapclient function is used to add javascript files or html headers.
 */
function mapclient_init_header() {
  // Include javascript for the internal map client
  //$scope = 'footer';
  //drupal_add_js(drupal_get_path('module', 'mapclient') .'/mapclient.js', 'module', $scope);
  
  // add css for internal map client
  //drupal_add_css(drupal_get_path('module', 'mapclient') .'/mapclient_client.css');
}

/**
 * Render body of map.
 */
function mapclient_render_body(&$attrs) {
  global $_mappingkit_logger;
  
  $output = '';
  
  /*
  // open div
  $output .= '<div id="' . $attrs['id'] . '" class="map-block ' . $attrs['class'] . '"';
  if (!empty($attrs['style'])) {
    $output .= ' style="' . $attrs['style'] . '"';
  }
  $output .= '>';
  */
  // open outer block
  $output .= mapclient_common_open_outer_block($attrs);
  
  // open map block
  $output .= mapclient_common_open_map_block($attrs, array('class' => 'mapclient'));

  // generate output image
  $width = $attrs['width'];
  $height = $attrs['height'];
  $image = imagecreatetruecolor($width, $height);
  $bg = imagecolorallocate($image, 255, 255, 255);
  imagefilledrectangle($image, 0, 0, $width, $height, $bg);
  imagealphablending($image, TRUE);
  
  if (isset($attrs['extent'])) {
    $bbox = $attrs['extent'];
  }
  else {
    $bbox = $attrs['maxextent'];
  }
  
  $bboxa = explode(',', $bbox);
  syncWorldBbox($bboxa, $width, $height);
  $bbox = implode(',', $bboxa);
  
  $srs = NULL;
  if (isset($attrs['contexts'])) {
    $contexts = explode(',', $attrs['contexts']);
    foreach ($contexts as $context) {
      // Get images for wms layers
      if (preg_match('/node\/(?P<nid>\d+)(\/revisions\/(?P<vid>\d+))?/', $context, $matches)) {
        // is a mapcontext node?
        $node = node_load($matches['nid'], isset($matches['vid']) ? $matches['vid'] : NULL);
        if ($node->type == 'mapcontext') {
          //drupal_set_message(var_export($node, TRUE));
          
          if (!$bbox) {
            $bboxa = array($node->minx, $node->miny, $node->maxx, $node->maxy);
            syncWorldBbox($bboxa, $width, $height);
            $bbox = implode(',', $bboxa);
          }
          $srs = 'EPSG:' . $node->srid;
          
          foreach ($node->layers as $layer) {
            // if hidden == 0
            $params = array('width' => $width
              , 'height' => $height
              , 'layers' => $layer['names']
              , 'version' => $layer['version']
              , 'bbox' => $bbox
              , 'srs' => $srs
              , 'transparent' => 'TRUE'
              , 'service' => 'WMS'
              , 'format' => 'image/png'
              , 'request' => 'GetMap'
              , 'styles' => ''
              );
            $url = $layer['href'];
            if (strpos($url, '?') === FALSE) {
              $url .= '?';
            }
            $url .= '&'. drupal_query_string_encode($params);
            //drupal_set_message($url);
            if ($layer_image = @imagecreatefrompng($url)) {
              imagecopy($image, $layer_image, 0, 0, 0, 0, $width, $height);
            }
          }
        }
      }
    }
  }
  
  if (empty($attrs['projection'])) {
    $attrs['projection'] = is_null($srs) ? 'EPSG:4326' : $srs;
  }
  
  if (isset($attrs['layers'])) {
    $layers = explode(',', $attrs['layers']);
    foreach ($layers as $id) {
      $layer = explode('|', $attrs[$id]);
      switch ($layer[0]) {
      case 'wms':
        // 1=url, 2=title, 3=layers, 4=options
        // show only visible layers
        $params = array('request' => 'GetMap'
          , 'width' => $width
          , 'height' => $height
          , 'bbox' => $bbox
          , 'layers' => $layer[3]
          , 'srs' => $attrs['projection']
          , 'transparent' => 'TRUE'
          , 'service' => 'WMS'
          , 'version' => '1.1.1'
          , 'format' => 'image/png'
          , 'styles' => ''
          );
        $url = $layer[1];
        if (strpos($url, '?') === FALSE) {
          $url .= '?';
        }
        $url .= '&'. drupal_query_string_encode($params);
        $_mappingkit_logger->info($url);
        if ($layer_image = @imagecreatefrompng($url)) {
          imagecopy($image, $layer_image, 0, 0, 0, 0, $width, $height);
        }
        else {
          $_mappingkit_logger->info('Error creating image: ' . $url);
        }
        break;
        
      case 'image':
        $url = $layer[1];
        $name = basename(parse_url($url, PHP_URL_PATH));
        $type = substr($name, strrpos($name, '.') + 1);
        $function = 'imagecreatefrom' . $type;
        if (function_exists($function)) {
          if ($layer_image = @$function($url)) {
            imagecopyresized($image, $layer_image, 0, 0, 0, 0, $width, $height, imagesx($layer_image), imagesy($layer_image));
          }
        }
        break;
      }
    }
  }
  
  $image_path = mappingkit_tempnam('.png');
  imagepng($image, $image_path);
  $output .= '<img src="' . mapclient_get_url(file_directory_temp() . '/'. basename($image_path), TRUE) . '"/>';
  
  // close div
  //$output .= '</div>';
  // close map block
  $output .= mapclient_common_close_map_block();
  
  // close  block
  $output .= mapclient_common_close_outer_block();
  
  return $output;
}

/**
 * Finish for all maps.
 * This mapclient function is used to add javascript code in the footer.
 */
function mapclient_finish($map_count) {
  // nothing for the internal map client
}

/**
 * Adapt bbox to width/height of map
 */
function syncWorldBbox(&$bbox, $pixwidth, $pixheight, $minwidth = 2) {
  
  $width = $bbox[2] - $bbox[0];
  $height = $bbox[3] - $bbox[1];
  
  if ($width < $minwidth) {
    $diff = $minwidth - $width;
    $diff /= 2;
    $bbox[0] -= $diff;
    $bbox[2] += $diff;
    $width = $bbox[2] - $bbox[0];
  }
  $ratio = $pixwidth/$pixheight;
  if ($height == 0) {
    // enlarge height of bbox
    $diff = $width / $ratio - $height;
    $diff /= 2;
    $bbox[1] -= $diff;
    $bbox[3] += $diff;
  }
  else if ($height > 0) {
    if ($ratio > $width/$height) {
      // enlarge width of bbox
      $diff = $height * $ratio - $width;
      $diff /= 2;
      $bbox[0] -= $diff;
      $bbox[2] += $diff;
    }
    else {
      // enlarge height of bbox
      $diff = $width / $ratio - $height;
      $diff /= 2;
      $bbox[1] -= $diff;
      $bbox[3] += $diff;
    }
  }
  return $bbox;
}

