<?php

require_once('dom.inc.php');

/**
 *
 */
class MapContext {
  
  public $dom;
  protected $root;
  public $general;
  public $extension;
  protected $window;
  protected $list;
  
  function MapContext( $node) {
    $this->dom = new dom_DOMDocument('1.0', 'UTF-8');
    
    $this->root = $this->init();
    
    // window
    $this->window = dom_appendChild($this->general, $this->dom->createElement( 'Window' ) );
    // title
    dom_appendChild($this->general, $this->dom->createElement('Title', $node->title));

    // optional: body as abstract
    dom_appendChild($this->general, $this->dom->createElement('Abstract', $node->body));
    
    // keywordlist from taxonomy
    if (is_array($node->taxonomy) && count($node->taxonomy) > 0) {
      // @todo: get taxonomy terms
      $keywordlist = dom_appendChild($this->general, $this->dom->createElement('KeywordList'));
      
      //drupal_set_message('taxonomy = '.var_export($node->taxonomy, true));
      
      $taxonomy = $node->taxonomy;
      if ($node->taxonomy['tags']) {
        // when submitting a node
        $all_tags = $node->taxonomy['tags']; // copy, do not change the node!
        while ($tags = drupal_explode_tags(array_shift($all_tags))) {
          foreach ($tags as $keyword) {
            dom_appendChild($keywordlist, $this->dom->createElement('Keyword', $keyword));
          }
        }
      }
      else {
        // when having loaded a node e.g. for node/x/context
        foreach ($node->taxonomy as $keyword) {
          dom_appendChild($keywordlist, $this->dom->createElement('Keyword', $keyword->name));
        }
      }
    }
    
    // todo: DescriptionURL
    
    // Extension
    $this->extension = dom_appendChild($this->general, $this->dom->createElement('Extension'));
  }
  
  function init() {
    $this->root = $root = $this->dom->createElement( 'ViewContext' );
    dom_setAttribute($root, 'version', '1.1.0' );
    dom_setAttribute($root, 'xmlns', "http://www.opengis.net/context");
    dom_setAttribute($root, 'xmlns:xlink', "http://www.w3.org/1999/xlink");
    dom_setAttribute($root, 'xmlns:xsi', "http://www.w3.org/2001/XMLSchema-instance");
    dom_setAttribute($root, 'xmlns:ol', "http://openlayers.org/context");
    dom_setAttribute($root, 'xmlns:sld', "http://www.opengis.net/sld");
    dom_setAttribute($root, 'xsi:schemaLocation', "http://www.opengis.net/context http://schemas.opengis.net/context/1.1.0/context.xsd");

    dom_appendChild($root, $this->dom->createComment(t('This file is generated by the drupal mapcontext module at !time.', array('!time' => format_date(time())))));
    
    $this->general = dom_appendChild($this->root, $this->dom->createElement( 'General' ) );
    $this->list = dom_appendChild($this->root, $this->dom->createElement( 'LayerList' ) );
    
    return dom_appendChild($this->dom->doc, $root );
  }
  
  function setWindow($width, $height) {
    dom_setAttribute($this->window, 'width', $width );
    dom_setAttribute($this->window, 'height', $height );
  }
  
  function setBoundingBox($parent, $bbox, $srs = NULL) {
    $bboxNode = dom_appendChild($parent, $this->dom->createElement('BoundingBox'));
    if (isset($srs)) {
      dom_setAttribute($bboxNode, 'SRS', $srs);
    }
    $this->addExtentAttrs($bboxNode, $bbox);
  }
  
  function addExtentAttrs($node, $bbox) {
    dom_setAttribute($node, 'minx', $bbox[0]);
    dom_setAttribute($node, 'miny', $bbox[1]);
    dom_setAttribute($node, 'maxx', $bbox[2]);
    dom_setAttribute($node, 'maxy', $bbox[3]);
  }
  
  function addLayer($layerData) {
    $layer = $this->dom->createElement('Layer');
    dom_setAttribute($layer, 'hidden', $layerData['hidden']);
    dom_setAttribute($layer, 'queryable', $layerData['queryable']);
    
    dom_appendChild($layer, $this->dom->createElement('Name', $layerData['names']));
    dom_appendChild($layer, $this->dom->createElement('Title', $layerData['layer_title']));
    
    $server = dom_appendChild($layer, $this->dom->createElement( 'Server'));
    dom_setAttribute($server, 'version', $layerData['version']);
    dom_setAttribute($server, 'service', "OGC:WMS");
    $onlineResource = dom_appendChild($server, $this->dom->createElement('OnlineResource'));
    dom_setAttribute($onlineResource, 'xlink:type', 'simple');
    dom_setAttribute($onlineResource, 'xlink:href', $layerData['href']);
    
    // WMC version 1.1.0 introducs sld:MinScaleDenominator and sld:MaxScaleDenominator
    if ($layerData['minscaledenominator'] > 0) {
      dom_appendChild($layer, $this->dom->createElement('sld:MinScaleDenominator', $layerData['minscaledenominator']));
    }
    if ($layerData['maxscaledenominator'] > 0) {
      dom_appendChild($layer, $this->dom->createElement('sld:MaxScaleDenominator', $layerData['maxscaledenominator']));
    }
    
    // set default image format
    $item = dom_appendChild($layer, $this->dom->createElement('FormatList'));
    $image_format = empty($layerData['image_format']) ? variable_get('mappingkit_image_format', DEFAULT_MAPPINGKIT_IMAGE_FORMAT) : ('image/' . $layerData['image_format']);
    $item = dom_appendChild($item, $this->dom->createElement('Format', $image_format));
    dom_setAttribute($item, 'current', '1');
    
    // legend graphic
    $item = dom_appendChild($layer, $this->dom->createElement('StyleList'));
    $style = dom_appendChild($item, $this->dom->createElement('Style'));
    dom_setAttribute($style, 'current', '1');
    dom_appendChild($style, $this->dom->createElement('Name', 'default'));
    dom_appendChild($style, $this->dom->createElement('Title', 'default'));
    if ($layerData['legendurl'] != '') {
      $legend_url = $layerData['legendurl'];
      
      // todo: derive image format from legend_url
      $legend_format = variable_get('mappingkit_image_format', DEFAULT_MAPPINGKIT_IMAGE_FORMAT);
    }
    else {
      // maybe the legend can be generated by the service (assume umn mapserver type)
      $legend_format = variable_get('mappingkit_image_format', DEFAULT_MAPPINGKIT_IMAGE_FORMAT);
      $legend_url = $layerData['href']
      . '&version=' . $layerData['version']
      . '&request=GetLegendGraphic&layer=' . $layerData['names']
      . '&format=' . $legend_format;
    }
    
    $url = dom_appendChild($style, $this->dom->createElement('LegendURL'));
    dom_setAttribute($url, 'format', $legend_format);
    $rsc = dom_appendChild($url, $this->dom->createElement('OnlineResource'));
    dom_setAttribute($rsc, 'xlink:type', 'simple');
    dom_setAttribute($rsc, 'xlink:href', $legend_url);
    
    // extension: default settings for openlayers map client
    $layer_extension = dom_appendChild($layer, $this->dom->createElement('Extension'));
    dom_appendChild($layer_extension, $this->dom->createElement('ol:transparent', 'true'));
    dom_appendChild($layer_extension, $this->dom->createElement('ol:isBaseLayer', $layerData['baselayer'] == 1 ? 'true' : 'false'));
    if (isset($layerData['opacity'])) {
      dom_appendChild($layer_extension, $this->dom->createElement('ol:opacity', $layerData['opacity']));
    }
    // ol:maxExtent
    // ol:displayInLayerSwitcher
    // ol:singleTile
    // ol:units
    // ol:numZoomLevels
    
    dom_appendChild($this->list, $layer );
  }
  
  function getXML() {
    return $this->dom->saveXML();
  }
}

/**
 *
 */
class OwsContext extends MapContext {
  
  function OwsContext( $node) {
    parent::MapContext( $node );
  }
  
  function init() {
    $this->root = $root = $this->dom->createElement('OWSContext');
    dom_setAttribute($root, 'version', '0.0.13' );
    dom_setAttribute($root, 'xmlns', "http://www.opengis.net/context" );
    dom_setAttribute($root, 'xmlns:xlink', "http://www.w3.org/1999/xlink" );
    dom_setAttribute($root, 'xmlns:xsi', "http://www.w3.org/2001/XMLSchema-instance" );

    dom_setAttribute($root, 'xmlns:sld', "http://www.opengis.net/sld");
    dom_setAttribute($root, 'xmlns:ogc', "http://www.opengis.net/ogc");
    dom_setAttribute($root, 'xmlns:ows', "http://www.opengis.net/ows");
    dom_setAttribute($root, 'xmlns:param', "http;//www.opengis.net/param");
    dom_setAttribute($root, 'xsi:schemaLocation', "http://www.opengis.net/oc oc_0_0_13.xsd");

    $this->general = dom_appendChild($this->root, $this->dom->createElement( 'General' ) );
    $this->list = dom_appendChild($this->root, $this->dom->createElement( 'ResourceList' ) );
    
    return dom_appendChild($this->dom->doc, $root );
  }
  
  function setBoundingBox($parent, $bbox, $srs = NULL) {
    $node = dom_appendChild($this->general, $this->dom->createElement( 'ows:BoundingBox' ) );
    if (isset($srs)) {
      dom_setAttribute($node, 'crs', $srs);
    }
    dom_appendChild($node, $this->dom->createElement('ows:LowerCorner', $bbox[0].' '.$bbox[1]));
    dom_appendChild($node, $this->dom->createElement('ows:UpperCorner', $bbox[2].' '.$bbox[3]));
  }
  
}

function mapcontext_node_as_xml($node) {
  if ($node->type != 'mapcontext') {
    return NULL;
  }
  
  // create context object
  if ($node->xml_schema == 1) {
    $context = new OwsContext($node);
  }
  else {
    $context = new MapContext($node);
  }
  
  // set window width and height
  $context->setWindow(isset($params['width']) ? $params['width'] : $node->width
    , isset($params['height']) ? $params['height'] : $node->height);
  
  // set bounding box of default/initial extent
  if (is_numeric($node->dminx)) {
    $bbox = array($node->dminx, $node->dminy, $node->dmaxx, $node->dmaxy);
  }
  else {
    $bbox = array($node->minx, $node->miny, $node->maxx, $node->maxy);
  }
  $context->setBoundingBox($context->general, $bbox, 'EPSG:'. $node->srid);
  
  // extension
  if (!empty($node->extension['baselayers'])) {
    dom_appendChild($context->extension, $context->dom->createElement('ol:baselayers', $node->extension['baselayers']));
  }
  $bbox = NULL;
  if (is_numeric($node->minx)) {
    $bbox = array($node->minx, $node->miny, $node->maxx, $node->maxy);
  }
  else if ($node->srid == 4326) {
    $bbox = array(-180, -90, 180, 90);
  }
  if ($bbox) {
    $extNode = dom_appendChild($context->extension, $context->dom->createElement('ol:maxExtent'));
    $context->addExtentAttrs($extNode, $bbox);
  }
  
  // add layers to context
  foreach ($node->layers as $layerData) {
    $context->addLayer($layerData);
  }
  
  return $context->getXML();
}

function mapcontext_xml($node) {
  
  $xml = mapcontext_node_as_xml($node);
  if ($xml) {
    // output xml
    drupal_set_header('Content-Type: text/xml; charset=utf-8');
    echo $xml;
    exit();
  }
}

function mapcontext_write_xml($node) {
  $xml = mapcontext_node_as_xml($node);
  if ($xml) {
    if ($node->nid) {
      $path = mapcontext_get_context_path($node->nid);
      
      // check that folder exists?
      
      if (FALSE === file_put_contents($path, $xml)) {
        drupal_set_message(t('Failed to write %path.', array('%path' => $path)), 'warning');
      }
      else {
        drupal_set_message(t('Xml is written to %path.', array('%path' => $path)));
        return TRUE;
      }
    }
    else {
      drupal_set_message('no nid');
    }
  }
  else {
    drupal_set_message('no xml');
  }
  return FALSE;
}

/**
 * Returns context paths with node titles.
 */
function mapcontext_autocomplete($string = '') {
  // The user enters a comma-separated list. We only autocomplete the last item.
  $array = drupal_explode_tags($string);

  // Fetch last tag
  $last_string = trim(array_pop($array));
  $matches = array();
  //if ($last_string != '') {
    $result = db_query_range("SELECT DISTINCT nid, title FROM {node} WHERE type = 'mapcontext' and LOWER(title) LIKE LOWER('%%%s%%') ORDER BY title", $last_string, 0, 10);
    
    $prefix = count($array) ? implode(', ', $array) .', ' : '';
    
    while ($rec = db_fetch_object($result)) {
      $matches[$prefix .'node/'. $rec->nid .'/context'] = $rec->title;
    }
  //}
  
  drupal_json($matches);
}
