<?php

class WMC {
  public $namespaces;
  
  public $doc;
  public $root;
  public $context;
  
  function __construct() {
    $this->namespaces = array(
      'ol'=> "http://openlayers.org/context",
      'wmc'=> "http://www.opengis.net/context",
      'sld'=> "http://www.opengis.net/sld",
      'xlink'=> "http://www.w3.org/1999/xlink",
      'xsi'=> "http://www.w3.org/2001/XMLSchema-instance"
    );
  }
  
  /**
   * @return $context
   *   Associative array containing map context properties.
   */
  function load($xml) {
    $this->doc = new DOMDocument('1.0', 'UTF-8');
    $this->doc->loadXML($xml);
    $this->doc->formatOutput = TRUE;
    $this->root = $this->doc->documentElement;
    $this->context = array('version' => $this->root->getAttribute('version'));
    $this->runChildNodes($this->context, $this->root);
    return $this->context;
  }
  
  function getNamespacePrefix($uri = NULL) {
    if (!$uri) {
      return 'wmc';
    }
    else {
      foreach ($this->namespaces as $prefix => $namespace) {
        if ($namespace == $uri) {
          return $prefix;
        }
      }
    }
    return NULL;
  }
  
  function runChildNodes(&$obj, $node) {
    foreach ($node->childNodes as $childNode) {
      if ($childNode->nodeType == 1) {
        $prefix = 'wmc';
        $prefix = $this->getNamespacePrefix($childNode->namespaceURI);
        $local = array_pop(explode(':', $childNode->nodeName));
        $function = 'read_'. $prefix .'_'. $local;
        if (method_exists($this, $function)) {
          $this->$function($obj, $childNode);
        }
        else if (function_exists($function)) {
          $function($obj, $childNode);
        }
      }
    }
  }
}

function run_child_nodes(&$obj, $node) {
  static $wmc = NULL;
  
  if (!$wmc) {
    $wmc = new WMC();
  }
  
  foreach ($node->childNodes as $childNode) {
    if ($childNode->nodeType == 1) {
      $prefix = 'wmc';
      $prefix = $wmc->getNamespacePrefix($childNode->namespaceURI);
      $local = array_pop(explode(':', $childNode->nodeName));
      $function = 'read_'. $prefix .'_'. $local;
      if (function_exists($function)) {
        $function($obj, $childNode);
      }
    }
  }
}

function get_child_value($node, $default = '') {
  $value = $node->firstChild->nodeValue;
  return $value ? $value : $default;
}

function read_wmc($name, &$context, $node) {
  $value = get_child_value($node);
//  drupal_set_message(var_export(array('tag'=>$node->tagName, 'nodeValue' => $node->nodeValue, 'value' => $value),true));
  if ($value) {
    $context[$name] = $value;
  }
}

function read_wmc_General(&$context, $node) {
  run_child_nodes($context, $node);
}

function read_wmc_Window(&$context, $node) {
  $context['width'] = $node->getAttribute('width');
  $context['height'] = $node->getAttribute('height');
}

function read_wmc_Name(&$context, $node) {
  read_wmc('Name', $context, $node);
}

function read_wmc_Title(&$context, $node) {
  read_wmc('Title', $context, $node);
}

function read_wmc_Abstract(&$context, $node) {
  read_wmc('Abstract', $context, $node);
}

function read_wmc_Server(&$layer_info, $node) {
  $layer_info['params']['version'] = $node->getAttribute('version');
  run_child_nodes($layer_info, $node);
}

function read_wmc_OnlineResource(&$obj, $node) {
  $obj['href'] = $node->getAttributeNS('http://www.w3.org/1999/xlink', 'href');
}

function read_wmc_BoundingBox(&$context, $node) {
  $context['srs'] = $node->getAttribute('SRS');
  $context['bbox'] = array('minx' => $node->getAttribute('minx')
    , 'miny' => $node->getAttribute('miny')
    , 'maxx' => $node->getAttribute('maxx')
    , 'maxy' => $node->getAttribute('maxy')
    );
}

function read_wmc_Extension(&$obj, $node) {
  $extension = array();
  run_child_nodes($extension, $node);
  $obj['extension'] = $extension;
}

function read_ol_maxExtent(&$obj, $node) {
  $obj['maxExtent'] = array('minx' => $node->getAttribute('minx')
    , 'miny' => $node->getAttribute('miny')
    , 'maxx' => $node->getAttribute('maxx')
    , 'maxy' => $node->getAttribute('maxy')
    );
}

function read_ol_opacity(&$obj, $node) {
  $obj['opacity'] = get_child_value($node);
}

function read_ol_singleTile(&$obj, $node) {
  $obj['singleTile'] = get_child_value($node);
}

function read_wmc_LayerList(&$context, $node) {
  $context['layers'] = array();
  run_child_nodes($context, $node);
}

function read_wmc_Layer(&$context, $node) {
  $layer_info = array('params' => array()
    , 'options' => array('visibility' => $node->getAttribute('hidden') != '1')
    , 'queryable' => $node->getAttribute('queryable') == '1'
    , 'formats' => array()
    , 'styles' => array()
    );
  run_child_nodes($layer_info, $node);
//  drupal_set_message('layer info:'.var_export($layer_info,true));
  $context['layers'][] = $layer_info;
}

function read_wmc_FormatList(&$layer_info, $node) {
  run_child_nodes($layer_info, $node);
}

function read_wmc_Format(&$layer_info, $node) {
  $format = get_child_value($node);
  $layer_info['formats'][] = $format;
  if ($node->getAttribute('current') == '1') {
    $layer_info['params']['format'] = $format;
  }
}

function read_wmc_StyleList(&$layer_info, $node) {
  run_child_nodes($layer_info, $node);
}

function read_wmc_Style(&$layer_info, $node) {
  $style = array();
  run_child_nodes($style, $node);
  if ($node->getAttribute('current') == '1') {
    if (isset($style['href'])) {
      // linked SLD
      $layer_info['params']['sld'] = $style['href'];
    }
    else if (isset($style['body'])) {
      // inline SLD
      $layer_info['params']['sld_body'] = $style['body'];
    }
    else {
      // named style
      $layer_info['params']['styles'] = $style['Name'];
    }
  }
  $layer_info['styles'][] = $style;
}

function read_wmc_LegendURL(&$style, $node) {
  $legend = array();
  if ($node->hasAttribute('width')) {
    $legend['width'] = $node->getAttribute('width');
  }
  if ($node->hasAttribute('height')) {
    $legend['height'] = $node->getAttribute('height');
  }
  $links = $node->getElementsByTagName('OnlineResource');
  if ($links->length > 0) {
    read_wmc_OnlineResource($legend, $links->item(0));
  }
  $style['legend'] = $legend;
}

