<?php

/**
 * @file
 * Parser for WMC documents.
 */

// include xml parser
module_load_include('inc', 'mappingkit', 'includes/xml.parser');

class WMC_Parser extends XML_Parser {
  //public $context;
  
  function __construct() {
    parent::__construct();
    
    $this->namespaces['ol'] = "http://openlayers.org/context";
    $this->namespaces['sld'] = "http://www.opengis.net/sld";
    $this->namespaces['wmc'] = "http://www.opengis.net/context";
    
    $this->defaultNamespacePrefix = 'wmc';
  }
  
  function read_wmc_General(&$context, $node) {
    $this->run_child_nodes($context, $node);
  }
  
  function read_wmc_Window(&$context, $node) {
    $context['width'] = $node->getAttribute('width');
    $context['height'] = $node->getAttribute('height');
  }
  
  function read_wmc_Name(&$context, $node) {
    $this->read_property('Name', $context, $node);
  }
  
  function read_wmc_Title(&$context, $node) {
    $this->read_property('Title', $context, $node);
  }
  
  function read_wmc_Abstract(&$context, $node) {
    $this->read_property('Abstract', $context, $node);
  }
  
  function read_wmc_Server(&$layer_info, $node) {
    $layer_info['params']['version'] = $node->getAttribute('version');
    $this->run_child_nodes($layer_info, $node);
  }
  
  function read_wmc_OnlineResource(&$obj, $node) {
    $obj['href'] = $node->getAttributeNS('http://www.w3.org/1999/xlink', 'href');
  }
  
  function read_wmc_BoundingBox(&$context, $node) {
    $context['srs'] = $node->getAttribute('SRS');
    $context['bbox'] = array('minx' => $node->getAttribute('minx')
      , 'miny' => $node->getAttribute('miny')
      , 'maxx' => $node->getAttribute('maxx')
      , 'maxy' => $node->getAttribute('maxy')
      );
  }
  
  function read_wmc_Extension(&$obj, $node) {
    $extension = array();
    $this->run_child_nodes($extension, $node);
    $obj['extension'] = $extension;
  }
  
  function read_ol_maxExtent(&$obj, $node) {
    $obj['maxExtent'] = array('minx' => $node->getAttribute('minx')
      , 'miny' => $node->getAttribute('miny')
      , 'maxx' => $node->getAttribute('maxx')
      , 'maxy' => $node->getAttribute('maxy')
      );
  }
  
  function read_ol_opacity(&$obj, $node) {
    $obj['opacity'] = $this->get_child_value($node);
  }
  
  function read_ol_singleTile(&$obj, $node) {
    $obj['singleTile'] = $this->get_child_value($node);
  }
  
  function read_wmc_LayerList(&$context, $node) {
    $context['layers'] = array();
    $this->run_child_nodes($context, $node);
  }
  
  function read_wmc_Layer(&$context, $node) {
    $layer_info = array('params' => array()
      , 'options' => array('visibility' => $node->getAttribute('hidden') != '1')
      , 'queryable' => $node->getAttribute('queryable') == '1'
      , 'formats' => array()
      , 'styles' => array()
      );
    $this->run_child_nodes($layer_info, $node);
  //  drupal_set_message('layer info:'.var_export($layer_info,true));
    $context['layers'][] = $layer_info;
  }
  
  function read_wmc_FormatList(&$layer_info, $node) {
    $this->run_child_nodes($layer_info, $node);
  }
  
  function read_wmc_Format(&$layer_info, $node) {
    $format = $this->get_child_value($node);
    $layer_info['formats'][] = $format;
    if ($node->getAttribute('current') == '1') {
      $layer_info['params']['format'] = $format;
    }
  }
  
  function read_wmc_StyleList(&$layer_info, $node) {
    $this->run_child_nodes($layer_info, $node);
  }
  
  function read_wmc_Style(&$layer_info, $node) {
    $style = array();
    $this->run_child_nodes($style, $node);
    if ($node->getAttribute('current') == '1') {
      if (isset($style['href'])) {
        // linked SLD
        $layer_info['params']['sld'] = $style['href'];
      }
      else if (isset($style['body'])) {
        // inline SLD
        $layer_info['params']['sld_body'] = $style['body'];
      }
      else {
        // named style
        $layer_info['params']['styles'] = $style['Name'];
      }
    }
    $layer_info['styles'][] = $style;
  }
  
  function read_wmc_LegendURL(&$style, $node) {
    $legend = array();
    if ($node->hasAttribute('width')) {
      $legend['width'] = $node->getAttribute('width');
    }
    if ($node->hasAttribute('height')) {
      $legend['height'] = $node->getAttribute('height');
    }
    $links = $node->getElementsByTagName('OnlineResource');
    if ($links->length > 0) {
      $this->read_wmc_OnlineResource($legend, $links->item(0));
    }
    $style['legend'] = $legend;
  }
}
