<?php

/**
 * @file
 * Page callback file for the maplayer module.
 */

/**
 * Menu callback; displays a Drupal page containing recent map layers of a given user.
 * @param $account
 *   Account object for user whose map layers need to be displayed.
 * @return
 *   Output string or drupal_not_found() if the user does not exist.
 */
function maplayer_page_user($account) {
  //drupal_set_message(var_export($account,true));
  global $user;

  //$account = user_load(array((is_numeric($uid) ? 'uid' : 'name') => $uid, 'status' => 1));

  if (!empty($account->uid)) {
    drupal_set_title($title = t("@name's map layers", array('@name' => $account->name)));

    $items = array();
    if (($account->uid == $user->uid) && user_access('create maplayer')) {
      $items[] = l(t('Post a new Map layer.'), "node/add/maplayer");
    }
    else if ($account->uid == $user->uid) {
      $items[] = t('You are not allowed to post a new Map layer.') .'</li>';
    }

    $output = theme('item_list', $items);
    
    $result = pager_query(db_rewrite_sql("SELECT n.nid, n.sticky, n.created FROM {node} n WHERE n.type = 'maplayer' AND n.uid = %d AND n.status = 1 ORDER BY n.sticky DESC, n.created DESC")
      , variable_get('default_nodes_main', 10), 0, NULL, $account->uid);
    while ($node = db_fetch_object($result)) {
      $output .= node_view(node_load($node->nid), 1);
    }
    $output .= theme('pager', NULL, variable_get('nodes_default_main', 10));
    drupal_add_feed(url('maplayer/'. $account->uid .'/feed'), t('RSS - !title', array('!title' => $title)));

    return $output;
  }
  else {
    drupal_not_found();
  }
}

/**
 * Menu callback; displays a Drupal page containing recent map layers of all users.
 */
function maplayer_page_last() {
  global $user;

  $output = '';

  $result = pager_query(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n WHERE n.type = 'maplayer' AND n.status = 1 ORDER BY n.sticky DESC, n.created DESC")
    , variable_get('default_nodes_main', 10));

  while ($node = db_fetch_object($result)) {
    $output .= node_view(node_load($node->nid), 1);
  }
  $output .= theme('pager', NULL, variable_get('default_nodes_main', 10));
  drupal_add_feed(url('maplayer/feed'), t('RSS - Map layers'));

  return $output;
}

/**
 * Menu callback; displays an RSS feed containing recent Map layer entries of a given user.
 */
function maplayer_feed_user($account) {
  $result = db_query_range(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n WHERE n.type = 'maplayer' AND n.uid = %d AND n.status = 1 ORDER BY n.created DESC")
    , $account->uid, 0, variable_get('maplayer_max_items', 10));
  $channel['title'] = t("@name's map layers", array('@name' => $account->name));
  $channel['link'] = url("maplayer/{$account->uid}", array('absolute' => TRUE));

  $items = array();
  while ($row = db_fetch_object($result)) {
    $items[] = $row->nid;
  }
  node_feed($items, $channel);
}

/**
 * Menu callback; displays an RSS feed containing recent Map layer entries of all users.
 */
function maplayer_feed_last() {
  $result = db_query_range(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n WHERE n.type = 'maplayer' AND n.status = 1 ORDER BY n.created DESC")
    , 0, variable_get('maplayer_max_items', 10));
  $channel['title'] = t("@name's map layers", array('@name' => variable_get('site_name', 'Drupal')));
  $channel['link'] = url('maplayer', array('absolute' => TRUE));

  while ($row = db_fetch_object($result)) {
    $items[] = $row->nid;
  }
  node_feed($items, $channel);
}

