<?php
/**
 * @file
 * Mappingkit rss feed of users containing thumbnails and location of users.
 */

/**
 * Menu callback; displays an RSS feed containing users.
 */
function mappingkit_user_feed() {
  global $base_url, $language;
  
  $cached_feed = cache_get('mappingkit_user_feed');
  if ($cached_feed !== 0) {
    $output = $cached_feed->data;
  }
  else {
    $channel['title'] = variable_get('site_name', 'Drupal') .' Users';
    $channel['link'] = url('user', array('absolute' => TRUE));
    
    $uids = array();
    $result = db_query('SELECT n.uid, n.created FROM {users} n WHERE n.status = 1 ORDER BY n.created DESC');
    while ($row = db_fetch_object($result)) {
      $uids[] = $row->uid;
    }
    
    $item_length = variable_get('feed_item_length', 'teaser');
    $namespaces = array('xmlns:dc' => 'http://purl.org/dc/elements/1.1/');
    
    $user_picture_default = variable_get('user_picture_default', '');
    if (!empty($user_picture_default)) {
      $user_picture_default_url = mapclient_get_url($user_picture_default, TRUE);
    }
    
    $items = '';
    foreach ($uids as $uid) {
      // Load the specified user
      $item = user_load($uid);
      //global $user; $item = $user;
      $item->build_mode = NODE_BUILD_RSS;
      $item->link = url("user/$uid", array('absolute' => TRUE));
      
      // get user view
      user_build_content($item);
      // To theme user profiles, copy modules/user/user_profile.tpl.php
      // to your theme directory, and edit it as instructed in that file's comments.
      $item_text = theme('user_profile', $item);
      
      // Allow modules to add additional item fields and/or modify $item
      $extra = array();
      user_module_invoke('rss item', $extra, $item);
      
      // add properties pubDate and guid
      $extra = array_merge($extra, array(
        array('key' => 'pubDate', 'value' => gmdate('r', $item->created))
        , array('key' => 'guid', 'value' => $item->uid .' at '. $base_url, 'attributes' => array('isPermaLink' => 'false'))
        ));
      // add thumbnail
      if (!empty($item->picture)) {
        $extra[] = array('key' => 'thumbnail', 'attributes' => array('url' => mapclient_get_url($item->picture, TRUE)));
      }
      else if (!empty($user_picture_default)) {
        $extra[] = array('key' => 'thumbnail', 'attributes' => array('url' => $user_picture_default_url));
      }
      
      foreach ($extra as $element) {
        if (isset($element['namespace'])) {
          $namespaces = array_merge($namespaces, $element['namespace']);
        }
      }
      $items .= format_rss_item($item->name, $item->link, $item_text, $extra);
    }
  
    $channel_defaults = array(
      'version'     => '2.0',
      'title'       => variable_get('site_name', 'Drupal'),
      'link'        => $base_url,
      'description' => variable_get('site_mission', ''),
      'language'    => $language->language
    );
    $channel = array_merge($channel_defaults, $channel);
  
    $output = '<?xml version="1.0" encoding="utf-8"?>'."\n";
    $output .= '<rss version="'. $channel['version'] .'" xml:base="'. $base_url .'" '. drupal_attributes($namespaces) .'>'."\n";
    $output .= format_rss_channel($channel['title'], $channel['link'], $channel['description'], $items, $channel['language']);
    $output .= "</rss>\n";
    
    // save in cache
    cache_set('mappingkit_user_feed', $output, 'cache', CACHE_TEMPORARY);
  }
  drupal_set_header('Content-Type: application/rss+xml; charset=utf-8');
  echo $output;
  //exit();
}

