<?php

function mapserver_settings() {
  $form['mapscript'] = array(
    '#type' => 'fieldset'
    , '#title' => t('MapScript')
    , '#collapsible' => TRUE
    );
  
  // check if mapscript extension is available
  $mapscript_extension = variable_get('mapserver_mapscript_extension', DEFAULT_MAPSCRIPT_EXTENSION);
  $form['mapscript']['mapserver_mapscript_extension'] = array('#type' => 'textfield'
    , '#default_value' => $mapscript_extension
    );
  if (extension_loaded('MapScript')) {
    $form['mapscript']['mapserver_mapscript_extension']['#description'] = 
      t('The PHP MapScript extension is loaded');
  }
  else {
    // try to load php_mapscript.so
    // TODO: mapscript functions in include file
    if (function_exists('dl') && dl($mapscript_extension)) {
      $form['mapscript']['mapserver_mapscript_extension']['#description'] = 
        t('This PHP MapScript extension has been confirmed to load dynamically');
    }
    else {
      //form_set_error('mapserver_mapscript_extension', t('The PHP MapScript extension failed to load'));
      $form['mapscript']['mapserver_mapscript_extension']['#description'] = 
        t('This PHP MapScript extension failed to load dynamically. This module will be less functional if MapScript is not available.');
    }
  }
  
  $form['mapserver'] = array(
    '#type' => 'fieldset'
    , '#title' => t('Mapserver')
    , '#collapsible' => TRUE
    );
  $form['mapserver']['mappingkit_mapserver_url'] = array(
    '#type' => 'textfield'
    , '#title' => t('Mapserver URL')
    , '#description' => t('Web path to Mapserver CGI')
    , '#default_value' => variable_get('mappingkit_mapserver_url', DEFAULT_MAPPINGKIT_MAPSERVER_URL)
    );
  $mapserver_map_path = variable_get('mapserver_map_path', DEFAULT_MAPSERVER_MAP_PATH);
  if (!file_exists($mapserver_map_path)) {
    if (mkdir($mapserver_map_path)) {
      drupal_set_message(t('Folder %folder was created for storage of map files.', array('%folder' => $mapserver_map_path)));
    }
  }
  $form['mapserver']['mapserver_map_path'] = array(
    '#type' => 'textfield'
    , '#title' => t('Map file folder')
    , '#description' => t('Path to folder for storage of map files. This path can be relative to the Drupal installation path. If it does not exist, it will be created.')
    , '#default_value' => variable_get('mapserver_map_path', DEFAULT_MAPSERVER_MAP_PATH)
    );
  
  $projections = mappingkit_get_projections();
  
  $form['mapserver']['mapserver_default_projection'] = array(
    '#type' => 'select'
    , '#title' => t('Default projection')
    , '#options' => $projections
    , '#description' => t('Default projection for map files')
    , '#default_value' => variable_get('mapserver_default_projection', '4326')
    );
  
  $form['config'] = array('#type' => 'fieldset'
    , '#title' => t('Config options')
    , '#collapsible' => TRUE
    );
  
  $form['config']['mapserver_config_proj_lib'] = array('#type' => 'textfield'
    , '#title' => 'PROJ_LIB'
    , '#default_value' => variable_get('mapserver_config_proj_lib', '')
    , '#description' => t('Set PROJ_LIB to the folder containing the epsg file. Use this option only in case of msProcessProjection() errors.')
    );
  if (empty($_ENV['PROJ_LIB'])) {
    $supp = t("You may want to add a statement like '%statement' to your apache configuration. See also !link.", array('%statement' => 'SetEnv PROJ_LIB /path/to/nad'
      , '!link' => l('this page', 'http://mapserver.org/errors.html?highlight=proj_lib#setting-the-location-of-the-epsg-file')));
  }
  else {
    $supp = t("Note that the PROJ_LIB environment variable is set to %env", array('%env' => $_ENV['PROJ_LIB']));
  }
  $form['config']['mapserver_config_proj_lib']['#description'] .= ' ' . $supp;
  
  $form['contact'] = array('#type' => 'fieldset'
    , '#title' => t('Contact information')
    , '#collapsible' => TRUE
    , '#description' => t('This information is used for map files and will appear in their WMS/WFS capabilities responses.')
    );
  $form['contact']['mapserver_contact_person'] = array('#type' => 'textfield'
    , '#title' => t('Person')
    , '#default_value' => variable_get('mapserver_contact_person', '')
    );
  $form['contact']['mapserver_contact_organisation'] = array('#type' => 'textfield'
    , '#title' => t('Organisation')
    , '#default_value' => variable_get('mapserver_contact_organisation', '')
    );
  $form['contact']['mapserver_contact_mail'] = array('#type' => 'textfield'
    , '#title' => t('E-mail')
    , '#default_value' => variable_get('mapserver_contact_mail', '')
    );
  
  $form['rendering'] = array('#type' => 'fieldset'
    , '#title' => t('Rendering')
    , '#collapsible' => TRUE
    );
  $form['rendering']['mapserver_maxsize'] = array('#type' => 'textfield'
    , '#title' => t('Maximum size')
    , '#default_value' => variable_get('mapserver_maxsize', DEFAULT_MAPSERVER_MAXSIZE)
    , '#description' => t('')
    );
  /*
  $form['rendering']['mapserver_output_format'] = array('#type' => 'select'
    , '#title' => t('Output format')
    , '#options' => array()
    , '#default_value' => variable_get('mapserver_output_format', '')
    );
  */
  $form['rendering']['mappingkit_png_driver'] = array('#type' => 'select'
    , '#title' => t('PNG Renderer')
    , '#options' => array('GD/PNG' => 'GD renderer', 'AGG/PNG' => 'AGG renderer')
    , '#default_value' => variable_get('mappingkit_png_driver', DEFAULT_MAPPINGKIT_PNG_DRIVER)
    );
  $form['rendering']['mapserver_copyleft'] = array('#type' => 'textfield'
    , '#title' => t('Copyleft text')
    , '#default_value' => variable_get('mapserver_copyleft', '© Mapping kit')
    , '#description' => t('Text for copyleft message, displayed at the left bottom corner of each tile. Make empty if not desired.')
    );
  
  $form['#validate'][] = 'mapserver_settings_form_validate';
  $form['#submit'][] = 'mapserver_settings_form_submit';
  
  return $form;
}

function mapserver_settings_form() {
  return system_settings_form(mapserver_settings());
}

/**
 * Validation function for mapserver settings form.
 */
function mapserver_settings_form_validate(&$form, &$form_state) {
  //
}

/**
 * Submit function for mapserver settings form.
 */
function mapserver_settings_form_submit(&$form, &$form_state) {
  if ($form_state['values']['op'] == t('Save configuration')) {
    //variable_set('mappingkit_mapserver_url', $form_state['values']['mappingkit_mapserver_url']);
    //variable_set('mapserver_map_path', $form_state['values']['mapserver_map_path']);
    //variable_set('mapserver_default_projection', $form_state['values']['mapserver_default_projection']);
    $mapserver_map_path = $form_state['values']['mapserver_map_path'];
    if (!file_exists($mapserver_map_path)) {
      if (mkdir($mapserver_map_path)) {
        drupal_set_message(t('Folder %folder was created for storage of map files.', array('%folder' => $mapserver_map_path)));
      }
    }
  }
  else if ($form_state['values']['op'] == t('Reset to defaults')) {
    //variable_del('mappingkit_mapserver_url');
    //variable_del('mapserver_map_path');
    //variable_del('mapserver_default_projection');
  }
  else {
    drupal_set_message('unknown op in mapserver submit', 'error');
  }
}


