<?php

// make certain that mapscript extension is enabled
require_once('mapscript.inc.php');

define('CLASSIFY_EQUAL_INTERVAL', 1);
define('CLASSIFY_QUANTILE', 2);

function mapserver_layer_edit($node, $layer_nr) {
  //drupal_set_message('layer_edit:' . var_export(array($node, $b), TRUE));
  if ($layer_nr == 'add') {
    drupal_set_title(t('Add layer to %title', array('%nr' => $layer_nr, '%title' => $node->title)));
    
    if (function_exists('geodata_form')) {
      // only if geodata module is active
      return drupal_get_form('mapserver_new_layer_form', $node, $layer_nr);
    }
  }
  else {
    drupal_set_title(t('Edit layer %nr of %title', array('%nr' => $layer_nr, '%title' => $node->title)));
  }
  return drupal_get_form('mapserver_layer_form', $node, $layer_nr);
}

function mapserver_new_layer_form(&$form_state, $node, $layer_nr) {
  require_once('mapscript.inc.php');
  
  //drupal_set_message('form_state: '.var_export($form_state, TRUE));
  
  $map_path = variable_get('mapserver_map_path', DEFAULT_MAPSERVER_MAP_PATH);
  $file_name = mapserver_get_real_path($node->mapfile);
  if (file_exists($file_name)) {
    $oMap = ms_newMapObj($file_name);
    
    // new layer
    $layerObj = ms_newLayerObj($oMap);
  }
  else {
    // TODO: Error: mapfile not found!
    $oMap = NULL;
  }
  
  $form['mapfile'] = array('#type' => 'hidden'
    , '#default_value' => $node->mapfile
    );
  $form['layer_nr'] = array('#type' => 'hidden'
    , '#default_value' => $layer_nr
    );
  $form['nid'] = array('#type' => 'hidden'
    , '#default_value' => $node->nid
    );
  
  $layer_type_options = array(
    MS_LAYER_POINT => t('POINT'), MS_LAYER_LINE => t('LINE'), MS_LAYER_POLYGON => t('POLYGON') 
    , MS_LAYER_RASTER => t('RASTER'), MS_LAYER_ANNOTATION => t('ANNOTATION')
    , MS_LAYER_QUERY => t('QUERY')
    , MS_LAYER_CIRCLE => t('CIRCLE')
    , MS_LAYER_TILEINDEX => t('TILEINDEX')
    );
  global $unit_options;
  
  $form['identification'] = array('#type' => 'fieldset'
    , '#title' => t('Layer identification')
    , '#collapsible' => TRUE
    );
  $form['identification']['layer_name'] = array('#type' => 'textfield'
    , '#title' => t('Name')
    , '#default_value' => ''
    , '#description' => t('Short name for this layer. Limit is 20 characters. This name is the link between the mapfile and web interfaces that refer to this name. They must be identical. The name should be unique, unless one layer replaces another at different scales. Use the GROUP option to associate layers with each other.')
    , '#required' => TRUE
    , '#maxlength' => 20
    );
  $form['identification']['layer_title'] = array('#type' => 'textfield'
    , '#title' => t('Title')
    , '#default_value' => ''
    , '#description' => t('Metadata information. Mandatory as layer title in the capabilities.')
    , '#required' => TRUE
    );
  $form['identification']['layer_abstract'] = array('#type' => 'textarea'
    , '#title' => t('Abstract')
    , '#default_value' => ''
    , '#size' => 100
    , '#rows' => 3
    , '#description' => t('Metadata information. Used for layer abstract in the capabilities.')
    );
  
  // geodata
  $form['data_source'] = array('#type' => 'fieldset'
    , '#title' => t('Data source')
    , '#collapsible' => TRUE
    );
  $form['data_source']['geodata'] = array('#type' => 'textfield'
    , '#title' => t('Geodata')
    , '#default_value' => ''
    , '#size' => 100
    , '#description' => t('Enter title of geodata node or node/id. The field information from the geodata node will be used for the classify section.')
    , '#autocomplete_path' => 'geodata/autocomplete/title'
    );
  $form['data_source']['prepare'] = array('#type' => 'button'
    , '#default_value' => t('Prepare')
    , '#validate' => array('mapserver_prepare_geodata')
    );
  
  // classification
  $form['classification'] = array('#type' => 'fieldset'
    , '#title' => t('Classification')
    , '#collapsible' => TRUE
    , '#description' => t('')
    );
  
  $geodata_node = NULL;
  $field_options = array('' => t('Select a field'));
  if (0 != preg_match('/node\/(?P<nid>\d*)/', $form_state['post']['geodata'], $matches)) {
    //drupal_set_message('matches:'.var_export($matches, TRUE));
    if ($geodata_node = node_load(array('nid' => $matches['nid']))) {
      $field_options += array_combine($geodata_node->fields, $geodata_node->fields);
    }
    //$field_options = array_combine();
  }
  $form['classification']['#collapsed'] = count($field_options) == 1;
  
  $form['classification']['geodata_nid'] = array('#type' => 'hidden'
    , '#default_value' => isset($geodata_node) ? $geodata_node->nid : ''
    );
  
  $form['classification']['field'] = array('#type' => 'select'
    , '#title' => t('Field')
    , '#options' => $field_options
    , '#default_value' => ''
    , '#required' => TRUE
    );
  $form['classification']['filter'] = array('#type' => 'textfield'
    , '#title' => t('Filter')
    , '#default_value' => ''
    , '#size' => 100
    , '#description' => t('This parameter allows for data specific attribute filtering that is done at the same time spatial filtering is done, but before any CLASS expressions are evaluated. For OGR and shapefiles the string is simply a mapserver regular expression. For spatial databases the string is a SQL WHERE clause that is valid with respect to the underlying database.')
    );
  $form['classification']['classification'] = array('#type' => 'select'
    , '#title' => t('Classification type')
    // no standard deviation, no natural breaks
    , '#options' => array('0' => t('Select a classification type'), CLASSIFY_EQUAL_INTERVAL => t('Equal interval'), CLASSIFY_QUANTILE => t('Quantile'))
    , '#default_value' => 0
    , '#required' => TRUE
    );
  $form['classification']['numclasses'] = array('#type' => 'textfield'
    , '#title' => t('Number of classes')
    //, '#options' => array(1 => 1, 2 => 2, 3 => 3, 4 => 4, 5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10)
    , '#default_value' => 5
    , '#size' => 10
    //, '#description' => t('')
    );
  $form['classification']['color1'] = array('#type' => 'textfield'
    , '#title' => t('Color 1')
    , '#attributes' => array('onchange' => "this.style.backgroundColor = this.value;")
    , '#default_value' => '00FF00'
    , '#size' => 10
    , '#maxlength' => 11 // in future, format like rrr,ggg,bbb may be permitted
    , '#theme' => 'color'
    , '#description' => t('Colour for lower value classes.')
  );
  $form['classification']['color2'] = array('#type' => 'textfield'
    , '#title' => t('Color 2')
    , '#attributes' => array('onchange' => "this.style.backgroundColor = this.value;")
    , '#default_value' => 'FF0000'
    , '#size' => 10
    , '#maxlength' => 11
    , '#theme' => 'color'
    , '#description' => t('Colour for higher value classes.')
  );
  /*
  $form['classification']['color0'] = array('#type' => 'textfield'
    , '#title' => t('Outline color')
    , '#attributes' => array('onchange' => "this.style.backgroundColor = this.value;")
    , '#default_value' => '000000'
    , '#size' => 10
    , '#maxlength' => 11
    , '#theme' => 'color'
    , '#description' => t('Colour for outline of polygons.')
  );
  */
  
  /*
  $form['classification']['classify'] = array('#type' => 'button'
    , '#value' => t('Classify')
    , '#validate' => array('mapserver_classify_geodata')
    );
  
  $classes = '';
  if (!is_null($geodata_node) && $form_state['post']['op'] == t('Classify')) {
    $classes = "1\n2";
  }*/
  
  $form['classification']['classes'] = array('#type' => 'value'
    , '#default_value' => ''
    );
  
  $form['#validate'][] = 'mapserver_new_layer_validate';
  $form['#submit'][] = 'mapserver_new_layer_submit';
  
  $form['submit'] = array('#type' => 'submit'
    , '#default_value' => t('Submit')
    );
  
  $form['#parents'] = array('identification', 'layer_abstract');
  
  return $form;
}

function mapserver_prepare_geodata($form, &$form_state) {
  // check something?
}

/**
 * Make a classification for the selected field.
 */
function mapserver_new_layer_validate(&$form, &$form_state) {
//function mapserver_classify_geodata(&$form, &$form_state) {
  //drupal_set_message('mapserver_classify_geodata '.var_export($form_state, TRUE));
  if ($form_state['values']['field'] == '') {
    form_set_error('field', t('A field must be selected.'));
  }
  
  if ($form_state['values']['classification'] == 0) {
    form_set_error('classification', t('A classification type must be selected.'));
  }
  
  $num_classes = (int)$form_state['values']['numclasses'];
  if ($num_classes < 1 || $num_classes > 100) {
    form_set_error('numclasses', t('The number of classes must be between 1 and 100'));
  }
  
  if (count(form_set_error()) == 0) {
    // determine class boundaries in table 
    // equal interval: get min/max and determine boundaries
    //drupal_set_message($form_state['values']['geodata']);
    if (0 != preg_match('/node\/(?P<nid>\d*)/', $form_state['values']['geodata'], $matches)) {
      //drupal_set_message('matches:'. var_export($matches, TRUE));
      if ($node = node_load(array('nid' => $matches['nid']))) {
        
        $options = array('numclasses' => $num_classes);
        if (!empty($form_state['values']['filter'])) {
          $options['filter'] = $form_state['values']['filter'];
        }
        
        if ($form_state['values']['classification'] == CLASSIFY_EQUAL_INTERVAL) {
          // request min/max/count of geodata node
          $result = geodata_get_min_max($node, $form_state['values']['field']);
          //drupal_set_message('result:'. var_export($result, TRUE));
          
          // Equal interval: equal range of values
          $class_lo = array(); // lower values of classes
          $range_per_class = ($result->max - $result->min)/$num_classes;
          for ($n = 0; $n <= $num_classes; $n++) {
            $class_lo[] = $result->min + $range_per_class * $n;
          }
        }
        else if ($form_state['values']['classification'] == CLASSIFY_QUANTILE) {
          // Quantile: equal number of items in each class, for the given filter
          $class_lo = geodata_get_quantiles($node, $form_state['values']['field'], $options);
        }
        
        form_set_value($form['classification']['classes'], implode("\n", $class_lo), $form_state);
        form_set_value($form['classification']['geodata_nid'], $node->nid, $form_state);
        
        //drupal_set_message(var_export(array('classes' => implode("\n", $class_lo), 'state' => $form_state), TRUE));
      }
    }
  }
}

//function mapserver_new_layer_validate(&$form, &$form_state) {
  //form_set_error('', t('Not implemented yet'));
//}

/**
 * Submit the new layer.
 */
function mapserver_new_layer_submit(&$form, &$form_state) {
  global $_mappingkit_logger;
  
  // create the new layer with its classes/symbology
  //drupal_set_message(var_export(array('state' => $form_state), TRUE));
  
  $map_path = variable_get('mapserver_map_path', DEFAULT_MAPSERVER_MAP_PATH);
  $file_name = mapserver_get_real_path($form_state['values']['mapfile']);
  if (file_exists($file_name)) {
    $oMap = ms_newMapObj($file_name);
    
    $layerObj = ms_newLayerObj($oMap);
    
    $numclasses = (int)$form_state['values']['numclasses'];
    $class_lo = explode("\n", $form_state['values']['classes']);
    
    /*
      Required for each LAYER in a WMS configuration:
      * Layer NAME
      * Layer PROJECTION
      * Layer METADATA
            o wms_title
            o wms_srs (optional since the layers inherit the map's SRS value)
      * Layer STATUS
            o Layers set to STATUS DEFAULT will always be sent to the client.
            o Layers set to STATUS ON or STATUS OFF can be requested by the client.
      * Layer TEMPLATE (required for GetFeatureInfo requests)
      * Layer DUMP TRUE (only required for GetFeatureInfo GML requests)
    */
    $geodata_node = node_load(array('nid' => (int)$form_state['values']['geodata_nid']));
    //drupal_set_message('geodata node:'. var_export($node, TRUE));
    
    if (-1 == $layerObj->set('name', $form_state['values']['layer_name'])) {
      drupal_set_message(t('Failed to set name'));
    }
    $layerObj->set('status', MS_ON);
    if ($geodata_node->geodata_type == GEODATA_IMPORT_POSTGIS) {
      $layerObj->set('connectiontype', MS_POSTGIS);
      
      // get connection data from db_url for geodata database
      global $db_url;
      $dbinfo = parse_url($db_url[$geodata_node->database_name]);
      $dbinfo['database'] = substr($dbinfo['path'], 1);
      $_mappingkit_logger->info('dbinfo:'. var_export($dbinfo, TRUE));
      $connection_str = "user={$dbinfo['user']} password={$dbinfo['pass']} dbname={$dbinfo['database']} host={$dbinfo['host']}";
      if (isset($dbinfo['port'])) {
        $connection_str .= ' port='. $dbinfo['port'];
      }
      $layerObj->set('connection', $connection_str);
      
      // set DATA to query statement
      $layerObj->set('data', "{$geodata_node->geometry_column} FROM (select * from {$geodata_node->data_path}) AS foo USING UNIQUE {$geodata_node->uniquefield}");
    }
    else {
      // set DATA to absolute path to shapefile
      $layerObj->set('data', realpath($geodata_node->data_path));
    }
    switch ($geodata_node->geometry_type) {
    case 'POLYGON':
    case 'MULTIPOLYGON':
      $layer_type = MS_LAYER_POLYGON;
      break;
    case 'POINT':
      $layer_type = MS_LAYER_POINT;
      break;
    default:
      $layer_type = MS_LAYER_LINE;
      break;
    }
    if (-1 == $layerObj->set('type', $layer_type)) {
      drupal_set_message(t('Failed to set type'));
    }
    if (-1 == $layerObj->setProjection('init=epsg:'. $geodata_node->srid, FALSE)) {
      drupal_set_message(t('Failed to set srs'));
    }
    
    if (!empty($form_state['values']['filter'])) {
      // todo: use filter expression in classification (for determining ranges of values)
      $layerObj->setFilter($form_state['values']['filter']);
    }
    
    // set units
    $layerObj->sizeunits = ($geodata_node->srid == 4326 ? MS_DEGREES : MS_METERS);
    
    setMetaData($layerObj, 'wms_title', $form_state['values']['layer_title']);
    setMetaData($layerObj, 'wms_abstract', $form_state['values']['layer_abstract']);
    $layerObj->setMetaData('wms_srs', 'EPSG:'. $geodata_node->srid);
    $layerObj->setMetaData('wms_extent', implode(' ', array($geodata_node->minx, $geodata_node->miny, $geodata_node->maxx, $geodata_node->maxy)));
    $layerObj->setMetaData('gml_include_items', 'all');
    $classitem = $form_state['values']['field'];
    
    $ramp = new ColorRamp($form_state['values']['color1'], $form_state['values']['color2']);
    $colors = $ramp->get_colors($numclasses);
    
    for ($i = 1; $i <= $numclasses; $i++) {
      $classObj = ms_newClassObj($layerObj);
      // set name of class (required for legend graphic)
      $classObj->set('name', "{$class_lo[$i-1]} - {$class_lo[$i]}");
      $classObj->setExpression("([{$classitem}] <= {$class_lo[$i]})");
      
      // style of class
      $styleObj = ms_newStyleObj($classObj);
      $styleObj->set('size', 1);
      $styleObj->set('width', 1);
      
      // determine color for $i-th class (colorramp)
      //$styleObj->color->setRGB(round($i * 255/$numclasses), 0, 0);
      $styleObj->color->setRGB($colors[$i-1]['R'], $colors[$i-1]['G'], $colors[$i-1]['B']);
      
      $styleObj->outlinecolor->setRGB(0, 0, 0);
    }
    
    // save to same file (use path relative to this map file)
    $oMap->save(basename($file_name));
    
    // return to layer edit page
    $form_state['redirect'] = 'node/'. $form_state['values']['nid'] .'/edit';
  }
  
  //drupal_set_message(var_export($form_state['values'], TRUE));
}

function mapserver_layer_form(&$form_state, $node, $layer_nr) {
  global $_mappingkit_logger;
  require_once('mapscript.inc.php');
  
  //drupal_set_message('layer:'.var_export(array('form_state'=>$form_state,'node'=>$node, 'layer_nr'=>$layer_nr), TRUE));
  
  $map_path = variable_get('mapserver_map_path', DEFAULT_MAPSERVER_MAP_PATH);
  $file_name = mapserver_get_real_path($node->mapfile);
  if (file_exists($file_name)) {
    $oMap = ms_newMapObj($file_name);
    
    // read layer (0 = first layer)
    if (is_numeric($layer_nr)) {
      $layerObj = $oMap->getLayer($layer_nr);
    }
    else if ($layer_nr == 'add') {
      $layerObj = ms_newLayerObj($oMap);
    }
    else {
      $layerObj = NULL;
    }
    if (!$layerObj) {
      drupal_set_message(t('Invalid layer index'));
      return NULL;
    }
  }
  else {
    // TODO: Error: mapfile not found!
    $oMap = NULL;
  }
  $form['mapfile'] = array('#type' => 'hidden'
    , '#default_value' => $node->mapfile
    );
  $form['layer_nr'] = array('#type' => 'hidden'
    , '#default_value' => $layer_nr
    );
  
  $connectiontype_options = array(
    MS_INLINE => t('Inline')
    , MS_SHAPEFILE => t('Shapefile')
    , MS_TILED_SHAPEFILE => t('Tiled shapefile')
    , MS_SDE => t('SDE')
    , MS_OGR => t('OGR')
    //, MS_TILED_OGR => 'Tiled OGR'
    , MS_POSTGIS => t('PostGIS')
    , MS_WMS => t('WMS')
    , MS_ORACLESPATIAL => t('Oracle Spatial')
    , MS_WFS => t('WFS')
    , MS_GRATICULE => t('Graticule')
    , MS_MYGIS => t('MyGIS')
    );
  $layer_type_options = array(
    MS_LAYER_POINT => t('POINT'), MS_LAYER_LINE => t('LINE'), MS_LAYER_POLYGON => t('POLYGON') 
    , MS_LAYER_RASTER => t('RASTER'), MS_LAYER_ANNOTATION => t('ANNOTATION')
    , MS_LAYER_QUERY => t('QUERY')
    , MS_LAYER_CIRCLE => t('CIRCLE')
    , MS_LAYER_TILEINDEX => t('TILEINDEX')
    );
  global $unit_options;
  
  $metadata['wms_title'] = $layerObj->getMetaData('wms_title');
  $metadata['wms_abstract'] = $layerObj->getMetaData('wms_abstract');
  
  $form['identification'] = array('#type' => 'fieldset'
    , '#title' => t('Layer identification')
    , '#collapsible' => TRUE
    );
  $form['identification']['layer_name'] = array('#type' => 'textfield'
    , '#title' => t('Name')
    , '#default_value' => ($layerObj) ? $layerObj->name : ''
    , '#description' => t('Short name for this layer. Limit is 20 characters. This name is the link between the mapfile and web interfaces that refer to this name. They must be identical. The name should be unique, unless one layer replaces another at different scales. Use the GROUP option to associate layers with each other.')
    , '#required' => TRUE
    , '#maxlength' => 20
    );
  $form['identification']['layer_title'] = array('#type' => 'textfield'
    , '#title' => t('Title')
    , '#default_value' => $metadata['wms_title']
    , '#description' => t('Metadata information. Mandatory as layer title in the capabilities.')
    , '#required' => TRUE
    );
  $form['identification']['layer_abstract'] = array('#type' => 'textfield'
    , '#title' => t('Abstract')
    , '#default_value' => $metadata['wms_abstract']
    , '#description' => t('Metadata information. Used for layer abstract in the capabilities.')
    );
  
  $form['data_source'] = array('#type' => 'fieldset'
    , '#title' => t('Data source')
    , '#collapsible' => TRUE
    );
  $form['data_source']['status'] = array('#type' => 'select'
    , '#title' => t('Status')
    , '#options' => array(MS_ON => 'On', MS_OFF => 'Off', MS_DEFAULT => 'Default', MS_DELETE => 'Delete')
    , '#default_value' => $layerObj->status
    , '#description' => t('Sets the current status of the layer. Often modified by MapServer itself. Default turns the layer on permanently.')
    );
  $form['data_source']['layer_type'] = array('#type' => 'select'
    , '#title' => t('Layer type')
    , '#options' => $layer_type_options
    , '#default_value' => $layerObj->type
    , '#description' => t('Specifies how the data should be drawn. Need not be the same as the shapefile type. For example, a polygon shapefile may be drawn as a point layer, but a point shapefile may not be drawn as a polygon layer. Common sense rules. Annotation means that a label point will be calculated for the features, but the feature itself will not be drawn although a marker symbol can be optionally drawn. this allows for advanced labeling like numbered highway shields. Points are labeled at that point. Polygons are labeled first using a centroid, and if that doesn\'t fall in the polygon a scanline approach is used to guarantee the label falls within the feature. Lines are labeled at the middle of the longest arc in the visible portion of the line. Query only means the layer can be queried but not drawn.')
    );
  $form['data_source']['connectiontype'] = array('#type' => 'select'
    , '#title' => t('Connection type')
    , '#options' => $connectiontype_options
    , '#default_value' => $layerObj->connectiontype
    , '#description' => t('Type of connection. Default is local. See additional documentation for any other type.')
    );
  $form['data_source']['connection'] = array('#type' => 'textfield'
    , '#title' => t('Connection')
    , '#default_value' => $layerObj->connection
    , '#size' => 100
    , '#description' => t('Database connection string to retrieve remote data. A PostGIS connection string is basically a regular PostgreSQL connection string, it takes the form of "user=nobody password=****** dbname=dbname host=localhost port=5432"')
    , '#autocomplete_path' => 'mapserver/autocomplete/connection'
    );
  $form['data_source']['data'] = array('#type' => 'textfield'
    , '#title' => t('Data')
    , '#default_value' => $layerObj->data
    , '#size' => 100
    , '#description' => t('If this is a PostGIS layer, the parameter should be in the form of "&lt;columnname&gt; from &lt;tablename&gt;", where "columnname" is the name of the column containing the geometry objects and "tablename" is the name of the table from which the geometry data will be read.')
    );
  $projection = $layerObj->getProjection();
  $proj_parts = explode(':', $projection);
  $form['data_source']['srs'] = array('#type' => 'select'
    , '#title' => t('Projection')
    , '#options' => mappingkit_get_projections()
    , '#default_value' => $proj_parts[1] //variable_get('mapserver_default_projection', '4326')
    , '#description' => t('Projection for this layer')
    );
  $form['data_source']['feature_info'] = array('#type' => 'checkbox'
    , '#title' => t('Feature info')
    , '#default_value' => $layerObj->dump
    , '#description' => t('Switch to allow mapserver to return data in GML format. Usefull when used with WMS GetFeatureInfo operations. "false" by default.')
    );
  
  /*
  $form['features'] = array('#type' => 'fieldset'
    , '#title' => t('Features')
    , '#collapsible' => TRUE
    );
  $form['features']['count'] = array('#type' => 'markup'
    , '#value' => '<p>'. t('Number of features') .': '. $layerObj->maxfeatures .'</p>'
    );
  */
  
  // symbology
  $form['symbology'] = array('#type' => 'fieldset'
    , '#title' => t('Symbology')
    , '#collapsible' => TRUE
    );
  $form['symbology']['sizeunits'] = array('#type' => 'select'
    , '#title' => t('Unit')
    , '#options' => $unit_options
    , '#default_value' => $layerObj->sizeunits
    , '#description' => t('Sets the unit of CLASS object SIZE values (default is pixels). Useful for simulating buffering.')
    );
  $form['symbology']['classitem'] = array('#type' => 'textfield'
    , '#title' => t('Class item')
    , '#default_value' => $layerObj->classitem
    , '#description' => t('Item name in attribute table to use for class lookups.')
    );
  $form['symbology']['scale'] = array('#type' => 'fieldset'
    , '#title' => t('Scale')
    , '#collapsible' => TRUE
    , '#description' => t('Minimum and maximum denominators at this layer is drawn, or -1.')
    );
  $form['symbology']['scale']['minscale'] = array('#type' => 'textfield'
    , '#title' => t('Minimum scale')
    , '#default_value' => $layerObj->minscale
    //, '#description' => t('Minimum scale denominator at which this layer is drawn, or -1')
    , '#size' => 30, '#maxlength' => 15
    , '#prefix' => '<div class="form-inline">'
    );
  $form['symbology']['scale']['maxscale'] = array('#type' => 'textfield'
    , '#title' => t('Maximum scale')
    , '#default_value' => $layerObj->maxscale
    //, '#description' => t('Maximum scale denominator at which this layer is drawn, or -1.')
    , '#size' => 30, '#maxlength' => 15
    , '#suffix' => '</div><div style="clear: both"/>'
    );
  
  $form['symbology']['labelitem'] = array('#type' => 'textfield'
    , '#title' => t('Label item')
    , '#default_value' => $layerObj->labelitem
    , '#description' => t('Item name in attribute table to use for class annotation (i.e. labeling).')
    );
  $form['symbology']['labelscale'] = array('#type' => 'fieldset'
    , '#title' => t('Label scale')
    , '#collapsible' => TRUE
    , '#description' => t('Minimum and maximum denominators at the layer is labeled, or -1.')
    );
  $form['symbology']['labelscale']['labelminscale'] = array('#type' => 'textfield'
    , '#title' => t('Minimum label scale')
    , '#default_value' => $layerObj->labelminscale
    //, '#description' => t('Minimum scale denominator at which the layer is labeled, or -1.')
    , '#size' => 30, '#maxlength' => 15
    , '#prefix' => '<div class="form-inline">'
    );
  $form['symbology']['labelscale']['labelmaxscale'] = array('#type' => 'textfield'
    , '#title' => t('Maximum label scale')
    , '#default_value' => $layerObj->labelmaxscale
    //, '#description' => t('Maximum scale denominator at which the layer is labeled, or -1.')
    , '#size' => 30, '#maxlength' => 15
    , '#suffix' => '</div><div style="clear: both"/>'
    );
  $form['symbology']['labelsizeitem'] = array('#type' => 'textfield'
    , '#title' => t('Label size item')
    , '#default_value' => $layerObj->labelsizeitem
    , '#description' => t('Item name in attribute table to use for class annotation sizes. Values should be in pixels.')
    );
  $form['symbology']['labelangleitem'] = array('#type' => 'textfield'
    , '#title' => t('Label angle item')
    , '#default_value' => $layerObj->labelangleitem
    , '#description' => t('Item name in attribute table to use for class annotation angles. Values should be in degrees.')
    );
  
  $header = array(t('#'), t('Name')
    , t('Type'), t('Expression'), t('Symbol'), t('# styles'));
  for ($i = 0; $i < $layerObj->numclasses; $i++) {
    $classObj = $layerObj->getClass($i);
    
    if ($classObj->numstyles == 1) {
      $styleObj = $classObj->getStyle(0);
      
      /*
      Members:
      
        int           symbol
        string        symbolname
        int           size
        int           minsize 
        int           maxsize 
        int           offsetx
        int           offsety
        colorObj      color
        colorObj      backgroundcolor
        colorObj      outlinecolor
        int           width
        int           minwidth
        int           maxwidth
        double        angle
        string        angleitem  
        string        sizeitem
        double        minvalue
        double        maxvalue
        string        rangeitem
      */
      $symbol = $styleObj->color->red .','. $styleObj->color->green .','. $styleObj->color->blue;
    }
    else {
      $symbol = NULL;
    }
    $expression = $classObj->getExpression();
    
    $rows[] = array(l($i,"node/{$node->nid}/layer/{$layer_nr}/class/{$i}/edit"), $classObj->name
      , $classObj->type, $expression, $symbol, $classObj->numstyles);
  }
  $rows[] = array('', l('add', "node/{$node->nid}/layer/{$layer_nr}/class/add/edit")
    , '', '', '', '');
  $form['symbology']['classes'] = array('#value' => theme('table', $header, $rows));
  
  $form['submit'] = array('#type' => 'submit'
    , '#default_value' => t('Submit')
    );
  if (is_numeric($layer_nr)) {
    $form['delete'] = array('#type' => 'submit'
      , '#default_value' => t('Delete')
      , '#submit' => array('mapserver_layer_delete_submit')
      );
  }
  
  $form['#validate'][] = 'mapserver_layer_validate';
  $form['#submit'][] = 'mapserver_layer_submit';
  
  return $form;
}

function mapserver_layer_validate(&$form, &$form_state) {
  //form_set_error('', 'validate: Not yet implemented: '. var_export($form_state['values'], TRUE));
  if ($form_state['values']['op'] == t('Classify')) {
    
    //TODO: strange use of unknown table for classify
    form_set_error('', 'Classify is not yet implemented.');
    return;
    
    db_set_active('geodata');
    $result = db_query('select lki_code, count(*) from gbkn_l group by lki_code order by lki_code');
    $rows = array();
    while ($row = db_fetch_object($result)) {
      $rows[] = $row;
    }
    db_set_active();
    
    $numclasses = $form_state['values']['numclasses'];
    $numvalues = count($rows);
    $valuesperclass = $numvalues/$numclasses;
    $classes = array();
    for ($i = 0; $i < $numclasses; $i++) {
      $classes[] = array();
    }
    
    drupal_set_message('values:'.var_export($rows, TRUE));
    return;
  }
}

function mapserver_layer_submit(&$form, &$form_state) {
  
  //drupal_set_message('values:'.var_export($form_state['values'], TRUE));
  
  $map_path = variable_get('mapserver_map_path', DEFAULT_MAPSERVER_MAP_PATH);
  $file_name = mapserver_get_real_path($form_state['values']['mapfile']);

  if (file_exists($file_name)) {
    $oMap = ms_newMapObj($file_name);
    
    // read layer (0 = first layer)
    $layer_nr = $form_state['values']['layer_nr'];
    if (is_numeric($layer_nr)) {
      $layerObj = $oMap->getLayer($layer_nr);
    }
    else if ($layer_nr == 'add') {
      $layerObj = ms_newLayerObj($oMap);
    }
    else {
      $layerObj = NULL;
    }
    if (!$layerObj) {
      drupal_set_message(t('Invalid layer index'));
      return NULL;
    }
  }
  
  // todo: on delete redirect to confirm page
  if ($form_state['values']['op'] == t('Delete')) {
    drupal_set_message(t('Delete not yet implemented'), 'error');
    
    // goto higher level to continue editing map file
    $form_state['redirect'] = 'node/'. arg(1) .'/edit';
    return;
  }
  
  //drupal_set_message('values: '.var_export($form_state['values'], TRUE));
  //drupal_set_message('filename '.$file_name);
  /*
    Required for each LAYER in a WMS configuration:

    * Layer NAME
    * Layer PROJECTION
    * Layer METADATA
          o wms_title
          o wms_srs (optional since the layers inherit the map's SRS value)
    * Layer STATUS
          o Layers set to STATUS DEFAULT will always be sent to the client.
          o Layers set to STATUS ON or STATUS OFF can be requested by the client.
    * Layer TEMPLATE (required for GetFeatureInfo requests)
    * Layer DUMP TRUE (only required for GetFeatureInfo GML requests)
  */
  
  if (-1 == $layerObj->set('name', $form_state['values']['layer_name'])) {
    drupal_set_message(t('Failed to set name'));
  }
  if (-1 == $layerObj->set('status', $form_state['values']['status'])) {
    drupal_set_message(t('Failed to set status'));
  }
  if (-1 == $layerObj->setProjection('init=epsg:'. $form_state['values']['srs'], FALSE)) {
    drupal_set_message(t('Failed to set srs'));
  }
  if (-1 == $layerObj->set('connection', $form_state['values']['connection'])) {
    drupal_set_message(t('Failed to set connection'));
  }
  if (-1 == $layerObj->set('connectiontype', $form_state['values']['connectiontype'])) {
    drupal_set_message(t('Failed to set connectiontype'));
  }
  if (-1 == $layerObj->set('data', $form_state['values']['data'])) {
    drupal_set_message(t('Failed to set data'));
  }
  if (-1 == $layerObj->set('type', $form_state['values']['layer_type'])) {
    drupal_set_message(t('Failed to set type'));
  }
  if (-1 == $layerObj->set('dump', $form_state['values']['feature_info'])) {
    drupal_set_message(t('Failed to set feature info flag'));
  }
  if ($form_state['values']['feature_info'] == 1) {
    $layerObj->set('template', 'blank.html');
    
    //$layerObj->setMetaData('');
  }
  
  //drupal_set_message(var_export($form_state['values'], TRUE));
  $layerObj->set('sizeunits', $form_state['values']['sizeunits']);
  $layerObj->set('classitem', stringOrNull($form_state['values']['classitem']));
  $layerObj->set('minscale', floatval($form_state['values']['minscale']));
  $layerObj->set('maxscale', floatval($form_state['values']['maxscale']));
  
  $layerObj->set('labelminscale', floatval($form_state['values']['labelminscale']));
  $layerObj->set('labelmaxscale', floatval($form_state['values']['labelmaxscale']));
  $layerObj->set('labelitem', stringOrNull($form_state['values']['labelitem']));
  if ($form_state['values']['labelsizeitem'] != '') {
    $layerObj->set('labelsizeitem', stringOrNull($form_state['values']['labelsizeitem']));
  }
  if ($form_state['values']['labelangleitem'] != '') {
    $layerObj->set('labelangleitem', stringOrNull($form_state['values']['labelangleitem']));
  }
  
  setMetaData($layerObj, 'wms_title', $form_state['values']['layer_title']);
  setMetaData($layerObj, 'wms_abstract', $form_state['values']['layer_abstract']);
  $layerObj->setMetaData('wms_srs', 'EPSG:'. $form_state['values']['srs']);
  $layerObj->setMetaData('gml_include_items', 'all');
  
  // save to same file (use path relative to this map file)
  $oMap->save(basename($file_name));
  
  // goto higher level to continue editing map file
  $form_state['redirect'] = 'node/'. arg(1) .'/edit';
}

function mapserver_class_edit($node, $layer_nr, $class_nr) {
  if ($class_nr == 'add') {
    drupal_set_title(t('Add class to layer %lnr of %title', array('%lnr' => $layer_nr, '%snr' => $class_nr, '%title' => $node->title)));
  }
  else {
    drupal_set_title(t('Edit class %snr of layer %lnr of %title', array('%lnr' => $layer_nr, '%snr' => $class_nr, '%title' => $node->title)));
  }
  return drupal_get_form('mapserver_class_form', $node, $layer_nr, $class_nr);
}

function mapserver_class_form(&$form_state, $node, $layer_nr, $class_nr) {
  global $_mappingkit_logger;
  require_once('mapscript.inc.php');
  
  $map_path = variable_get('mapserver_map_path', DEFAULT_MAPSERVER_MAP_PATH);
  $file_name = mapserver_get_real_path($node->mapfile);
  if (file_exists($file_name)) {
    $oMap = ms_newMapObj($file_name);
    
    // read layer (0 = first layer)
    if (is_numeric($layer_nr)) {
      $layerObj = $oMap->getLayer($layer_nr);
    }
    else {
      $layerObj = NULL;
    }
    if (!$layerObj) {
      drupal_set_message(t('Invalid layer index'));
      return NULL;
    }
    
    if (is_numeric($class_nr)) {
      $classObj = $layerObj->getClass($class_nr);
    }
    else if ($class_nr == 'add') {
      $classObj = ms_newClassObj($layerObj);
    }
    if (!$classObj) {
      drupal_set_message(t('Invalid class index'));
      return NULL;
    }
  }
  
  $form = array();
  
  $form['mapfile_path'] = array('#type' => 'hidden', '#value' => $file_name);
  $form['nid'] = array('#type' => 'hidden', '#value' => $node->nid);
  $form['layer'] = array('#type' => 'hidden', '#value' => $layer_nr);
  $form['class'] = array('#type' => 'hidden', '#value' => $class_nr);
  
  $form['general'] = array('#type' => 'fieldset'
    , '#title' => t('General')
    , '#collapsible' => TRUE
    );
  $form['general']['name'] = array('#type' => 'textfield'
    , '#title' => t('Name')
    , '#default_value' => $classObj->name
    , '#description' => t("Name to use in legends for this class. If not set class won't show up in legend.")
    );
  $form['general']['title'] = array('#type' => 'textfield'
    , '#title' => t('Title')
    , '#default_value' => $classObj->title
    );
  $form['general']['expression'] = array('#type' => 'textfield'
    , '#title' => t('Expression')
    , '#default_value' => $classObj->getExpression()
    , '#description' => t('Four types of expressions are supported to define class membership: string comparisons, regular expressions using <em>/regex/</em>, simple logical expressions, and string functions. See !reference for more information.', array('!reference' => l('Mapserver documentation', 'http://mapserver.gis.umn.edu/docs/reference/mapfile/class')))
    );
  $form['general']['minscale'] = array('#type' => 'textfield'
    , '#title' => t('Minimum scale')
    , '#default_value' => $classObj->minscale
    , '#description' => t('Maximum scale at which this CLASS is drawn.')
    );
  $form['general']['maxscale'] = array('#type' => 'textfield'
    , '#title' => t('Maximum scale')
    , '#default_value' => $classObj->maxscale
    , '#description' => t('Minimum scale at which this CLASS is drawn.')
    );
  
  $graphic_path = realpath(file_directory_path()).'/temp.png';
  $imageObj = $classObj->createLegendIcon(64, 64);
  $imageObj->saveImage($graphic_path, $oMap);
  $imageObj->free();
  
  $form['general']['image'] = array('#type' => 'markup'
    , '#value' => '<img src="'. file_directory_path() .'/temp.png'. '"/>'
    , '#weight' => 100
    );
  
  $form['styles'] = array('#type' => 'fieldset'
    , '#title' => t('Styles')
    , '#collapsible' => TRUE
    );
  $form['styles']['number'] = array('#type' => 'markup'
    , '#value' => '<p>'. t('Number of styles') .': '. $classObj->numstyles .'</p>'
    , '#weight' => -1
    );
  
  for ($i = 0; $i < $classObj->numstyles; $i++) {
    $styleObj = $classObj->getStyle($i);
    
    $_mappingkit_logger->debug('style #' . $i, $styleObj);
    
    // symbolname
    $form['styles']['symbolname'] = array('#type' => 'textfield'
      , '#title' => t('Symbol name')
      , '#default_value' => $styleObj->symbolname
      , '#size' => 30
      );
    // size
    $form['styles']['size'] = array('#type' => 'textfield'
      , '#title' => t('Size')
      , '#default_value' => $styleObj->size
      , '#size' => 10
      );
    // color
    $form['styles']['color'] = array('#type' => 'textfield'
      , '#title' => t('Color')
      , '#theme' => 'color'
      , '#default_value' => mapserver_color2str($styleObj->color)
      , '#size' => 10
      );
    if ($styleObj->backgroundcolor->red != -1) {
      // backgroundcolor
      $form['styles']['background-color'] = array('#type' => 'textfield'
        , '#title' => t('Background color')
        , '#theme' => 'color'
        , '#default_value' => mapserver_color2str($styleObj->backgroundcolor)
        , '#size' => 10
        );
    }
    if ($styleObj->outlinecolor->red != -1) {
      // backgroundcolor
      $form['styles']['outline-color'] = array('#type' => 'textfield'
        , '#title' => t('Outline color')
        , '#theme' => 'color'
        , '#default_value' => mapserver_color2str($styleObj->outlinecolor)
        , '#size' => 10
        );
    }
  }
  
  $_mappingkit_logger->debug('label', $classObj->label);
  /*
  $labelObj = $classObj->label;
  $form['label'] = array('#type' => 'fieldset'
    , '#title' => t('Label')
    , '#collapsible' => TRUE
    );
  $form['label']['object'] = array('#type' => 'markup'
    , '#value' => '<p>'. var_export($labelObj, TRUE) .'</p>'
    );
  $fields = array('font', 'type');
  foreach ($fields as $key) {
    $form['label'][$key] = array('#type' => 'textfield'
      , '#title' => $key
      , '#default_value' => $labelObj->$key
      );
  }
  */
  
  $form['submit'] = array('#type' => 'submit'
    , '#default_value' => t('Submit')
    );
  
  if (is_numeric($class_nr)) {
    $form['delete'] = array('#type' => 'submit'
      , '#default_value' => t('Delete')
      , '#submit' => array('mapserver_class_delete_submit')
      );
  }
  
  $form['#validate'][] = 'mapserver_class_validate';
  $form['#submit'][] = 'mapserver_class_submit';
  
  return $form;
}

function mapserver_class_validate($form, &$form_state) {
  //drupal_set_message('mapserver_class_validate');
}

function mapserver_class_submit($form, &$form_state) {
  //drupal_set_message('mapserver_class_submit ');
  
  $form_values = $form_state['values'];
  $file_name = $form_values['mapfile_path'];
  $oMap = ms_newMapObj($file_name);
  if ($oMap) {
    $layerObj = $oMap->getLayer($form_values['layer']);
    if ($layerObj) {
      if ($form_values['class'] == 'add') {
        $classObj = ms_newClassObj($layerObj);
      }
      else {
        $classObj = $layerObj->getClass($form_values['class']);
      }
      if ($classObj) {
        //drupal_set_message('class ok');
        
        $classObj->set('name', stringOrNull($form_values['name']));
        $classObj->set('title', stringOrNull($form_values['title']));
        $classObj->setExpression(stringOrNull($form_values['expression']));
        if ($form_values['minscale'] >= 0) {
          $classObj->set('minscale', $form_values['minscale']);
        }
        if ($form_values['maxscale'] >= 0) {
          $classObj->set('maxscale', $form_values['maxscale']);
        }
        
        // Save style(s)
        $styleObj = $classObj->getStyle(0);
        $color = array_map('hexdec', str_split($form_values['color'], 2));
        $styleObj->color->setRGB($color[0], $color[1], $color[2]);
        if (isset($form_values['background-color'])) {
          $color = array_map('hexdec', str_split($form_values['background-color'], 2));
          $styleObj->backgroundcolor->setRGB($color[0], $color[1], $color[2]);
        }
        if (isset($form_values['outline-color'])) {
          $color = array_map('hexdec', str_split($form_values['outline-color'], 2));
          $styleObj->outlinecolor->setRGB($color[0], $color[1], $color[2]);
        }
        $styleObj->set('size', $form_values['size']);
        
        // save to same file (use path relative to this map file)
        $oMap->save(basename($file_name));
        
        // return to layer edit page
        $form_state['redirect'] = 'node/'. $form_state['values']['nid'] .'/layer/'. $form_state['values']['layer'] .'/edit';
      }
      else {
        drupal_set_message(t('Failed to get class'), 'error');
      }
    }
    else {
      drupal_set_message(t('Failed to get layer'), 'error');
    }
  }
  else {
    drupal_set_message(t('Failed to open map file'), 'error');
  }
}

function mapserver_layer_delete_submit(&$form, &$form_state) {
  $form_state['redirect'] = 'node/'. arg(1) .'/layer/'. arg(3) .'/delete';

  //drupal_set_message('mapserver_layer_delete_submit redirects to '. $form_state['redirect']);
}

function mapserver_layer_delete_form(&$form_state, $node, $layer) {
  //drupal_set_message('mapserver_layer_delete_form '.var_export(array($node, $layer, $class), TRUE));
  
  $form = array();
  $form['nid'] = array('#type' => 'hidden', '#value' => $node->nid);
  $form['layer'] = array('#type' => 'hidden', '#value' => $layer);
  $form['mapfile_path'] = array('#type' => 'hidden', '#value' => $node->mapfile_path);
  
  return confirm_form($form,
    t('Are you sure you want to delete the layer %nr?', array('%nr' => $layer)),
    'node/'. $node->nid .'/edit',
    t('This cannot be undone.'),
    t('Delete'), t('Cancel')
  );
}

function mapserver_layer_delete_form_submit($form, &$form_state) {
  //drupal_set_message('mapserver_layer_delete_form_submit not implemented');
  
  $file_name = realpath($form_state['values']['mapfile_path']);
  $oMap = ms_newMapObj($file_name);
  if ($oMap) {
    $layerObj = $oMap->getLayer((int)$form_state['values']['layer']);
    if ($layerObj) {
      // $oMap->removeLayer does not exists, we must set the
      // layer status to MS_DELETE
      if (-1 == $layerObj->set('status', MS_DELETE)) {
        drupal_set_message(t('Failed to delete layer'), 'error');
      }
      else {
        drupal_set_message(t('Layer was removed.'));
        
        // save to same file (use path relative to this map file)
        $oMap->save(basename($file_name));
      }
    }
  }
  
  $form_state['redirect'] = 'node/'. $form_state['values']['nid'] .'/edit';
}

function mapserver_class_delete_submit(&$form, &$form_state) {
  $form_state['redirect'] = 'node/'. arg(1) .'/layer/'. arg(3) .'/class/'. arg(5) .'/delete';

  //drupal_set_message('mapserver_class_delete_submit redirects to '. $form_state['redirect']);
}

function mapserver_class_delete_form(&$form_state, $node, $layer, $class) {
  //drupal_set_message('mapserver_class_delete_form '.var_export(array($node, $layer, $class), TRUE));
  
  $form = array();
  $form['nid'] = array('#type' => 'hidden', '#value' => $node->nid);
  $form['layer'] = array('#type' => 'hidden', '#value' => $layer);
  $form['class'] = array('#type' => 'hidden', '#value' => $class);
  $form['mapfile_path'] = array('#type' => 'hidden', '#value' => $node->mapfile_path);
  
  return confirm_form($form,
    t('Are you sure you want to delete the class %nr?', array('%nr' => $class)),
    'node/'. $node->nid .'/layer/'. $layer .'/edit',
    t('This cannot be undone.'),
    t('Delete'), t('Cancel')
  );
}

function mapserver_class_delete_form_submit($form, &$form_state) {
  //drupal_set_message('mapserver_class_delete_form_submit not implemented');
  
  $file_name = realpath($form_state['values']['mapfile_path']);
  $oMap = ms_newMapObj($file_name);
  if ($oMap) {
    $layerObj = $oMap->getLayer($form_state['values']['layer']);
    if ($layerObj) {
      if (NULL == $layerObj->removeClass($form_state['values']['class'])) {
        drupal_set_message(t('Failed to remove class'), 'error');
      }
      else {
        drupal_set_message(t('Class was removed. Note that the remaining classes will be renumbered by this operation.'));
        
        // save to same file (use path relative to this map file)
        $oMap->save(basename($file_name));
      }
    }
  }
  
  $form_state['redirect'] = 'node/'. $form_state['values']['nid'] .'/layer/'. $form_state['values']['layer'] .'/edit';
}

function mapserver_wms($node, $params = NULL) {
  global $_mappingkit_logger;
  
  if (!isset($params)) {
    $params = $_GET;
  }
  unset($params['q']);
  
  // convert params to lowercase
  if (count($params) > 0) {
    $params = array_combine(array_map('strtolower', array_keys($params)), array_values($params));
  }
  
  $_mappingkit_logger->debug('params:', $params);
  
  // todo: add param map=path/to/mapfile for this node
  $file_name = mapserver_get_real_path($node->mapfile);
  $params['map'] = $file_name;
  
  $params['service'] = 'WMS';
  if (!isset($params['version'])) {
    $params['version'] = '1.1.1';
  }
  
  $cgi_url = variable_get('mappingkit_mapserver_url', DEFAULT_MAPPINGKIT_MAPSERVER_URL);
  $cgi_url .= (strpos($cgi_url, '?') !== FALSE ? '&' : '?')
    . drupal_query_string_encode($params);
  
  $_mappingkit_logger->debug('cgi_url:', $cgi_url);
  
  $response = drupal_http_request($cgi_url);
  if (isset($response->headers)) {
    foreach ($response->headers as $key => $value) {
      drupal_set_header("$key: $value");
    }
  }
  
  // return capabilities as text/xml to be able to view it in browser
  if (strcasecmp($params['request'],'GetCapabilities') == 0) {
    drupal_set_header("Content-Type: text/xml");
  }
  
  echo $response->data;
  //return;//exit();
  
  /*
  header("HTTP/1.1 501 Not implemented");
  echo 'WMS is not yet implemented<br/>';
  echo var_export(array($node, $_GET), TRUE);
  // TODO: http response code (501=not implemented) or drupal_not_found()?
  exit();
  */
}

function mapserver_image($node, $format = '') {
  require_once('mapscript.inc.php');
  
  if ($format == '') {
    $format = 'png';
  }
  $map_path = variable_get('mapserver_map_path', DEFAULT_MAPSERVER_MAP_PATH);
  if (!isset($node->mapfile)) {
    // generate unique mapfile name for new nodes
    $node->mapfile = uniqid('', TRUE) . '.map';
  }
  $file_name = mapserver_get_real_path($node->mapfile);
  
  // TODO: existing file -> read first
  if (($file_name) && file_exists($file_name)) {
    
    // cgi manier
    //$params = array('request' => 'GetCapabilities'
    //  , 'version' => '1.0.7');
    //return mapserver_wms($node, $params);
    
    // mapscript manier
    $oMap = ms_newMapObj($file_name);
    drupal_set_header("Content-Type: ". $oMap->outputformat->mimetype);
    // TODO: http response code
    //ini_set("max_execution_time", 120); // todo: check present value
    $imageObj = $oMap->draw();
    // with caching?
    return $imageObj->saveImage('');
    
    // without caching?
    $oMap = NULL;
    $imageObj = NULL;
    exit();
  }
  echo var_export(array($node, $format), TRUE);
}

class ColorRamp {
  
  public $color1;
  public $color2;
  
  function __construct($color1, $color2) {
    if (is_string($color1)) {
      // convert #rrggbb to array
      if ($color1[0] == '#') {
        $color1 = substr($color1, 1);
      }
      $color1 = array_map('hexdec', str_split($color1, 2));
    }
    if (is_string($color2)) {
      // convert #rrggbb to array
      if ($color2[0] == '#') {
        $color2 = substr($color2, 1);
      }
      $color2 = array_map('hexdec', str_split($color2, 2));
    }
    $this->color1 = $color1;
    $this->color2 = $color2;
  }
  
  function get_colors($count) {
    //
    $hsv = array();
    $hsv[0] = RGB_TO_HSV($this->color1[0], $this->color1[1], $this->color1[2]);
    $hsv[$count - 1] = RGB_TO_HSV($this->color2[0], $this->color2[1], $this->color2[2]);
    //print 'hsv' . var_export($hsv, TRUE) . "\n";
    // interpolate hue over $count classes
    $h_step = ($hsv[$count - 1]['H'] - $hsv[0]['H'])/($count - 1);
    $s_step = ($hsv[$count - 1]['S'] - $hsv[0]['S'])/($count - 1);
    $v_step = ($hsv[$count - 1]['V'] - $hsv[0]['V'])/($count - 1);
    for ($i = 1; $i < $count - 1; $i++) {
      $hsv[$i] = array('H' => $hsv[0]['H'] + $h_step * $i
        , 'S' => $hsv[0]['S'] + $s_step * $i
        , 'V' => $hsv[0]['V'] + $v_step * $i
        );
    }
    ksort($hsv);
    $rgbs = array();
    foreach ($hsv as $hsv_color) {
      $rgb = HSV_TO_RGB($hsv_color['H'], $hsv_color['S'], $hsv_color['V']);
      $rgbs[] = array('R' => (int)$rgb['R'], 'G' => (int)$rgb['G'], 'B' => (int)$rgb['B']);
    }
    return $rgbs;
  }
}

function RGB_TO_HSV($R, $G = NULL, $B = NULL) { // RGB Values:Number 0-255
  if (is_array($R)) {
    $G = $R['G'];
    $B = $R['B'];
    $R = $R['R'];
  }
  // HSV Results: Number 0-1
  $HSL = array();
  
  $var_R = ((float)$R / 255);
  $var_G = ((float)$G / 255);
  $var_B = ((float)$B / 255);
  
  $var_Min = min($var_R, $var_G, $var_B);
  $var_Max = max($var_R, $var_G, $var_B);
  $del_Max = $var_Max - $var_Min;
  
  $V = $var_Max;
  
  if ($del_Max == 0)
  {
  $H = 0;
  $S = 0;
  }
  else
  {
  $S = $del_Max / $var_Max;
  
  $del_R = ( ( ( $max - $var_R ) / 6 ) + ( $del_Max / 2 ) ) / $del_Max;
  $del_G = ( ( ( $max - $var_G ) / 6 ) + ( $del_Max / 2 ) ) / $del_Max;
  $del_B = ( ( ( $max - $var_B ) / 6 ) + ( $del_Max / 2 ) ) / $del_Max;
  
  if ($var_R == $var_Max) $H = $del_B - $del_G;
  else if ($var_G == $var_Max) $H = ( 1 / 3 ) + $del_R - $del_B;
  else if ($var_B == $var_Max) $H = ( 2 / 3 ) + $del_G - $del_R;
  
  if (H<0) $H++;
  if (H>1) $H--;
  }
  
  $HSL['H'] = $H;
  $HSL['S'] = $S;
  $HSL['V'] = $V;
  
  return $HSL;
}

/**
 * @param $H
 *   Value of hue, or associative array for hue ('H'), value ('V') 
 *   and saturation ('S').
 * @return Associative array with elements R,G,B for color values in RGB 
 *   color model.
 */
function HSV_TO_RGB ($H, $S = NULL, $V = NULL) {// HSV Values:Number 0-1
  // RGB Results:Number 0-255
  if (is_array($H)) {
    $S = $H['S'];
    $V = $H['V'];
    $H = $H['H'];
  }
  $RGB = array();
  
  if ($S == 0)
  {
  $R = $G = $B = $V * 255;
  }
  else
  {
  $var_H = $H * 6;
  $var_i = floor( $var_H );
  $var_1 = $V * ( 1 - $S );
  $var_2 = $V * ( 1 - $S * ( $var_H - $var_i ) );
  $var_3 = $V * ( 1 - $S * (1 - ( $var_H - $var_i ) ) );
  
  if ($var_i == 0) { $var_R = $V ; $var_G = $var_3 ; $var_B = $var_1 ; }
  else if ($var_i == 1) { $var_R = $var_2 ; $var_G = $V ; $var_B = $var_1 ; }
  else if ($var_i == 2) { $var_R = $var_1 ; $var_G = $V ; $var_B = $var_3 ; }
  else if ($var_i == 3) { $var_R = $var_1 ; $var_G = $var_2 ; $var_B = $V ; }
  else if ($var_i == 4) { $var_R = $var_3 ; $var_G = $var_1 ; $var_B = $V ; }
  else { $var_R = $V ; $var_G = $var_1 ; $var_B = $var_2 ; }
  
  $R = $var_R * 255;
  $G = $var_G * 255;
  $B = $var_B * 255;
  }
  
  $RGB['R'] = $R;
  $RGB['G'] = $G;
  $RGB['B'] = $B;
  
  return $RGB;
}

function mapserver_color2str($color) {
  return str_pad(dechex($color->red), 2, '0', STR_PAD_LEFT)
    . str_pad(dechex($color->green), 2, '0', STR_PAD_LEFT)
    . str_pad(dechex($color->blue), 2, '0', STR_PAD_LEFT);
}


