
/**
 * @requires OpenLayers/Util.js
 * @requires OpenLayers.Events.js
 */

/**
 * Class: MappingKit.Legend
 */
MappingKit.Legend = OpenLayers.Class({
    
    /**
     * APIProperty: div
     * {DOMElement} The element that contains the legend
     */
    div: null,
    
    /**
     * APIProperty: map
     * {DOMElement} The map object
     */
    map: null,
    
    /**
     * Constructor: MappingKit.Legend
     * 
     * Parameters:
     * map - {Object}
     * div - {string}
     * options - {Object}
     */
    initialize: function(map, div, options) {
        // set defaults
        
        // now override default options
        OpenLayers.Util.extend(this, options);
        
        // set div
        this.div = OpenLayers.Util.getElement(div);
        
        // link to map
        if(!(map instanceof OpenLayers.Map)) {
            map = MappingKit.getMap(map);
        }
        this.setMap(map);
    },
    
    /** 
     * Method: setMap
     *
     * Properties:
     * map - {<OpenLayers.Map>} 
     */
    setMap: function(map) {
        this.map = map;
        
        this.map.events.on({
            'addlayer': this.redraw,
            'changelayer': this.redraw,
            'removelayer': this.redraw,
            'changebaselayer': this.redraw,
            scope: this
        });
    },
    
    redraw: function() {
        if (!this.div) {
            return;
        }
      
        var layers = this.map.layers;

        for (var i = 0; i < layers.length; i++) {
            var layer = layers[i];
            var known_layer = false;
            
            if (layer.displayInLegend && layer.visibility && layer.legend) {
                
              $('p', this.div).each(function(index, domEle){
                  if ($(domEle).text() == layer.name) {
                    // mark to show legend
                    $(domEle).parent().addClass('show-legend');
                    // it's a known layer
                    known_layer = true;
                    return;
                  }
              });
              
              if (!known_layer) {
                // add legend element for new layer
                var layerElem = document.createElement('div');
                layerElem.className = 'show-legend';
                
                var titleElem = document.createElement('p');
                titleElem.innerHTML = layer.name;
                titleElem.className = 'legend-title';
                layerElem.appendChild(titleElem);
                
                var imgElem = document.createElement('img');
                imgElem.src = layer.legend.href;
                imgElem.title = layer.name;
                layerElem.appendChild(imgElem);
                
                this.div.appendChild(layerElem);
              }
            }
        }
        
        // hide div's without show-legend
        $('div:not(.show-legend)', this.div).hide();
        
        // remove show-legend classnames
        $('div.show-legend', this.div).show().removeClass('show-legend');
        
        return this.div;
    },
    
    CLASS_NAME: "MappingKit.Legend"
});

