
OpenLayers.Control.Ruler = OpenLayers.Class(OpenLayers.Control, {
  
  /**
   * Div for output of measure length.
   */
  outputDiv: null,
  
  /**
   * Style to draw ruler in.
   */
  rulerStyleMap: null,
  
  /**
   * Measure controls.
   */
  measureControls: null,
  
  initialize: function(options) {
    options.type = OpenLayers.Control.TYPE_TOOL;
    
    this.outputDiv = document.getElementById(options.div);
    //this.rulerStyleMap = new OpenLayers.StyleMap({strokeWidth: 2, strokeColor: '#EE9900'});
    
    OpenLayers.Control.prototype.initialize.apply(this, [options]);
  },
  
  setMap: function(map) {
    OpenLayers.Control.prototype.setMap.apply(this, arguments);
    
    // style the sketch fancy
    var sketchSymbolizers = {
      "Point": {
        pointRadius: 4,
        graphicName: "square",
        fillColor: "white",
        fillOpacity: 1,
        strokeWidth: 1,
        strokeOpacity: 1,
        strokeColor: "#333333"
      },
      "Line": {
        strokeWidth: 3,
        strokeOpacity: 1,
        strokeColor: "#666666",
        strokeDashstyle: "dash"
      },
      "Polygon": {
        strokeWidth: 2,
        strokeOpacity: 1,
        strokeColor: "#666666",
        fillColor: "white",
        fillOpacity: 0.3
      }
    };
    var style = new OpenLayers.Style();
    style.addRules([
        new OpenLayers.Rule({symbolizer: sketchSymbolizers})
    ]);
    var styleMap = new OpenLayers.StyleMap({"default": style});
    
    this.measureControls = {
      line: new OpenLayers.Control.Measure(
        OpenLayers.Handler.Path, {
          persist: true,
          handlerOptions: {
            layerOptions: {styleMap: styleMap}
          }
        }
      ) /*,
      polygon: new OpenLayers.Control.Measure(
        OpenLayers.Handler.Polygon, {
          persist: true,
          handlerOptions: {
            layerOptions: {styleMap: styleMap}
          }
        }
      )*/
    };
    this.measureControls.line.events.register('measure', this, this.handleMeasurements);
    this.measureControls.line.events.register('measurepartial', this, this.handleMeasurements);
    this.map.addControl(this.measureControls.line);
  },
  
  activate: function() {
    //alert('activate');
    this.measureControls.line.activate();
    
    //this.layer.setVisibility(true);
    //this.map.events.register('mousemove', this, this.redraw);
    return OpenLayers.Control.prototype.activate.apply(this);
  },
  
  deactivate: function() {
    //alert('deactivate');
    this.measureControls.line.deactivate();
    
    //this.map.events.unregister('mousemove', this, this.redraw);
    //this.layer.setVisibility(false);
    //this.layer.destroyFeatures();
    this.outputDiv.innerHTML = '';
    return OpenLayers.Control.prototype.deactivate.apply(this);
  },
  
  /**
   * Method: draw
   * Called when the control is added to the map.
   */
  draw: function() {
    OpenLayers.Control.prototype.draw.apply(this, arguments);
    
    // add drawing layer?
    //this.layer = new OpenLayers.Layer.Vector('_Ruler_', {'displayInLayerSwitcher': true, 'visibility': true, 'styleMap': this.rulerStyleMap});
    //this.map.addLayer(this.layer);
  },
  
  redraw: function(evt) {
    var dist = this.getDistance();
    if (dist) {
      this.outputDiv.innerHTML = Drupal.t('Distance: ') + dist;
    }
  },
  
  handleMeasurements: function(event) {
    var geometry = event.geometry;
    var units = event.units;
    var order = event.order;
    var measure = event.measure;
    var out = '';
    if (order == 1) {
      out += Drupal.t('measure: ') + measure.toFixed(3) + " " + units;
    } else {
      out += Drupal.t('measure: ') + measure.toFixed(3) + " " + units + "<sup>2</" + "sup>";
    }
    this.outputDiv.innerHTML = out;
  },
  
  /**
   * Round value to a number of significant digits.
   */
  round_sig: function(val, sig) {
    if (val == 0) {
      return 0;
    }
    var sign = val > 0 ? 1 : -1;
    val = Math.abs(val);
    var exp = Math.floor(Math.log(val)/Math.LN10 + 1);
    var powers = [1, 10, 100, 1000, 10000, 100000, 1000000, 10000000, 100000000, 1000000000, 10000000000];
    var pow10sig = powers[sig];//Math.pow(10, sig);
    var pow10exp = powers[exp];//Math.pow(10, exp);
    //var pow10exp_sig = (exp > sig) ? powers[exp - sig] : 1/powers[sig - exp];
    var significand = val / pow10exp;
    val = Math.round(significand * pow10sig)*pow10exp/pow10sig;
    return sign * val;
  },

  CLASS_NAME: "OpenLayers.Control.Ruler"
});
