
MappingKit.Control.StreetView =
  OpenLayers.Class(OpenLayers.Control, {

  /**
   * Property: type
   * TYPE_BUTTON.
   */
  type: OpenLayers.Control.TYPE_TOGGLE,
  
  /**
   * Embedded map div
   */
  embeddedMapDiv: null,
  
  switcherDiv: null,
  
  closerDiv: null,
  
  isDraggable: true,
  
  isOpen: null,
  
  vemap: null,
  
  vemapDiv: null,
  
  streetview: null,
  
  streetviewDiv: null,
  
  cyclomedia: null,
  
  cyclomediaDiv: null,
  
  previousTab: null,
  
  xmlhttp: null,
  
  initialize: function(div, options) {
    
    options.eventListeners = {'activate': this.trigger
      , 'deactivate': this.trigger
      };
    
    // todo: attach functions to .embed_switcher_tab's
    var self = this;
    $('.embed_switcher_tab').bind('click', { 'control': this }, this.onClickTab);
    
    this.embeddedMapDiv = OpenLayers.Util.getElement(div);
    this.switcherDiv = OpenLayers.Util.getElement('embed_switcher');
    this.closerDiv = OpenLayers.Util.getElement('embed_closer');
    
    this.vemapDiv = OpenLayers.Util.getElement('vemap');
    this.streetviewDiv = OpenLayers.Util.getElement('streetview');
    this.cyclomediaDiv = OpenLayers.Util.getElement('cyclomedia');
    
    OpenLayers.Control.prototype.initialize.apply(this, [options]);
  },
  
  onClickTab: function(e) {
    //alert( maptype );
    var maptype = $(this).text();

    var control = e.data.control;
    $(this).addClass('active');
    if (control.previousTab && control.previousTab != this) {
      $(control.previousTab).removeClass('active');
    }
    control.previousTab = this;
    
    var reset = false;
    switch (maptype) {
    case 'Virtual Earth':
      control.vemapDiv.style.display = 'block';
      if (control.streetviewDiv) control.streetviewDiv.style.display = 'none';
      if (control.cyclomediaDiv) control.cyclomediaDiv.style.display = 'none';
      break;
    case 'Street View':
      if (control.vemapDiv) control.vemapDiv.style.display = 'none';
      control.streetviewDiv.style.display = 'block';
      if (control.cyclomediaDiv) control.cyclomediaDiv.style.display = 'none';
      reset = true;
      break;
    case 'Cyclomedia':
      if (control.vemapDiv) control.vemapDiv.style.display = 'none';
      if (control.streetviewDiv) control.streetviewDiv.style.display = 'none';
      control.cyclomediaDiv.style.display = 'block';
      break;
    }
    control.createViewMap(maptype, reset);
  },
  
  destroy: function() {
    this.vemap = null;
    this.streetview = null;
    this.cyclomedia = null;
    
    OpenLayers.Control.prototype.destroy.apply(this, arguments);
  },
  
  setMap: function(map) {
    OpenLayers.Control.prototype.setMap.apply(this, arguments);
    
    map.events.register('moveend', this, this.onMoveEnd);
    
    var self = this;
    $('#embed_closer').bind('click', {that: this}, function(e) {
      // e.data.that === self
      self.active = false;
      self.events.triggerEvent('deactivate', {});
    });
    
    if (this.isDraggable) {
      // make view window draggable
      $('#embedded_map_main').draggable({handle: '#embed_header'});
    }
    else {
      $('#embed_header').css({'cursor': 'default'});
    }
  },
  
  /*
   * Method: trigger
   * 
   */
  trigger: function() {
    if (this.active) {
      // show streetview popup
      $(this.embeddedMapDiv).show();
      
      // todo: activate first tab
      var first_tab = $('.embed_switcher_tab')[0];
      var maptype = $(first_tab).text();
      if (!this.previousTab) {
        $(first_tab).addClass('active');
        this.previousTab = first_tab;
        try {
          this.createViewMap(maptype, true);//'Virtual Earth', true);
        }
        catch (e) {
        }
      }
      
      //if (this.vemap == null && this.streetview == null) {
      //  this.createViewMap(maptype, true);//'Virtual Earth', true);
      //}
    }
    else {
      // hide popup
      $(this.embeddedMapDiv).hide();
    }
  },
  
  onMoveEnd: function(e) {
    if (this.embeddedMapDiv.style.display != 'none') {
      var maptype = null;
      if (this.vemap != null && this.vemapDiv.style.display != 'none') {
        maptype = 'Virtual Earth';
      }
      else if (this.streetview != null && this.streetviewDiv.style.display != 'none') {
        maptype = 'Street View';
      }
      else if (this.cyclomedia != null && this.cyclomediaDiv.style.display != 'none') {
        maptype = 'Cyclomedia';
      }
      if (maptype) {
        this.createViewMap(maptype, false);
      }
    }
  },
  
  createViewMap: function(maptype, reset) {
    var longlat = this.map.getCenter();
    if (this.map.displayProjection) {
      longlat.transform(this.map.projection, this.map.displayProjection);
    }
    if (maptype == "Virtual Earth") {
      if (this.vemap == null) {
        this.vemap = new VEMap('vemap');
        // set your dimensions here, otherwise LoadMap will set them
        //$('#vemap').css({'width': '400px', 'height': '300px'});
        //$(this.switcherDiv).html('Switch to Google Street View');
        this.vemapDiv.style.display = 'block'; // $('#vemap').show();
        this.vemap.LoadMap(new VELatLong(longlat.lat, longlat.lon), this.map.getZoom() + 0, VEMapStyle.Birdseye);
        if (this.vemap.dashboardVersion === undefined) {
          this.vemap.HideDashboard();
        }
      }
      else if (reset) {
        this.vemap.SetMapMode(VEMapMode.Mode2D);
      }
      //var previousZoom = this.vemap.GetZoomLevel();
      this.vemap.SetZoomLevel(this.map.getZoom() + 0);
      //if (this.vemap.GetZoomLevel() != previousZoom) {
        // check birdseye
        //this.vemap.SetMapStyle(VEMapStyle.BirdseyeHybrid);
      //}
      this.vemap.PanToLatLong(new VELatLong(longlat.lat, longlat.lon));
    }
    else if (maptype == "Google Street View" || maptype == 'Street View') {
      if (reset) {
        this.streetviewDiv.style.display = 'block';
        this.streetview = new GStreetviewPanorama(document.getElementById("streetview"));
        GEvent.addListener(this.streetview, 'error', streetViewError);
      }
      this.streetview.setLocationAndPOV(new GLatLng(longlat.lat, longlat.lon));
    }
    else if (maptype == 'Cyclomedia') {
      if (this.cyclomedia == null) {
        this.cyclomedia = document.getElementById('CSLite');
      }
      /* */
      // search for cyclomedia locations at longlat
      var cyclomedia = this.cyclomedia;
      //this.timestamp = (new Date()).valueOf();
      if (this.xmlhttp != null) {
        if (this.xmlhttp.readyState != 4) {
          this.xmlhttp.abort();
        }
        this.xmlhttp = null;
      }
      this.xmlhttp = window.jQuery.ajax({'type': 'GET'
        , 'url': Drupal.settings.basePath + 'sites/all/modules/mappingkit/openlayers/cyclomedialocations.php'
        , 'data': { 'lat': longlat.lat, 'lon': longlat.lon }
        , 'dataType': 'json'
        , 'success': this.onCyclomediaLocationsSuccess
        , 'error': function(xmlhttp, status, error) { alert(status); }
        //, 'complete': this.onCyclomediaLocationsComplete
      });
      /* */
      //this.cyclomedia.URL = '33008Z0J';
      //this.cyclomedia.LookAt(2, 0, 84457.797, 447441.077);
    }
  },
  
  onCyclomediaLocationsComplete: function(xmlhttp, status) {
    //this; // the options for this ajax request
  },
  
  onCyclomediaLocationsSuccess: function(data, status) {
    // find closest point
    var min_dist = 1e9, min_id = '';
    for (var id in data.points) {
      var lon = parseFloat(data.params.lon), lat = parseFloat(data.params.lat);
      var dist = sqr(data.points[id].lon - lon) + sqr(data.points[id].lat - lat);
      if (dist < min_dist) {
        min_dist = dist;
        min_id = id;
      }
    }
    var cyclomedia = document.getElementById('CSLite');
    var url = cyclomedia.URL ? cyclomedia.URL.match(/.+\/(.+)\.CMI/) : null;
    if (!url || url.length < 2 || url[1] != min_id) {
      cyclomedia.URL = min_id;
    }
  },
  
  CLASS_NAME: 'mk.Control.StreetView'
});

function streetViewError(errorCode) {
  if (errorCode == 603) {
    alert(Drupal.t('Error: Flash doesn\'t appear to be supported by your browser'));
    return;
  }
  else if (errorCode == 600) {
    $('#streetview').html(Drupal.t('Google Street View not available for this location'));
    return;
  }
}

function sqr(x) {
  return x * x;
}

function CSLiteOnChangeOrientation(Viewdir) {
  //alert(Viewdir);
  // orientate marker at MappingKit.streetview.map
}

function CSLiteOnChangeLocation(item, loc) {
  // check if item is a known cyclomedia location; if it is, use that lat/lon,
  // otherwise convert the rd coordinates to lat/lon
  //alert(item + ' ' + loc.x + ',' + loc.y);
  
  var xmlhttp = window.jQuery.ajax({'type': 'GET'
    , 'url': Drupal.settings.basePath + 'sites/all/modules/mappingkit/transform.php'
    , 'data': { 'x': loc.x, 'y': loc.y, 'from': 'EPSG:28992' }
    , 'dataType': 'json'
    , 'success': locationChanged
    , 'error': function(xmlhttp, status, error) { alert(status); }
    //, 'complete': this.onCyclomediaLocationsComplete
  });
  // convert loc.x/loc.y to lat/lon?
}

function locationChanged(data, status) {
  //alert(data.lat + '/' + data.lon);
  // move marker at MappingKit.streetview.map
  //MappingKit.streetview.map.panTo(new OpenLayers.LonLat(data.lon, data.lat));
}

