
if (!OpenLayers.Filter.Logical.prototype.clone) {
  // These clone functions are not yet defined in OpenLayers 2.7
  
  OpenLayers.Filter.Logical.prototype.clone = function() {
    var filters = [];
    for(var i=0, len=this.filters.length; i<len; ++i) {
      filters.push(this.filters[i].clone());
    }
    return new OpenLayers.Filter.Logical({
      type: this.type,
      filters: filters
    });
  }
};

if (!OpenLayers.Filter.Comparison.prototype.clone) {
  OpenLayers.Filter.Comparison.prototype.clone = function() {
    return OpenLayers.Util.extend(new OpenLayers.Filter.Comparison(), this);
  }
};

MappingKit.Layer.WFS = OpenLayers.Class(
  OpenLayers.Layer.WFS, {
  
  /**
   * Filter property.
   */
  filter: null,
  
  attributeFilter: null,
  
  isNewFilter: true,
  
  spatialFilter: null,
  
  /**
   * Timeline object.
   */
  timeline: null,
  
  timelineTile: null,
  
  /**
   * Features in timeline.
   */
  timelineFeatures: null,
  
  /**
   * Time range for features in timeline.
   */
  timelineRange: null,
  
  // same arguments as OpenLayers.Layer.WFS(name, url, params, options)
  initialize: function() {
    OpenLayers.Layer.WFS.prototype.initialize.apply(this, arguments);
    
    // attributeFilter should not contain a spatial filter
    
    this.timelineFeatures = [];
  },
  
  destroy: function() {
    OpenLayers.Layer.WFS.prototype.destroy.apply(this, arguments);
  },
  
  setTimeFilter: function(beginDate_property, minVisibleDate, endDate_property, maxVisibleDate) {
    // set date filter of layer
    this.attributeFilter = new OpenLayers.Filter.Logical({
      type: OpenLayers.Filter.Logical.AND,
      filters: [
        new OpenLayers.Filter.Comparison({
          type: OpenLayers.Filter.Comparison.GREATER_THAN_OR_EQUAL_TO,
          property: beginDate_property,
          value: minVisibleDate
        }),
        new OpenLayers.Filter.Comparison({
          type: OpenLayers.Filter.Comparison.LESS_THAN_OR_EQUAL_TO,
          property: endDate_property,
          value: maxVisibleDate
        })
      ]
    });
    this.isNewFilter = true;
    
    if (! this.visibility) {
      return;
      // nothing to do for invisible layers
    }
    
    // refresh layer
    this.redraw();
  },
  
  updateFilter: function(spatialFilter) {
    if (! this.attributeFilter) {
      this.filter = spatialFilter;
      return;
    }
    
    this.filter = this.attributeFilter.clone();
    if (this.filter instanceof OpenLayers.Filter.Logical) {
      // add spatial filter to filters
      this.filter.filters.push(spatialFilter);
    }
    else if (this.filter instanceof OpenLayers.Filter.Comparison) {
      this.filter = new OpenLayers.Filter.Logical({
          type: OpenLayers.Filter.Logical.AND,
          filters: [this.filter]
      });
      this.filter.filters.push(spatialFilter);
    }
    else {
      // todo: error?
      alert('wrong type of filter parameter for wfs layer');
    }
  },
  
  timelineRequestSuccess: function(request) {                
    if (this.layer.timelineFeatures) {
      var doc = request.responseXML;
      if (!doc || !doc.documentElement) {
        doc = request.responseText;
      }
      
      // read features
      this.layer.timelineFeatures = this.layer.formatObject.read(doc);
      
      // add features to timeline
      MappingKit.Timeline.addFeatures(this.layer, this.layer.timelineFeatures);
    }
    
    this.request = null;
  },
  
  /** 
   * Method: moveTo
   * 
   * Parameters:
   * bounds - {<OpenLayers.Bounds>} 
   * zoomChanged - {Boolean} 
   * dragging - {Boolean} 
   */
  moveTo:function(bounds, zoomChanged, dragging) {
    if (this.vectorMode) {
      OpenLayers.Layer.Vector.prototype.moveTo.apply(this, arguments);
    } else {
      OpenLayers.Layer.Markers.prototype.moveTo.apply(this, arguments);
    }
    
    // don't load wfs features while dragging, wait for drag end
    if (dragging) {
      // TBD try to hide the vector layer while dragging
      // this.setVisibility(false);
      // this will probably help for panning performances
      return false;
    }
    
    if ( zoomChanged) {
      if (this.vectorMode) {
        this.renderer.clear();
      }
    }
    
  //DEPRECATED - REMOVE IN 3.0
    // don't load data if current zoom level doesn't match
    if (this.options.minZoomLevel) {
      OpenLayers.Console.warn(OpenLayers.i18n('minZoomLevelError'));
      
      if (this.map.getZoom() < this.options.minZoomLevel) {
        return null;
      }
    }
    
    if (bounds == null) {
      bounds = this.map.getExtent();
    }

    var firstRendering = (this.tile == null);

    if (this.timelineTile == null) {
      // fill timeline with all events

      //determine new tile bounds
      var center = bounds.getCenterLonLat();
      var tileWidth = bounds.getWidth() * this.ratio;
      var tileHeight = bounds.getHeight() * this.ratio;
      var tileBounds = 
        new OpenLayers.Bounds(center.lon - (tileWidth / 2),
                    center.lat - (tileHeight / 2),
                    center.lon + (tileWidth / 2),
                    center.lat + (tileHeight / 2));

      //determine new tile size
      var tileSize = this.map.getSize();
      tileSize.w = tileSize.w * this.ratio;
      tileSize.h = tileSize.h * this.ratio;

      //determine new position (upper left corner of new bounds)
      var ul = new OpenLayers.LonLat(tileBounds.left, tileBounds.top);
      var pos = this.map.getLayerPxFromLonLat(ul);
      
      if (this._timeline) {
        //formulate request url string for features in timeline (all)
        var url = this.getFullRequestString();
        
        this.timelineTile = new OpenLayers.Tile.WFS(this, pos, tileBounds, 
                             url, tileSize);
        this.timelineTile.loadFeaturesForRegion(this.timelineRequestSuccess);
      }
    }
    
    //does the new bounds to which we need to move fall outside of the 
    // current tile's bounds?
    var outOfBounds = (!firstRendering &&
               !this.tile.bounds.containsBounds(bounds));

    if (this.isNewFilter || zoomChanged || firstRendering || (!dragging && outOfBounds)) {
      // reset isNewFilter
      this.isNewFilter = false;
      
      //determine new tile bounds
      var center = bounds.getCenterLonLat();
      var tileWidth = bounds.getWidth() * this.ratio;
      var tileHeight = bounds.getHeight() * this.ratio;
      var tileBounds = 
        new OpenLayers.Bounds(center.lon - (tileWidth / 2),
                    center.lat - (tileHeight / 2),
                    center.lon + (tileWidth / 2),
                    center.lat + (tileHeight / 2));

      //determine new tile size
      var tileSize = this.map.getSize();
      tileSize.w = tileSize.w * this.ratio;
      tileSize.h = tileSize.h * this.ratio;

      //determine new position (upper left corner of new bounds)
      var ul = new OpenLayers.LonLat(tileBounds.left, tileBounds.top);
      var pos = this.map.getLayerPxFromLonLat(ul);

      //formulate request url string
      var url = this.getFullRequestString();
      
      var params = {};
      if (!this.attributeFilter) {
        // get params.BBOX (behaviour of OpenLayers.Layer.WFS)
        var bounds = tileBounds.clone();
        if (this.map && !this.projection.equals(this.map.getProjectionObject())) {
          bounds.transform(this.map.getProjectionObject(), this.projection);
        }
        params.BBOX = this.encodeBBOX ? bounds.toBBOX() : bounds.toArray();
      }
      else {
        if (!this.spatialFilter) {
          // get spatial filter as current extent
          // OR restrictedExtent of layer (whichever is smaller)
          var bounds = tileBounds.clone();
          if (this.restrictedExtent
            && bounds.containsBounds(this.restrictedExtent)) {
            bounds = this.restrictedExtent;
          }
          if (this.map && !this.projection.equals(this.map.getProjectionObject())) {
            bounds.transform(this.map.getProjectionObject(), this.projection);
          }
          this.updateFilter(new OpenLayers.Filter.Spatial({
            type: OpenLayers.Filter.Spatial.BBOX,
            value: bounds,
            projection: this.projection,
            property: 'the_geom' // Important! Must be something but which value doesn't matter.
          }));
        }
        else {
          // use current spatialfilter
          this.updateFilter(this.spatialFilter);
        }
        
        // todo: when combining filters check if filter is logical or not; when
        // logical push to filters property.
        
        // add filter to params
        var xml = new OpenLayers.Format.XML();
        var format = new OpenLayers.Format.Filter(); // 2.7 only supports 1.0.0
        params.filter = xml.write(format.write(this.filter));
        
        OpenLayers.Console.log('ogc filter', params.filter);
      }

      url += "&" + OpenLayers.Util.getParameterString(params);

      if (!this.tile) {
        this.tile = new OpenLayers.Tile.WFS(this, pos, tileBounds, 
                           url, tileSize);
        this.addTileMonitoringHooks(this.tile);
        this.tile.draw();
      } else {
        if (this.vectorMode) {
          this.destroyFeatures();
          this.renderer.clear();
        } else {
          this.clearMarkers();
        }  
        this.removeTileMonitoringHooks(this.tile);
        this.tile.destroy();
        
        this.tile = null;
        this.tile = new OpenLayers.Tile.WFS(this, pos, tileBounds, 
                           url, tileSize);
        this.addTileMonitoringHooks(this.tile);
        this.tile.draw();
      }
    }
  },
  
  CLASS_NAME: "MappingKit.Layer.WFS"
});
