<?php
/**
 * Gebruikt CM_RecordingLocationService2.dll in c:\windows\download program files\
 * Dit moet eerst geregistreerd zijn dmv:
 *
 *   regsvr32 CM_RecordingLocationService2.dll
 */

function drupal_json($var = NULL) {
  // We are returning JavaScript, so tell the browser.
  isset($_GET['debug']) || header('Content-Type: text/javascript; charset=utf-8');
  
  if (isset($var)) {
    echo drupal_to_js($var);
  }
}

function drupal_to_js($var) {
  switch (gettype($var)) {
    case 'boolean':
      return $var ? 'true' : 'false'; // Lowercase necessary!
    case 'integer':
    case 'double':
      return $var;
    case 'resource':
    case 'string':
      return '"'. str_replace(array("\r", "\n", "<", ">", "&"),
                              array('\r', '\n', '\x3c', '\x3e', '\x26'),
                              addslashes($var)) .'"';
    case 'array':
      // Arrays in JSON can't be associative. If the array is empty or if it
      // has sequential whole number keys starting with 0, it's not associative
      // so we can go ahead and convert it as an array.
      if (empty ($var) || array_keys($var) === range(0, sizeof($var) - 1)) {
        $output = array();
        foreach ($var as $v) {
          $output[] = drupal_to_js($v);
        }
        return '[ '. implode(', ', $output) .' ]';
      }
      // Otherwise, fall through to convert the array as an object.
    case 'object':
      $output = array();
      foreach ($var as $k => $v) {
        $output[] = drupal_to_js(strval($k)) .': '. drupal_to_js($v);
      }
      return '{ '. implode(', ', $output) .' }';
    default:
      return 'null';
  }
}

function coordinate_transform($inproj, $outproj, &$points) {
  
  // assert: in/outproj start with epsg:
  $inproj = strtolower($inproj);
  $outproj = strtolower($outproj);
  if (substr($inproj, 0, 5) != 'epsg:'
    || substr($outproj, 0, 5) != 'epsg:') {
    die("Cannot reproject from $inproj to $outproj");
  }
  
  $projInObj = ms_newprojectionobj("init=". $inproj);
  $projOutObj = ms_newprojectionobj("init=". $outproj);
  $poPoint = ms_newpointobj();
  //drupal_set_message('points '.var_export($points, TRUE));
  foreach ($points as &$point) {
    $poPoint->setXY($point['x'], $point['y']);
    //drupal_set_message(var_export($poPoint, TRUE));
    if (MS_FAILURE == $poPoint->project($projInObj, $projOutObj)) {
      return NULL;
    }
    if ($outproj == 'epsg:4326') {
      $point['lat'] = $poPoint->y;
      $point['lon'] = $poPoint->x;
    }
    else {
      $point['x'] = $poPoint->x;
      $point['y'] = $poPoint->y;
    }
  }
  return TRUE;
}


global $proj_code;
$proj_code = 'EPSG:28992';

try {
  $oRecordingLocation = new COM('CycloMedia.DataServices.RecordingLocation.2')
    or die('lukt niet');
  $oRecordingLocation->UseProxy = "connector001.cycloconnect.nl";
  // synchronous interface of recordinglocation (versus $Async)
  
  // extent
  //$oData = $oRecordingLocation->Sync->GetInRangeSync(2, 0, 84380, 84430, 447500, 447550);
  
  // near x/y
  //$oData = $oRecordingLocation->Sync->GetNearXYSync(2, 0, 84405, 447525, 25, 100);
  // RD
  //$oData = $oRecordingLocation->Sync->GetNearXYSync(2, 0, 84405.1, 447525.5, 25, 100);
  
  //http://localhost/drupal-6.x/?q=node%2F132&zoom=16&lat=52.3728&lon=4.89404&layers=B00000F
  // Lon/Lat
  
  if (isset($_GET['debug'])) {
    error_reporting(E_ALL);
  }
  
  if (isset($_GET['top'])) {
    $points = array(array('x' => (float)$_GET['left'], 'y' => (float)$_GET['bottom'])
      , array('x' => (float)$_GET['right'], 'y' => (float)$_GET['top'])
      );
    // default srs=EPSG:4326 (lat/lon)
    $inproj = isset($_GET['srs']) ? strtoupper($_GET['srs']) : 'EPSG:4326';
  }
  else if (isset($_GET['lat']) && isset($_GET['lon'])) {
    $points = array(array('x' => (float)$_GET['lon'], 'y' => (float)$_GET['lat']));
    // srs=EPSG:4326 (lat/lon)
    $inproj = 'EPSG:4326';
  }
  else if (isset($_GET['x']) && isset($_GET['y'])) {
    $points = array(array('x'=> $_GET['x'], 'y'=> $_GET['y']));
    // default srs=EPSG:28992 (RD)
    $inproj = isset($_GET['srs']) ? strtoupper($_GET['srs']) : $proj_code;
  }
  else {
    drupal_json(array());
    exit();
  }
  
  isset($_GET['debug']) && print('Voor transformatie: '. var_export($points, TRUE) .'<br/>');
  $inpoints = $points;
  if ($inproj != 'EPSG:28992') {
    coordinate_transform($inproj, $proj_code, $points);
  }
  isset($_GET['debug']) && print('Punten in RD: '. var_export($points, TRUE) .'<br/>');
  
  if (count($points) == 2) {
    $bbox = $points;
    // check if area is too large
    $max_width = $max_height = 425;
    $width = $points[1]['x'] - $points[0]['x'];
    if ($width > $max_width) {
      $diff = ($width - $max_width)/2;
      $points[0]['x'] += $diff;
      $points[1]['x'] -= $diff;
    }
    $width = $points[1]['y'] - $points[0]['y'];
    if ($width > $max_width) {
      $diff = ($width - $max_width)/2;
      $points[0]['y'] += $diff;
      $points[1]['y'] -= $diff;
    }
    
    $oData = $oRecordingLocation->Sync->GetInRangeSync(2, 0
      , $points[0]['x'], $points[1]['x'], $points[0]['y'], $points[1]['y']);
    
    isset($_GET['debug']) && print('Aantal punten: '. count($oData) . '<br/>');
  }
  else {
    $bbox = NULL;
    $point = $points[0];
    
    $oData = $oRecordingLocation->Sync->GetNearXYSync(2, 0, $point['x'], $point['y']
      , 150, 1000);
  }
}
catch (Exception $e) {
  // e->message, code, file, line, trace (file, line, function, class, type, args)
  // -2147352567 == 
  //die('Exceptie: '. var_export($e, TRUE));
  //echo dechex($e->getCode());
  
  // getCode, getMessage, getFile, getLine, getTrace, getTraceAsString
  
  //$rC = new ReflectionObject($e);
  //Reflection::export($rC);
  drupal_json(array('code' => dechex($e->getCode()), 'message' => $e->getMessage()));
  exit();
}

/*
print "The type is " . variant_get_type($oData) . "<br/>";
print "count " . count($oData) . '<br/>';

echo 'VT_EMPTY=' . VT_EMPTY .'<br>'; // 0
echo 'VT_NULL=' . VT_NULL .'<br>';
echo 'VT_I2=' . VT_I2 .'<br>';
echo 'VT_I4=' . VT_I4 .'<br>';
echo 'VT_R4=' . VT_R4 .'<br>';
echo 'VT_R8=' . VT_R8 .'<br>';
echo 'VT_CY=' . VT_CY .'<br>';
echo 'VT_DATE=' . VT_DATE .'<br>';
echo 'VT_BSTR=' . VT_BSTR .'<br>'; // 8
echo 'VT_DISPATCH=' . VT_DISPATCH .'<br>';
echo 'VT_ERROR=' . VT_ERROR .'<br>';
echo 'VT_BOOL=' . VT_BOOL .'<br>'; // 11
echo 'VT_VARIANT='. VT_VARIANT .'<br>'; // 12
echo 'VT_UNKNOWN=' . VT_UNKNOWN .'<br>'; // 13
echo 'VT_DECIMAL=' . VT_DECIMAL .'<br>'; // 14

echo 'VT_I1=' . VT_I1 .'<br>'; // 16
echo 'VT_UI1=' . VT_UI1 .'<br>'; // 17
echo 'VT_UI2=' . VT_UI2 .'<br>'; // 18
echo 'VT_UI4=' . VT_UI4 .'<br>'; // 19

echo 'VT_INT=' . VT_INT .'<br>'; // 22
echo 'VT_UINT=' . VT_UINT .'<br>'; // 23

echo 'VT_ARRAY='. VT_ARRAY .'<br>'; // 8192 (vlag)
echo 'VT_BYREF=' . VT_BYREF .'<br>'; // 16384 (vlag)
//echo '=' . VT_ .'<br>';

// 8204 = array
// 9 = 
*/

$rect = $points;
$points = array();
foreach ($oData as $rec) {
  //print "The value is " . $rec->X . "<br/>";
  // points as [x, y, id]
  //$points[] = array('x' => $rec['X']->value, 'y' => $rec['Y']->value, 'id' => $rec['ImageID']->value);
  
  // points as [id] => [x, y]
  $points[ $rec['ImageID']->value ] = array('x' => $rec['X']->value, 'y' => $rec['Y']->value);
}

isset($_GET['debug']) && print('Voor transformatie: '. var_export($points, TRUE) .'<br/>');
coordinate_transform($proj_code, 'EPSG:4326', $points);
isset($_GET['debug']) && print('Na transformatie: '. var_export($points, TRUE) .'<br/>');

drupal_json(array('params' => $_GET, 'bbox' => $bbox, 'rect' => $rect, 'points' => $points));
//echo var_export($points, TRUE) .'<br/>';

//$rC = new ReflectionObject( $oData );
//Reflection::export($rC);
//print $rC;
//Reflection::export(new ReflectionClass('Exception'));

$oRecordingLocation = NULL;
