<?php
/**
 * @file
 * Administer openlayers mapclient.
 */

function mappingkit_openlayers_settings_form() {

  $form['openlayers'] = array(
    '#type' => 'fieldset'
    , '#title' => t('OpenLayers')
    , '#collapsible' => TRUE
    );
  
  $form['openlayers']['mappingkit_openlayers_path'] = array(
    '#type' => 'textfield'
    , '#title' => t('Path')
    , '#description' => t('Path to OpenLayers library. By default this uses the hosted version of <a href="http://openlayers.org" target="_blank">OpenLayers</a> at %link. A given path can be relative to the Drupal folder.', array('%link' => MAPPINGKIT_DEFAULT_OPENLAYERS_PATH))
    , '#default_value' => variable_get('mappingkit_openlayers_path', MAPPINGKIT_DEFAULT_OPENLAYERS_PATH)
    );
  
  $form['openlayers']['mappingkit_openlayers_release'] = array(
    '#type' => 'select'
    , '#title' => t('Release')
    , '#description' => t('Release of OpenLayers library.')
    , '#options' => array('2.7' => t('Previous release (2.7)'), '2.8' => t('Stable (2.8)'), '2.9' => t('Developer (2.9)'))
    , '#default_value' => variable_get('mappingkit_openlayers_release', MAPPINGKIT_DEFAULT_OPENLAYERS_RELEASE)
    , '#description' => t('With which OpenLayers release is the library used compatible?')
    );
  
  $form['openlayers']['mappingkit_openlayers_tile_size'] = array('#type' => 'textfield'
    , '#title' => t('Tile size')
    , '#default_value' => variable_get('mappingkit_openlayers_tile_size', MAPPINGKIT_DEFAULT_OPENLAYERS_TILE_SIZE)
    , '#description' => t('Size of tiles that are used for retrieving maps. At this moment, only square tiles are supported.')
    );
  
  $form['openlayers']['mappingkit_openlayers_resolution'] = array('#type' => 'textfield'
    , '#title' => t('Resolution')
    , '#default_value' => variable_get('mappingkit_openlayers_resolution', MAPPINGKIT_DEFAULT_OPENLAYERS_RESOLUTION)
    , '#description' => t('Display resolution (dots per inch) used to calculate map scale.')
    );
  
  $form['controls'] = array(
    '#type' => 'fieldset'
    , '#title' => t('Controls')
    , '#collapsible' => TRUE
    );
  
  $form['controls']['mappingkit_openlayers_scaleline_units'] = array('#type' => 'select'
    , '#title' => t('Scale line units')
    , '#options' => array('' => t('Default'), 'km' => t('Metric'), 'mi' => t('Imperial'))
    , '#default_value' => variable_get('mappingkit_openlayers_scaleline_units', '')
    );
  
  //$form['#validate'][] = 'openlayers_settings_form_validate';
  //$form['#submit'][] = 'openlayers_settings_form_submit';
  
  $form['proj4'] = array('#type' => 'fieldset'
    , '#title' => t('Proj.4')
    , '#collapsible' => TRUE
    );
  
  $form['proj4']['mappingkit_proj4js_path'] = array('#type' => 'textfield'
    , '#title' => t('Proj4js path')
    , '#default_value' => variable_get('mappingkit_proj4js_path', MAPPINGKIT_DEFAULT_PROJ4JS_PATH)
    , '#description' => t('Relative or absolute path to Proj4js.')
    );
  
  global $db_type;
  if ($db_type == 'pgsql') {
    global $db_url;
    if (!is_array($db_url)) {
      $db_names = array('default');
    }
    else {
      $db_names = array_keys($db_url);
    }
    
    $form['proj4']['mappingkit_spatial_database'] = array('#type' => 'select'
      , '#title' => t('Spatial database')
      , '#default_value' => variable_get('mappingkit_spatial_database', '')
      , '#options' => array('' => t('Select a PostGIS-enabled database')) + array_combine($db_names, $db_names)
      , '#description' => t('Spatial database to use for projection definitions from the PostGIS spatial_ref_sys table.')
      , '#element_validate' => array('mappingkit_spatial_database_validate')
      );
  }
  
  $form['proj4']['mappingkit_proj4_definitions'] = array('#type' => 'textarea'
    , '#title' => t('Proj4 definitions')
    , '#collapsible' => TRUE
    , '#rows' => 5
    , '#default_value' => variable_get('mappingkit_proj4_definitions', '')
    , '#description' => t('Enter the available projections as <em>srid=proj4 parameters</em>, e.g. <em>900913=+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext +no_defs</em>. Each projection on a separate line. Try !link for information on projections.', array('!link' => l('http://spatialreference.org', 'http://spatialreference.org')))
    , '#attributes' => array('wrap' => 'off')
    //, '#attributes' => array('style' => 'white-space: nowrap;')
    );
  
  return system_settings_form($form);
}

/**
 * Validate selection of spatial (PostGIS) database.
 */
function mappingkit_spatial_database_validate($form, $form_state) {
  $mappingkit_spatial_database = $form_state['values']['mappingkit_spatial_database'];
  if (!empty($mappingkit_spatial_database)) {
    $active = db_set_active($mappingkit_spatial_database);
    
    if (!db_result(@db_query('select 1 from spatial_ref_sys'))) {
      form_set_error('mappingkit_spatial_database', t('Database %db is not PostGIS enabled', array('%db' => $mappingkit_spatial_database)));
    }
    //drupal_set_message(var_export($form_state['values'], TRUE));
    
    db_set_active($active);
  }
}

