
MappingKit.OpenLayers = {};
MappingKit.selectControls = [];

if (!$defined(MappingKit.Control)) {
  MappingKit.Control = {};
};

if (!$defined(MappingKit.Format)) {
  MappingKit.Format = {};
};

if (!$defined(MappingKit.Layer)) {
  MappingKit.Layer = {};
};

MappingKit.calculateInRange = function() {
  var inRange = false;
  if (this.map) {
    var resolution = this.map.getResolution();
    inRange = ( (resolution >= this.minResolution) &&
                (resolution <= this.maxResolution) );
    var extent = this.map.getExtent();
    if (extent) {
      // check that maxExtent of layer is within the current extent
      inRange = inRange && extent.intersectsBounds(this.maxExtent);
    }
  }
  return inRange;
};

/**
 * custom translate function for OpenLayers to use Drupal language texts
 */
MappingKit.translate = function(key, context) {
  var en_dictionary = OpenLayers.Lang['en'];
  var message = en_dictionary[key];
  if (OpenLayers.Lang.getCode() != 'en') {
    // check Drupal's translation
    var drupal_message = Drupal.t(key);
    if (drupal_message != key) {
      // Drupal should return something different from the key
      message = drupal_message;
    }
  }
  if (!message) {
    message = key;
  }
  if (context) {
    message = OpenLayers.String.format(message, context);
  }
  return message;
};

// redirect OpenLayers language support to Drupal
OpenLayers.Lang.translate = MappingKit.translate;
OpenLayers.i18n = MappingKit.translate;

/**
 * Translate attribute name. Search order is first attribute:attributeName/layerName
 * and (if not found) attribute:attributeName. If not found, the attributeName
 * is returned.
 * @param attributeName
 *   {string} Name of attribute to have translated.
 * @param layerName
 *   {string} Optional name of layer.
 *
 * TODO: enable formatting of attribute value
 */
MappingKit.translateAttribute = function(attributeName, layerName) {
  var search = 'attribute:' + attributeName, find;
  if (layerName) {
    var searchMax = search + '/' + layerName;
    find = Drupal.t(searchMax);
    if (find != searchMax) {
      return find;
    }
  }
  
  find = Drupal.t(search);
  return find != search ? find : attributeName;
};

MappingKit.OpenLayers.onFeatureModification = function(feature) {
  var el = document.getElementById('edit-geometry');
  if (el) {
    el.value = feature.geometry;
  }
};

// baseLayer
// overlays
// permalink
// scale

MappingKit.movestart = function(evt) {
  if (!evt.dragging) {
    // reset array
    MappingKit.loadErrors = [];
    
    //$('div.mapclient-messages').empty().removeClass('error');
    var el = document.getElementById('mapclient-messages');
    if (el) {
      // remove list item
      $(el).empty();
      
      // remove error class from parent
      $(el).parent().removeClass('error');
    }
  }
};

MappingKit.removelayer = function(evt) {
  if (!MappingKit.loadErrors) {
    MappingKit.loadErrors = [];
  }
  if (evt.layer instanceof OpenLayers.Layer.WMS) {
    for (var i = 0; i < MappingKit.loadErrors.length; i++) {
      if (evt.layer.params && MappingKit.loadErrors[i] == evt.layer.params.LAYERS) {
        //alert('remove ' + evt.layer.name);
        MappingKit.loadErrors.splice(i, 1);
        var item = document.getElementById('message-' + evt.layer.params.LAYERS);
        if (item) {
          item.parentNode.removeChild(item);
        }
        return;
      }
    }
  }
};

MappingKit.onImageLoadError = function() {
  if (!MappingKit.loadErrors) {
    MappingKit.loadErrors = [];
  }
  var el = document.getElementById('mapclient-messages');
  if (el) {
    var parts = this.src.split('?');
    var request = {service: parts[0]};
    if (parts.length > 1) {
      var params = parts[1].split('&');
      for (var i = 0; i < params.length; i++) {
        var tmp = params[i].split('=');
        var key = tmp[0].toLowerCase();
        var value = unescape(tmp[1]);
        request[key] = value;
        if (key == 'layers') {
          var layers = unescape(tmp[1]);
          
          // check if layer error is already shown
          for (var j = 0; j < MappingKit.loadErrors.length; j++) {
            if (MappingKit.loadErrors[j] == layers) {
              return;
            }
          }
          if (j == 0) {
            $(el).parent().addClass('error');
          }
          MappingKit.loadErrors[j] = layers;
        }
      }
    }
    else if (request.service.indexOf('/404.png') >= 0) {
      // ignore this one
      return;
    }
    else {
      // not a wms layer request; only show host name part of service
      var matches = request.service.match(/http\:\/\/([^\/]*)\//);
      var layers = matches[0];
      // check if layer error is already shown
      for (var j = 0; j < MappingKit.loadErrors.length; j++) {
        if (MappingKit.loadErrors[j] == layers) {
          return;
        }
      }
      if (j == 0) {
        $(el).parent().addClass('error');
      }
      MappingKit.loadErrors[j] = layers;
    }
    var listItem = document.createElement('li');
    listItem.id = 'message-' + layers;
    listItem.innerHTML = Drupal.t('Error loading images for <em>@layers</em>.', {'@layers': layers});
    el.appendChild(listItem);
  }
};

function osm_getTileURL(bounds) {
  var res = this.map.getResolution();
  
  var x = Math.round((bounds.left - this.maxExtent.left) / (res * this.tileSize.w));
  var y = Math.round((this.maxExtent.top - bounds.top) / (res * this.tileSize.h));
  var z = this.map.getZoom();
  var limit = Math.pow(2, z);

  if (y < 0 || y >= limit) {
    return OpenLayers.Util.getImagesLocation() + "404.png";
  } else {
    x = ((x % limit) + limit) % limit;
    return this.url + z + "/" + x + "/" + y + "." + this.type;
  }
};

MappingKit.onPopupClose = function(evt) {
  this.map.removePopup(this);
  this.destroy();
};

MappingKit.handleFeatureSelected = function(e) {
  return MappingKit.onFeatureSelect.apply(this, [e.feature]);
};

MappingKit.handleFeatureUnselected = function(e) {
  return MappingKit.onFeatureUnselect.apply(this, [e.feature]);
};

MappingKit.onFeatureSelect = function(feature) {
  MappingKit.selectedFeature = feature;
  
  // check attributes
  var count = 0, description_type = 'none';
  for (var item in feature.attributes) {
    count += 1;
    if (item == 'description') {
      // todo: check nr of <*> and </*>
      description_type = 'html';
    }
  }
  
  if (description_type == 'html') {
    var content = '<div style="font-size:.8em;">';
    var title = feature.attributes['title'] || feature.attributes['name'];
    if (title) {
      content += '<h2>' + title + '</h2>';
    }
    if (feature.attributes['description']) {
      content += feature.attributes.description;
    }
    else {
      for (var item in feature.attributes) {
        if (item != 'name' && feature.attributes[item] != null && feature.attributes[item] != '') {
          content += '<p>' + item + '=' + feature.attributes[item] + '</p>';
        }
      }
    }
    content += '</div>';
  }
  else {
    var content = '';
    content += '<div class="feature-layer">' 
      + Drupal.t('Layer %layer', {'%layer': feature.layer.name })
      + '</div>'
      ;
    content += Drupal.theme('feature', feature);
  }
  /*
  var tmp = $.ajax({
    async: false,
    type: 'GET',
    url: '/drupal-6.x/',
    data: {
      q: 'get_node_form/wms'
    }
  });
  content = tmp.responseText;
  */
  
  var popup = new OpenLayers.Popup.FramedCloud("chicken"
    , feature.geometry.getBounds().getCenterLonLat()
    , null // new OpenLayers.Size(100, 100)
    , content //'<h2>' + feature.attributes.name + '</h2>' + feature.attributes.desc
    , null
    , true // closebox
    , MappingKit.onPopupClose
    );
  feature.popup = popup;
  this.map.addPopup(popup);
};

MappingKit.onFeatureUnselect = function(feature) {
  var popup = feature.popup;
  if (!popup) {
    //alert('feature popup is null');
    OpenLayers.Console.warn('feature popup is null');
    var popupElement = document.getElementById("chicken");
    var map = feature.layer.map;
    if (map.popups.length == 1 && map.popups[0].div === popupElement) {
      popup = map.popups[0];
    }
  }
  
  if (popup.map != null && feature.layer.map.popups.length > 0) {
    feature.layer.map.removePopup(popup);
    popup.destroy();
  }
  feature.popup = null;
};

MappingKit.supportsGeolocation = function() {
  return (navigator.geolocation) ? true : false;
};

MappingKit.getGeolocation = function(successHandler, errorHandler) {
  if (navigator.geolocation) {
    navigator.geolocation.getCurrentPosition(successHandler, errorHandler);
  }
};

MappingKit.getMapCookieKey = function(map) {
  return map.id + '_' + location.href.replace(/([^a-z0-9_])/g, '');
};

/**
 * Theme feature attributes
 */
Drupal.theme.prototype.feature = function(feature) {
  var tpl = '<div class="feature-item"><table class="feature-table">';
  var count = 0;
  for (var item in feature.attributes) {
    count += 1;
    if (feature.attributes[item]) {
      tpl += '<tr class="' + ((count & 1) != 0 ? 'odd' : 'even') + '">' + Drupal.theme('featureAttribute', feature, item, feature.attributes[item]) + '</tr>';
    }
  }
  tpl += '</table></div>';
  return tpl;
};

/**
 * Theme feature attribute (name and value).
 */
Drupal.theme.prototype.featureAttribute = function(featureType, name, value) {
  var tpl = '';
  tpl += '<td class="feature-attribute-name">' + MappingKit.translateAttribute(name) + '</td><td class="feature-attribute-value">' + value + '</td>';
  return tpl;
};

/**
 * @param obj
 *  {object} pressed button
 * @param search
 *  {object} search options (attribute, layer, version)
 */
MappingKit.search = function(obj, options) {
  var elem = obj.previousSibling.firstChild;
  var value = elem.value.trim();
  if (value.length <= 0) {
    alert(Drupal.t('Enter a value to search for !attr in !layer', {'!attr': options.attribute, '!layer': options.layer}));
  }
  else {
    var map = MappingKit.getMap(elem.id.substr(0, elem.id.indexOf('-search-input')));
    var layers = map.getLayersByName(options.layer);
    if (layers.length > 0) {
      var protocol = OpenLayers.Protocol.WFS.fromWMSLayer(layers[0], { 
        featurePrefix: options.featurePrefix,
        version: options.version,
        maxFeatures: options.maxFeatures ? options.maxFeatures : 1,
        filter: new OpenLayers.Filter.Comparison({
          type: OpenLayers.Filter.Comparison.LIKE,
          property: options.attribute,
          value: value
        }),
        callback: MappingKit.searchCallback
      });
      var response = protocol.read();
    }
  }
};

MappingKit.searchCallback = function(response) {
  if (response.features.length > 0) {
    // zoom to feature
    // @todo: determine which map object (in case of multiple maps)
    var map = MappingKit.getMap();
    var geom = response.features[0].geometry;
    var centroid = geom.getCentroid();
    var bounds = geom.getBounds();
    var zoom = map.getZoomForExtent(bounds);
    zoom -= 1; // zoom out
    // @todo: determine zoom level
    map.setCenter(new OpenLayers.LonLat(centroid.x, centroid.y), zoom);
  }
  else {
    // error message?
    alert(Drupal.t('No feature found.'));
  }
};
