<?php
/**
 * @file
 * Openlayers mapclient include file for Mapping kit.
 */

/**
 * OpenLayers release version number.
 */
define("MAPPINGKIT_OPENLAYERS_RELEASE", (float)variable_get('mappingkit_openlayers_release', MAPPINGKIT_DEFAULT_OPENLAYERS_RELEASE));

define('FOR_LAYERSWITCHER', 1);
define('FOR_MAPCONTEXT', 2);
define('FOR_GPX', 4);
define('FOR_GEORSS', 8);
define('FOR_GOOGLE', 16);                        
define('FOR_VE', 32);
define('FOR_YAHOO', 64);
define('FOR_TMS', 128); // openstreetmap, openaerialmap
define('FOR_HISTORY', 256);
define('FOR_CONTROLS', 512);
define('FOR_POPUP', 1024);
define('FOR_WFS', 2048);
define('FOR_KML', 4096);
define('FOR_IMAGE', 8192);
define('FOR_OVERVIEW', 16384);
define('FOR_RULER', 32768);
define('FOR_VECTOR', 65536);
define('FOR_EDITS', 131072); // 0x00020000
define('FOR_TILECACHE', 262144);
//define('FOR_', 524288);
//define('FOR_', 1048576);

/**
 * Initialize header.
 */
function mappingkit_openlayers_init_header() {
  static $init_done = FALSE;
  
  $scope = variable_get('mappingkit_add_js_scope', DEFAULT_MAPPINGKIT_ADD_JS_SCOPE);
  if (!$init_done) {
    $init_done = TRUE;
    
    $path = variable_get('mappingkit_openlayers_path', MAPPINGKIT_DEFAULT_OPENLAYERS_PATH);
    $parts = parse_url($path);
    
    // check if firebug must be loaded
    if (isset($_GET['firebug'])) {
      if ($_GET['firebug'] == 'true' || $_GET['firebug'] == 'on') {
        $_SESSION['firebug'] = TRUE;
      }
      else {
        unset($_SESSION['firebug']);
      }
    }
    if (isset($_SESSION['firebug'])) {
      // debug openlayers
      if (!isset($parts['scheme'])) {
        drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/'. dirname($path) .'/Firebug/firebug.js', 'module', 'header');
      }
      else {
        mappingkit_add_js_external('http://openlayers.org/dev/lib/Firebug/firebug.js', $scope);
        //drupal_set_html_head('<script type="text/javascript" src=""></script>');
      }
    }
    
    // determine css path /drupal-6.x/lib/OpenLayers-2.7/theme/default/style.css
    $pos = strpos($path, 'lib/OpenLayers.js');
    if ($pos === FALSE) {
      $pos = strpos($path, 'OpenLayers.js');
    }
    if ($pos === FALSE) {
      // todo: warning: cannot determine css path
    }
    else {
      $openlayers_default_css = substr($path, 0, $pos) . 'theme/default/style.css';
    }
    
    if (isset($parts['scheme'])) {
      // Absolute path with scheme so cannot use drupal_add_js
      mappingkit_add_js_external($path, $scope);
      
      // add openlayers default css
      drupal_set_html_head('<link href="' . $openlayers_default_css . '" type="text/css" rel="stylesheet"/>');
      
      //drupal_set_html_head('<script type="text/javascript" src="'. $path . '"></script>');
    }
    else {
      // else: openlayers.js will be loaded later on
      
      // relative css
      drupal_set_html_head('<link href="' . mapclient_get_url($openlayers_default_css) . '" type="text/css" rel="stylesheet"/>');
    }
    
    // add standard css
    drupal_add_css(drupal_get_path('module', 'mappingkit_openlayers') .'/openlayers.css');
    // css for theme
    // drupal_add_css(drupal_get_path('module', 'mappingkit') .'/themes/default/mappingkit-default.css');
    // add custom css
    $custom_css = drupal_get_path('module', 'mappingkit_openlayers') .'/openlayers.custom.css';
    if (file_exists($custom_css)) {
      drupal_add_css($custom_css);
    }
    
    // Add custom javascript code
    // prevent OpenLayers from overwriting $
    drupal_add_js("\nwindow.\$ = window.jQuery;\n", 'inline', $scope);
    
    // Set language of openlayers conform to drupal language
    $js = '';
    global $language;
    if ($language->language != 'en') {
      // create javascript object for this language
      $js .= "OpenLayers.Lang['{$language->language}'] = {};\n";
    }
    $js .= "OpenLayers.Lang.setCode('{$language->language}');\n";
    $js .= "MappingKit.settings.MAPPINGKIT_OPENLAYERS_RELEASE = ". MAPPINGKIT_OPENLAYERS_RELEASE .";\n";
    $js .= "var maps = new Array();\n"; // array of maps
    drupal_add_js($js, 'inline', $scope);
  }
}

/**
 * Render body of map.
 */
function mappingkit_openlayers_render_body(&$attrs) {
  global $_mappingkit_logger;
  
  $scope = variable_get('mappingkit_add_js_scope', DEFAULT_MAPPINGKIT_ADD_JS_SCOPE);
  mappingkit_openlayers_add_needed_js_files($attrs, $scope);
  
  // Remove mapclient class at this level to use it only for the map itself and
  // not for the statusbar/toolbar
  $classes = explode(' ', $attrs['class']);
  $classes = array_diff($classes, array('mapclient'));
  $attrs['class'] = implode(' ', $classes);
  
  // add openlayers class
  $attrs['class'] = (empty($attrs['class']) ? '' : $attrs['class'] . ' ') . 'openlayers';
  
  // include custom definition
  drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/MapProjection.js', 'module', $scope);
  //drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/LayerTree.js', 'module', 'header');
  
  if (isset($attrs['contexts'])) {
    // include custom definition
    if (count(explode(',', $attrs['contexts'])) > 1) {
      drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/ContextSwitcher.js', 'module', $scope);
    }
  }
  else if (isset($attrs['context'])) {
    $attrs['contexts'] = $attrs['context'];
  }
  
  if (isset($attrs['help'])) {
    // include help control
    $js_added = drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/Help.Control.js', 'extra', $scope);
  }

  //drupal_set_message('voor '.var_export(array('attrs' => $attrs), true));
  // include needed files for attributes (no return values)
  mapclient_attributes_invoke('openlayers', 'include_js', $attrs, $scope);
  //drupal_set_message('na '.var_export(array('attrs' => $attrs), true));
  
  if (isset($attrs['layers'])) {
    $layers = explode(',', $attrs['layers']);
    foreach ($layers as $layer_id) {
      if (isset($attrs[$layer_id])) {
        // process layer definition
        $layer = explode('|', $attrs[$layer_id]);
        
        switch ($layer[0]) {
        case 'georss':
          drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/GeoRSS.Format.js', 'module', $scope);
          break;
        case 'gpx':
          //drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/OpenLayers-2.7.Format.GPX.js', 'module', $scope);
          break;
          
        case 'wfs':
          drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/WFS.Layer.js', 'module', $scope);
          break;
          
        case 'tms':
          // set internal projection for tms layers
          $attrs['internalprojection'] = 'EPSG:900913';
          break;
          
        default:
          // todo: try to include plugin for layer type (check!!!)
          break;
        }
        //drupal_set_message('layer: '. var_export($layer, TRUE));
      }
    }
  }
  
  $baselayer_includes = array();
  
  $_mappingkit_logger->debug('attrs:', $attrs);
  if (isset($attrs['contexts'])) {
    $contexts = explode(',', $attrs['contexts']);
    foreach ($contexts as $context) {
      // wmc context or ows context?
      $tmp = explode('|', $context);
      $context = $tmp[0];
      $context_type = count($tmp) >= 2 ? $tmp[1] : 'wmc';
      // check if context is a local drupal node
      if (preg_match('/node\/(?P<nid>\d+)(\/revisions\/(?P<vid>\d+))?/', $context, $matches)) {
        // is a mapcontext node?
        $node = node_load($matches['nid'], isset($matches['vid']) ? $matches['vid'] : NULL);
        if ($node->type == 'mapcontext') {
          if (isset($node->extension['baselayers'])) {
            $baselayers = get_baselayers($node->extension['baselayers']);
            
            $_mappingkit_logger->debug('context baselayers:', $baselayers);
            
            foreach ($baselayers as $baselayer) {
              //$variant = $baselayer['variant'];
              //$baselayer_options = $baselayer['baselayer_options'];
              switch ($baselayer['baselayer_vendor']) {
              case 'google':
                if (!isset($baselayer_includes['google'])) {
                  $google_maps_key = variable_get('google_maps_key', DEFAULT_GOOGLE_MAPS_KEY);
                  $baselayer_includes['google'] = "http://maps.google.com/maps?file=api&amp;v=2&amp;key=$google_maps_key";
                }
                break;
              case 'multimap':
                if (!isset($baselayer_includes['multimap'])) {
                  $baselayer_includes['multimap'] = 'http://clients.multimap.com/API/maps/1.1/metacarta_04';
                }
                break;
              case 've':
                if (!isset($baselayer_includes['ve'])) {
                  //if (MAPPINGKIT_OPENLAYERS_RELEASE < 2.7) {
                  //  $baselayer_includes['ve'] = 'http://dev.virtualearth.net/mapcontrol/v3/mapcontrol.js';
                  //}
                  //else {
                    $baselayer_includes['ve'] = 'http://dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6.1';
                  //}
                }
                break;
              case 'yahoo':
                if (!isset($baselayer_includes['yahoo'])) {
                  $baselayer_includes['yahoo'] = 'http://api.maps.yahoo.com/ajaxymap?v=3.0&appid=euzuro-openlayers';
                }
                break;
              }
            }
            /*
            // check baselayers of mapcontext
            $baselayers = $node->extension['baselayers'];
            if ($baselayers == 'google') {
              if (!$google_src_set) {
                $google_maps_key = variable_get('google_maps_key', DEFAULT_GOOGLE_MAPS_KEY);
                $src = "http://maps.google.com/maps?file=api&amp;v=2&amp;key=$google_maps_key";
                drupal_set_html_head('<script type="text/javascript" src="'. $src .'"></script>');
                $_mappingkit_logger->debug('google included');
              }
            }
            */
          }
        }
      }
      else {
        // get map context xml
        $response = drupal_http_request(mapclient_get_url($context, TRUE), array(), 'GET');
        if ($response->code == 200) {
          // todo: context info: type (ows, wmc), baselayers, (max)extent, projection
          $context_info = array('data' => $response->data);
          // parse context info
          module_load_include('inc', 'mapcontext', 'wmc.parser');
          $parser = new WMC_Parser();
          $content = $parser->load($response->data);
          //drupal_set_message(var_export($content, true));
          // determine type of context
          if (strpos($response->data, '<OWSContext') !== FALSE) {
            $context_info['type'] = 'ows';
          }
          else { // ViewContext
            $context_info['type'] = 'wmc';
          }
          $attrs['_contexts'][] = $context_info;
        }
      }
    }
  }
  
  if (isset($attrs['baselayers'])) {
    // include scripts for base layers
    $baselayers = get_baselayers($attrs['baselayers']);
    $_mappingkit_logger->debug('baselayers', $baselayers);
    foreach ($baselayers as $baselayer) {
      $baselayer_vendor = $baselayer['baselayer_vendor'];
      //$variant = $baselayer['variant'];
      //$baselayer_options = $baselayer['baselayer_options'];
      switch ($baselayer_vendor) {
      case 'google':
        if (!isset($baselayer_includes['google'])) {
          $google_maps_key = variable_get('google_maps_key', DEFAULT_GOOGLE_MAPS_KEY);
          $baselayer_includes['google'] = "http://maps.google.com/maps?file=api&amp;v=2&amp;key=$google_maps_key";
        }
        break;
      case 'multimap':
        if (!isset($baselayer_includes['multimap'])) {
          $baselayer_includes['multimap'] = 'http://clients.multimap.com/API/maps/1.1/metacarta_04';
        }
        break;
      case 've':
        if (!isset($baselayer_includes['ve'])) {
          //if (MAPPINGKIT_OPENLAYERS_RELEASE < 2.7) {
          //  $baselayer_includes['ve'] = 'http://dev.virtualearth.net/mapcontrol/v3/mapcontrol.js';
          //}
          //else {
            $baselayer_includes['ve'] = 'http://dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6.1';
          //}
        }
        break;
      case 'yahoo':
        if (!isset($baselayer_includes['yahoo'])) {
          $baselayer_includes['yahoo'] = 'http://api.maps.yahoo.com/ajaxymap?v=3.0&appid=euzuro-openlayers';
        }
        break;
      case 'oam':
      case 'osm':
        // nothing to include
        break;
      }
    }
  }
  
  if (isset($attrs['streetview'])) {
    if (!isset($baselayer_includes['ve'])) {
      //if (MAPPINGKIT_OPENLAYERS_RELEASE < 2.7) {
      //  $baselayer_includes['ve'] = 'http://dev.virtualearth.net/mapcontrol/v3/mapcontrol.js';
      //}
      //else {
        $baselayer_includes['ve'] = 'http://dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6.1';
      //}
    }
    if (!isset($baselayer_includes['google'])) {
      // google was not yet included so do this now
      $google_maps_key = variable_get('google_maps_key', DEFAULT_GOOGLE_MAPS_KEY);
      $baselayer_includes['google'] = "http://maps.google.com/maps?file=api&amp;v=2&amp;key=$google_maps_key";
    }
  }

  if (!empty($attrs['edits']) && (isset($baselayer_includes['google']) || isset($baselayer_includes['ve']) || isset($baselayer_includes['yahoo']))) {
    // edits/vector layer doesn't display well over EPSG:4326 for google maps
    $attrs['internalprojection'] = 'EPSG:900913';
    $attrs['displayprojection'] = 'EPSG:4326';
  }
  
  foreach ($baselayer_includes as $vendor => $src) {
    //drupal_set_html_head('<script type="text/javascript" src="'. $src .'"></script>');
    mappingkit_add_js_external($src, $scope);
  }
  
  // check if proj4js is required
  if (!empty($attrs['internalprojection'])
    && !empty($attrs['displayprojection']) && $attrs['displayprojection'] != 'EPSG:4326') {
    // include proj4js
    $proj4js_path = variable_get('mappingkit_proj4js_path', MAPPINGKIT_DEFAULT_PROJ4JS_PATH);
    $scheme = parse_url($proj4js_path, PHP_URL_SCHEME);
    if (!empty($scheme)) {
      mappingkit_add_js_external($proj4js_path, $scope);
    }
    else {
      drupal_add_js($proj4js_path, 'extra', $scope);
    }
  }
  
  // default controls for OpenLayers map: Navigation, PanZoom, ArgParser, Attribution
  drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/ArgParser.Control.js', 'module', $scope);
  drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/Permalink.Control.js', 'module', $scope);
  drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/LoadingPanel.js', 'module', $scope);
  
  // Check controls
  if (!isset($attrs['controls'])) {
    $controls = array();
    
    if ($attrs['height'] > 200) {
      $controls[] = 'layerswitcher';
      $controls[] = 'scalebar';
      $controls[] = 'toolbar';
      $controls[] = 'statusbar';
    }
    // todo: ruler? info? context? print?
  }
  else {
    $controls = explode(',', $attrs['controls']);
    if (in_array('none', $controls)) {
      // no controls
    }
    else {
      // add controls by default?
    }
  }
  
  // open surrounding block
  $output = '';
  $output .= mapclient_common_open_outer_block($attrs);
  
  if (in_array('toolbar', $controls)) {
    // html for button panel
    $output .= '<div id="panels">';

    // default panel
    $output .= '<div id="'. $attrs['id'] .'-default-panel" class="panel">';
    $output .= '</div>';

    // mode panel
    $output .= '<div id="'. $attrs['id'] .'-mode-panel" class="panel">';
    $output .= '</div>';
    
    // action panel (save context, print)
    $output .= '<div id="'. $attrs['id'] .'-action-panel" class="panel">';
    $output .= '</div>';

    // edit panel
    $output .= '<div id="'. $attrs['id'] .'-edit-panel" class="olControlEditingToolbar panel">';
    $output .= '</div>';
    
    // search panel
    if (!empty($attrs['search'])) {
      $output .= '<div id="'. $attrs['id'] .'-search-panel" class="panel">';
      
      // render search box and button
      $search = mk_explode_options($attrs['search']);
      $search = array_merge(array('featurePrefix' => 'ms', 'version' => '1.1.0', 'maxFeatures' => 1), $search);
      $output .= '<form id="mappingkit-search-form">';
      $output .= '<div id="'. $attrs['id'] .'-search">';
      $output .= '<input type="edit" id="'. $attrs['id'] .'-search-input" title="'
        .t('Enter search value for !attr', array('!attr' => $search['attribute']))
        .'" style="float: left" onsubmit="javascript:$(\'div.mappingkit-search-button\').click(); return false;"/>';
      $output .= '</div>';
      $output .= '<div class="mappingkit-search-button" title="'
        . t('Search for !attr', array('!attr' => $search['attribute']))
        .'" onclick="javascript:MappingKit.search(this, '
        . str_replace("\"", "&quot;", drupal_to_js($search)) .')"></div>';
      $output .= '</form>';
      
      // close panel div
      $output .= '</div>';
    }
    
    // close panels
    $output .= '</div>';
    // clear div's
    $output .= '<div style="clear: both;"></div>';
  }

  // Html for map box (uses optional style parameter)
  // Add mapclient class to show the border around openlayers maps
  $output .= mapclient_common_open_map_block($attrs, array('class' => 'mapclient'));
  $output .= mapclient_common_close_map_block();
  
  if (in_array('statusbar', $controls)) {
    // Html for status bar below map (coordinates, scale, permalink)
    $output .= '<div id="'. $attrs['id'] .'-statusbar" class="statusbar-container" style="width: '
      . mapclient_format_style_size($attrs['_style']['width']) .';">'
               . '<div id="'. $attrs['id'] .'-statusbar-scale" class="statusbar statusbar-scale"></div>'
               . '<div class="statusbar statusbar-permalink"><a id="'. $attrs['id'] .'-statusbar-permalink">'. t('permalink') .'</a></div>'
               . '<div id="'. $attrs['id'] .'-statusbar-mouseposition" class="statusbar statusbar-mouseposition"></div>'
               . '<div id="'. $attrs['id'] .'-statusbar-projection" class="statusbar statusbar-projection"></div>'
               . '<div id="'. $attrs['id'] .'-statusbar-text" class="statusbar statusbar-text"></div>'
              .'</div>';
  }
  
  if (in_array('timeline', $controls)) {
    if (module_exists('maptimeline')) {
      // timeline div
      $output .= maptimeline_render_body($attrs);
    }
    else {
      drupal_set_message(t('The Timeline module of the Mapping kit is not enabled.'), 'warning');
    }
  }

  // render html for mapclient attributes (e.g. streetview, maptimeline, etc)
  $results = mapclient_attributes_invoke('render_html', 'openlayers', $attrs);
  $output .= implode("\n", $results);

  // error messages
  $output .= '<div class="mapclient-messages" style="display: block" ><ul id="mapclient-messages"></ul></div>';
  
  // popups?
  $output .= '<div id="docs"></div>';
  
  // render controls
  $output .= mapclient_common_render_controls($attrs);
  
  // end of outer block
  $output .= mapclient_common_close_outer_block();
  
  // remember maps for finish function
  //$_mapclient_maps[ $attrs['id'] ] = $attrs;
  
  return $output;
}

/**
 * Round up to a number of significant decimals.
 */
function ceil_sig($value, $sigFigs) {
  if ($value == 0) {
    return 0;
  }

  $sign = $value > 0 ? 1 : -1;
  $value = abs($value);
  
  $exponent = floor(log10($value) + 1);
  $significand = $value / pow(10, $exponent);
  if ($sign > 0) {
    $significand = ceil($significand * pow(10, $sigFigs)) / pow(10, $sigFigs);
  }
  else {
    $significand = floor($significand * pow(10, $sigFigs)) / pow(10, $sigFigs);
  }
  $value = $significand * pow(10, $exponent);
  
  return($sign * $value);
}

/**
 * Finish for all maps.
 */
function mappingkit_openlayers_finish($map_count) {
  //NB: next message appears on next page
  //drupal_set_message("openlayers_finish: $map_count");
  global $_mapclient_maps;
  global $_mappingkit_logger;
  
  $tilesize = variable_get('mappingkit_openlayers_tile_size', MAPPINGKIT_DEFAULT_OPENLAYERS_TILE_SIZE);
  
  $js .= "\n//<!--\n";
  $js .= "MappingKit.init = function() {\n";
  
  $js .= "OpenLayers.DOTS_PER_INCH = ". variable_get('mappingkit_openlayers_resolution', MAPPINGKIT_DEFAULT_OPENLAYERS_RESOLUTION) .";\n";
  $js .= "OpenLayers.IMAGE_RELOAD_ATTEMPTS = 1;\n"; // avoid pink tiles
  if (0 != variable_get('mapclient_load_errors', DEFAULT_MAPCLIENT_LOAD_ERRORS)) {
    $js .= "OpenLayers.Util.onImageLoadErrorColor = 'transparent';\n";
    $js .= "OpenLayers.Util.onImageLoadErrorStandard = OpenLayers.Util.onImageLoadError;\n";
    $js .= "OpenLayers.Util.onImageLoadError = function() { OpenLayers.Util.onImageLoadErrorStandard.apply(this); MappingKit.onImageLoadError.apply(this); };\n";
  }
  // A string to be used as the base for images for controls and the like. Should end with '/'
  //$js .= "OpenLayers.ImgPath = '../carto/ol/img/';
  
  // proxy host
  //$js .= "OpenLayers.ProxyHost = '" . mapclient_get_url(drupal_get_path('module', 'mappingkit') .'/proxy.php') ."?url=';\n";
  $js .= "OpenLayers.ProxyHost = '" . mapclient_get_url(variable_get('mapclient_proxy', 'proxy.php')) ."?url=';\n";
  
  $js .= "var options = {};\n";
  //{ resolutions:[1.40625,0.703125,0.3515625,0.17578125,0.087890625,0.0439453125,0.02197265625,0.010986328125,0.0054931640625,0.00274658203125,0.00137329101]};\n";
  $zoom = 1; // initial zoom
  
  // process all maps
  foreach ($_mapclient_maps as $map_id => $map) {
    if ($map['mapclient'] != 'openlayers') {
      continue;
    }
    
    $_mappingkit_logger->info('map for openlayers_finish: '. var_export($map, TRUE));
    
    $js .= "var client = {google: null, ve: null};\n";
    
    $js .= "var map_options = {};\n";
    $js .= "map_options.numZoomLevels = 20;\n";
    if ($tilesize != MAPPINGKIT_DEFAULT_OPENLAYERS_TILE_SIZE) {
      $js .= "map_options.tileSize = new OpenLayers.Size($tilesize, $tilesize);\n";
    }
    
    // todo: determine spherical mercator setting
    $sphericalMercator = isset($map['sphericalMercator']) ? $map['sphericalMercator'] : 'false';
    
    $display_projection = NULL;
    if (isset($map['internalprojection'])) {
      $map['maxextent'] = '-20037508.34,-20037508.34,20037508.34,20037508.34';
      $map['maxresolution'] = 2 * 20037508.34 / 256;
      $map_projection = 'EPSG:4326';
      $sphericalMercator = 'true';
      
      $display_projection = 'EPSG:4326';
    }
    else if (!empty($map['projection'])) {
      // set map (display) projection
      $map_projection = $map['projection'];
      
      $display_projection = $map_projection;
    }
    else {
      $map_projection = 'EPSG:4326';
      
      $display_projection = $map_projection;
    }
    
    if (isset($map['displayprojection'])) {
      $display_projection = $map['displayprojection'];
      
      if ($display_projection != 'EPSG:4326') {
        // check that definition for display_projection is available
        $proj4text = mappingkit_get_proj4text($display_projection);
        if (!empty($proj4text)) {
          $js .= "Proj4js.defs['$display_projection'] = '$proj4text';\n";
        }
      }
    }
    
    $numdigits = 5;
    if (isset($map['internalprojection'])) {
      $js .= "map_options.projection = new OpenLayers.Projection('EPSG:900913');\n";
      $js .= "map_options.displayProjection = new OpenLayers.Projection('EPSG:4326');\n";
      $js .= "map_options.units = 'm';\n";
      $js .= "map_options.maxExtent = new OpenLayers.Bounds({$map['maxextent']});\n";
    }
    else {
      $js .= "map_options.projection = new OpenLayers.Projection('{$map_projection}');\n";
      if ($map_projection == 'EPSG:4326') {
        $js .= "map_options.units = 'degrees';\n";
      }
      else {
        $js .= "map_options.units = 'm';\n";
        $numdigits = 0;
      }
    }
    /*
    $numdigits = 0;
    if (isset($map['projection'])) {
      switch ($map['projection']) {
      case 'EPSG:900913':
        $sphericalMercator = 'true';
        $js .= "map_options.projection = new OpenLayers.Projection('EPSG:900913');\n";
        $js .= "map_options.displayProjection = new OpenLayers.Projection('EPSG:4326');\n";
        //$js .= "map_options.maxExtent = new OpenLayers.Bounds(-10018754, -10018754, 10018754, 10018754);\n";
        $map['maxextent'] = '-20037508.34, -20037508.34, 20037508.34, 20037508.34';
        $map['maxresolution'] = 2*20037508.34/256;
        $js .= "map_options.maxExtent = new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34);\n";
        $js .= "map_options.units = 'm';\n";
        $numdigits = 5;
        break;
      case 'EPSG:4326':
        $js .= "map_options.units = 'degrees';\n";
        $numdigits = 5;
        break;
      default:
        $js .= "map_options.units = 'm';\n";
        break;
      }
    }
    */
    
    if (isset($map['extent'])) {
      $initial_extent = $map['extent'];
    }
    
    if (isset($map['restrictedextent'])) {
      $map['maxextent'] = $map['restrictedextent'];
    }
    elseif (isset($map['extent'])) {
      if (!isset($map['maxextent'])) {
        // use extent for maxextent
        $map['maxextent'] = $map['extent'];
      }
    }
    
    // todo: coherent use of maxextent/resolutions/projections/restrictedextent/etc/numresolutions
    // priority: resolutions > extent
    if (!empty($map['resolutions'])) {
      // get resolutions for map
      $js .= "map_options.resolutions = [{$map['resolutions']}];\n";
    }
    
    if (isset($map['maxresolution'])) {
      $maxResolution = $map['maxresolution'];
    }
    else {
      $maxResolution = NULL;
    }
    
    if (isset($map['restrictedextent'])) {
      $restrictedextent = explode(',', $map['restrictedextent']);
      
      if (!$initial_extent) {
        // NOTE: next line is required for OL 2.6, but not later
        $initial_extent = $map['restrictedextent'];
      }
      //drupal_set_message("restricted extent is $initial_extent", 'error');
      
      $js .= "var extent = new OpenLayers.Bounds({$map['restrictedextent']});\n";
      $js .= "if (map_options.displayProjection) { extent.transform(map_options.displayProjection, map_options.projection); }\n";
      $js .= "map_options.restrictedExtent = extent;\n";
      
      // get ideal resolution
      $idealResolution = max(($restrictedextent[2] - $restrictedextent[0])/$map['width']
        , ($restrictedextent[3] - $restrictedextent[1])/$map['height']);
      if (isset($maxResolution)) {
        // maybe because of EPSG:900913
        
        // Adapt $maxResolution for restricted extent.
        // Find resolution >= idealresolution
        // zoom n = (ln(maxreso) - ln(reso))/ln(2)
        //$js .= "var idealResolution = Math.max(map_options.restrictedExtent.getWidth()/{$map['width']}, map_options.restrictedExtent.getHeight()/{$map['height']});\n";
        //$js .= "map_options.maxResolution = $maxResolution / Math.pow(2, Math.floor((Math.log($maxResolution) - Math.log(idealResolution))/Math.log(2)));\n";
        
        $js .= "map_options.maxResolution = $maxResolution;\n";
      }
      else {
        // for baselayers in EPSG:4326
        $maxResolution = 1.40625;
        $js .= "map_options.maxResolution = $maxResolution;\n";
      }
      
      //$js .= "map_options.maxExtent = extent;\n";
    }
    elseif (isset($map['maxextent'])) {
      $maxextent = explode(',', $map['maxextent']);
      if (!$maxResolution) {
        $maxResolution = max(($maxextent[2] - $maxextent[0])/$map['width']
          , ($maxextent[3] - $maxextent[1])/$map['height']);
      }
      
      $js .= "map_options.maxExtent = new OpenLayers.Bounds({$map['maxextent']});\n";
      $js .= "map_options.maxResolution = $maxResolution;\n";
    }
    else {
      if (!$maxResolution) {
        $maxResolution = max(360/$map['width'], 180/$map['height']);
      }
      
      $js .= "map_options.maxExtent = new OpenLayers.Bounds(-180, -90, 180, 90);\n";
      $js .= "map_options.maxResolution = $maxResolution;\n";
    }
    
    /*
    if (isset($map['projection'])) {
      $js .= "map_options.projection = new OpenLayers.Projection('{$map['projection']}');\n";
    }
    */
    
    if (isset($map['title'])) {
      $js .= "map_options.title = '{$map['title']}';\n";
    }
    
    $zoom = isset($map['zoom']) ? $map['zoom'] : 5;
    $center = isset($map['center']) ? explode(',', $map['center']) : array(0, 0);
    
    if (!isset($map['controls'])) {
      // default controls
      $controls = array();
      if ($map['height'] > 200) {
        $controls[] = 'layerswitcher';
        $controls[] = 'scalebar';
        $controls[] = 'toolbar';
        $controls[] = 'statusbar';
      }
    }
    else {
      $controls = explode(',', (isset($map['controls']) ? $map['controls'] : ''));
    }
    if (in_array('none', $controls)) {
      $js .= "map_options.controls = [];\n";
    }
    
    $default_options = array('cookies' => 'true', 'navigable' => 'true');
    if (!empty($map['options'])) {
      // get map options
      $map_options = mk_explode_options($map['options']);
    }
    else {
      $map_options = array();
    }
    // combine with default map options
    $options = $map_options + $default_options;
    //drupal_set_message(var_export(array('map' => $map_options,'default' => $default_options, 'options' => $options), TRUE));
    
    $js .= "var controls = [new OpenLayers.Control.Attribution()];\n";
    if ($options['navigable'] == 'true') {
      // include navigation and panzoom controls (left-top)?
      $js .= "controls.push(new OpenLayers.Control.Navigation());\n";
      //$js .= "controls.push(new OpenLayers.Control.PanZoom());\n"; // separate images (7) for zoom/pan
      $js .= "controls.push(new OpenLayers.Control.PanPanel());\n"; // single image for pan controls
      $js .= "controls.push(new OpenLayers.Control.ZoomPanel());\n"; // single image for zoom controls
      $js .= "controls.push(new OpenLayers.Control.LoadingPanel());\n";
    }
    $js .= "var argParserOptions;\n";
    //$js .= "argParserOptions.args = document.getElementById('edit-submitted-kilometerhokken-map-status').value;\n";
    $js .= "controls.push(new MappingKit.Control.ArgParser(argParserOptions));\n";
    $js .= "map_options.options = {useCookies: {$options['cookies']}};\n";
    $js .= "map_options.controls = controls;\n";
    
    // (1) create map object
    $_SESSION['firebug'] && $js .= "OpenLayers.Console.debug('map_options', map_options);\n";
    $js .= "map_options.theme = null;\n";
    $js .= "var map = new OpenLayers.Map('". $map['id'] ."', map_options);\n";
    $js .= "MappingKit.MapClient.OpenLayers.registerMap('" . $map['id'] . "', map);\n";
    $js .= "client.map = map;\n";
    $js .= "client.baselayers = [];\n";
    $js .= "maps[maps.length] = map;\n";
    
    // add legend here to have new layers show their legendurl
    $js .= "var legend = new MappingKit.Legend(map, 'legend', {title: Drupal.t('Legend')});\n";
    
    // (2) add the baselayers
    if (isset($map['baselayers'])) {
      $baselayers = get_baselayers($map['baselayers']);
      foreach ($baselayers as $baselayer) {
        $baselayer_vendor = $baselayer['baselayer_vendor'];
        $variant = $baselayer['variant'];
        $baselayer_options = $baselayer['baselayer_options'];
        switch ($baselayer_vendor) {
        case 'google':
          // 20 zoom levels works fine
          // alternatives: Streets=(), Satellite=G_SATELLITE_MAP, Hybrid=G_HYBRID_MAP
          $variants = array('default' => 'G_HYBRID_MAP', 'streets' => 'G_NORMAL_MAP', 'satellite' => 'G_SATELLITE_MAP', 'hybrid' => 'G_HYBRID_MAP', 'physical' => 'G_PHYSICAL_MAP');
          $type = isset($variants[$variant]) ? $variants[$variant] : $variants['default'];
          $js .= "if (typeof $type != 'undefined') {\n";
          $js .= get_baselayer_options_js($baselayer_options);
          $js .= "layer_options.type = {$type};\n";
          $js .= "layer_options.sphericalMercator = {$sphericalMercator};\n";
          $js .= "client.google = new OpenLayers.Layer.Google('Google - $variant', layer_options);\n";
          $js .= "client.baselayers[client.baselayers.length] = client.google;\n";
          $js .= "}\n";
          break;
        case 've':
          // Roads=VEMapStyle.Road, Aerial=VEMapStyle.Aerial, VEMapStyle.Hybrid
          $variants = array('default' => 'Hybrid', 'streets' => 'Road', 'satellite' => 'Aerial', 'hybrid' => 'Hybrid');
          $type = isset($variants[$variant]) ? $variants[$variant] : $variants['default'];
          $js .= "if (typeof VEMapStyle != 'undefined') {\n";
          $js .= get_baselayer_options_js($baselayer_options);
          $js .= "layer_options.type = VEMapStyle.$type;\n";
          $js .= "layer_options.sphericalMercator = {$sphericalMercator};\n";
          $js .= "client.ve = new OpenLayers.Layer.VirtualEarth('Virtual Earth - $variant', layer_options);\n";
          $js .= "client.baselayers[client.baselayers.length] = client.ve;\n";
          $js .= "}\n";
          break;
        case 'yahoo':
          // (), YAHOO_MAP_SAT, YAHOO_MAP_HYB
          $variants = array('default' => 'YAHOO_MAP_HYB', 'satellite' => 'YAHOO_MAP_SAT', 'hybrid' => 'YAHOO_MAP_HYB');
          $type = isset($variants[$variant]) ? $variants[$variant] : $variants['default'];
          $js .= "if (typeof $type != 'undefined') {\n";
          $js .= get_baselayer_options_js($baselayer_options);
          $js .= "layer_options.type = {$type};\n";
          $js .= "layer_options.sphericalMercator = {$sphericalMercator};\n";
          $js .= "client.yahoo = new OpenLayers.Layer.Yahoo('Yahoo - $variant', layer_options);\n";
          $js .= "client.baselayers[client.baselayers.length] = client.yahoo;\n";
          $js .= "}\n";
          break;
        case 'oam':
          $js .= get_baselayer_options_js();
          $js .= "layer_options.type = 'png';\n";
          $js .= "layer_options.getURL = osm_getTileURL;\n";
          $js .= "layer_options.displayOutsideMaxExtent = true;\n";
          $js .= "layer_options.attribution = 'OpenAerialMap';\n";
          //$js .= "client.osm = new OpenLayers.Layer.TMS('OpenStreetMap - Tiles@Home', 'http://tah.openstreetmap.org/Tiles/tile.php/', layer_options);\n";
          $js .= "client.oam = new OpenLayers.Layer.TMS('OpenAerialMap', 'http://tile.openaerialmap.org/tiles/1.0.0/openaerialmap-900913/', layer_options);\n";
          $js .= "client.baselayers[client.baselayers.length] = client.oam;\n";
          break;
        case 'osm':
          $js .= get_baselayer_options_js();
          $js .= "layer_options.type = 'png';\n";
          $js .= "layer_options.getURL = osm_getTileURL;\n";
          $js .= "layer_options.displayOutsideMaxExtent = true;\n";
          $js .= "layer_options.attribution = 'OpenStreetMap';\n";
          //$js .= "client.osm = new OpenLayers.Layer.TMS('OpenStreetMap - Tiles@Home', 'http://tah.openstreetmap.org/Tiles/tile.php/', layer_options);\n";
          $js .= "client.osm = new OpenLayers.Layer.TMS('OpenStreetMap', 'http://tile.openstreetmap.org/', layer_options);\n";
          $js .= "client.baselayers[client.baselayers.length] = client.osm;\n";
          break;
        case 'none':
          //$js .= "map.addLayer(new OpenLayers.Layer.Image('None', 'white.png', map.maxExtent, new OpenLayers.Size(1, 1)));\n";
          $js .= "client.none = new OpenLayers.Layer(Drupal.t('None'), {isBaseLayer: true});\n";
          $js .= "client.baselayers[client.baselayers.length] = client.none;\n";
          break;
        }
      }
      /* */
      $js .= "if (\$defined(client.google) && \$defined(client.ve)) {\n";
      // give virtual earth same resolutions/max_zoom_level as google
      $js .= "  client.ve.RESOLUTIONS = client.google.RESOLUTIONS;\n";
      $js .= "  client.ve.MAX_ZOOM_LEVEL = client.google.MAX_ZOOM_LEVEL;\n";
      $js .= "}\n";
      /* */
      // now add all baselayers
      $js .= "for (var i = 0; i < client.baselayers.length; i++) {\n";
      $js .= "  map.addLayer(client.baselayers[i]);\n";
      $js .= "}\n";
    }
    
    // start selectable layers
    $js .= "var selectableLayers = [];\n";
    
    // (3) contexts
    if (isset($map['contexts'])) {
      $contexts = explode(',', $map['contexts']);
      foreach ($contexts as $context) {
        // wmc context or ows context?
        $tmp = explode('|', $context);
        $context = $tmp[0];
        $context_type = count($tmp) >= 2 ? $tmp[1] : 'wmc';
        // add context to map
        if (preg_match('/node\/(?P<nid>\d+)(\/revisions\/(?P<vid>\d+))?/', $context, $matches)) {
          // is a mapcontext node?
          $node = node_load($matches['nid'], isset($matches['vid']) ? $matches['vid'] : NULL);
          if ($node->type == 'mapcontext') {
            $context = "node/{$node->nid}/context";
            $url = mapclient_get_url($context);
            
            if (is_numeric($node->dminx)) {
              $initial_extent = $node->dminx .','. $node->dminy .','. $node->dmaxx .','. $node->dmaxy;
            }
          }
          else {
            drupal_set_message(t('Node is not a map context'), 'error');
            continue;
          }
        }
        else {
          $url = mapclient_get_url($context);
        }
        
        // save layer info before merging map context
        $js .= "var tmp = map.getNumLayers() == 1 && map.layers[0].CLASS_NAME == 'OpenLayers.Layer';\n";
        
        $js .= "var wmc_options = {};\n";
        if (! empty($map['_contexts'][0])) {
          // use context which was read server-side
          // b) use context info (responseText in context_info.data)
          $js .= "var context_info = ". drupal_to_js($map['_contexts'][0]) .";\n";
        }
        else {
          // read context client-side
          
          // a) use request (read xml into request.transport.responseXML
          $js .= "var url = '". $url ."';\n";
          
          // create map from context by synchronous load
          $js .= "var request = new OpenLayers.Ajax.Request(url, {method: 'get', asynchronous: false, parameters: {} });\n";
          $js .= "var context_response = request.transport.responseXML;\n";
          //$js .= "var text = '';\n";
          //$js .= "if (request && request.transport) {\n";
          //$js .= "  text = request.transport.responseText;\n";
          //$js .= "}\n";
          
          $js .= "
          var context_info = { data: request.transport.responseXML };
          if (request.transport.responseXML.documentElement.tagName == 'OWSContext') {
            context_info.type = 'ows';
          }
          else { // wmc context
            context_info.type = 'wmc';
          }
          ";
        }
        
        $js .= "
          if (context_info.type == 'ows') {
            wmc_options.layerParams = {'OpenLayers.Layer.WMS': {'TRANSPARENT': 'TRUE'}};
            wmc_options.layerOptions = {'OpenLayers.Layer.WMS': {}};
            var wmc = new OpenLayers.Format.OWSContext(wmc_options);
          }
          else { // wmc context
            wmc_options.layerOptions = {isBaseLayer: false, buffer: 0, displayInLegend: true};
            var wmc = new OpenLayers.Format.MapContext(wmc_options);
          }
        ";
        
        // merge layers from context
        // todo: assert responseXML?
        //$js .= "var wmc_info = {};\n";
        //$js .= "wmc.parser = new OpenLayers.Format.WMC.vMapContext_Parser()\n";
        //$js .= "wmc.parser.read_wmc_General(wmc_info, request.transport.responseXML.getElementsByTagName('General')[0]);\n";
        // wmc_info: abstract, baselayers, (initial)bounds, maxExtent, (display)projection, title
        
        $js .= "wmc.read(context_info.data, {'map': client.map});\n";
        
        // set baselayer to first one in context
        $js .= "if (tmp && wmc.context.layers.length > 0 && wmc.context.layers[0].isBaseLayer) { map.setBaseLayer(map.layers[1]); }\n";
        
        $_SESSION['firebug'] && $js .= "OpenLayers.Console.debug('wmc', wmc);\n";
        /*
        $js .= "if (!map.getCenter()) {\n";
        $js .= "  map.setCenter(wmc.context.bounds.getCenterLonLat());\n";
        $js .= "}\n";
        */
      }
      // make sure we have a baseLayer in case the default openlayers 
      // distribution is used without wmc patches
      $js .= "if (map.layers.length > 0 && !map.layers[0].isBaseLayer) { map.layers[0].setOpacity(0.5); map.setBaseLayer(map.layers[0]); }\n";
      
      // todo: check for (selectable) vector layers (WFS, KML)
      $js .= "if (wmc.context.layers.length > 0) {
      for (var i = 0, len = wmc.context.layers.length; i < len; i++) {
        var layer = wmc.context.layers[i];
        if (layer instanceof OpenLayers.Layer.Vector) {
          selectableLayers.push(layer);
          layer.events.on({'featureselected': MappingKit.handleFeatureSelected, 'featureunselected': MappingKit.handleFeatureUnselected});
        }
        else if (layer instanceof OpenLayers.Layer.WMS) {
          // todo: check for feature info
        }
      }
    }
      ";
    }
    
    // (4) layers
    $layer_count_wms = 0;
    if (isset($map['layers'])) {
      $layers = explode(',', $map['layers']);
      
      foreach ($layers as $layer_id) {
        if (isset($map[$layer_id])) {
          $layer = explode('|', $map[$layer_id]);
          
          $js .= "var layer_options = {};\n";
          if ($sphericalMercator == 'true') {
            // vector layers will have lat/lon projection
            $js .= "layer_options.projection = new OpenLayers.Projection('EPSG:4326');\n";
          }
          
          switch ($layer[0]) {
          case 'feed': // feed|url
            $url = mapclient_get_url($layer[1]);
            $js .= "MappingKit.feedMap($url, {$map['id']});\n";
            break;
            
          case 'georss': // georss|url|title|options
            if ($maxResolution) {
              $js .= "layer_options.maxResolution = $maxResolution;\n";
            }
            $title = t('GeoRSS');
            $url = mapclient_get_url($layer[1]);
            $icon_url = NULL;
            if (!empty($layer[2])) {
              $title = $layer[2];
              // title is include in layer specs; do NOT use title from rss feed
              $js .= "layer_options.useFeedTitle = false;\n";
            }
            if (!empty($layer[3])) {
              $extras = mk_explode_options($layer[3]);
              //drupal_set_message('georss extras:' . var_export($extras, TRUE));
              foreach ($extras as $key => $value) {
                switch ($key) {
                case 'icon':
                  $icon_url = mapclient_get_url($value, TRUE);
                  break;
                default:
                  $js .= mk_add_properties_js('layer_options', $key, $value);
                  break;
                }
              }
              //$js .= mk_add_properties_js('layer_options', $extras);
            }
            else {
              $extras = array();
            }
            
            $title = js_ize($title); // check title for '

            // default georss layer
            //$js .= "var georss = new OpenLayers.Layer.GeoRSS('{$title}', '{$url}', layer_options);\n";
            
            $js .= "layer_options.format = MappingKit.Format.GeoRSS;\n";
            if ($extras['icon'] == 'thumbnail') {
              $js .= "var style = new OpenLayers.Style({externalGraphic: '\${thumbnail}'});\n";
              $js .= "style.addRules([new OpenLayers.Rule({symbolizer: {pointRadius: 20}})]);\n";
              $js .= "layer_options.styleMap = new OpenLayers.StyleMap({'default': style, 'select': new OpenLayers.Style({pointRadius: 35})});\n";
            }
            else {
              if (!empty($icon_url)) {
                $js .= "var style = new OpenLayers.Style({externalGraphic: '$icon_url', graphicWidth: 21, graphicHeight: 21, graphicXOffset: -10.5, graphicYOffset: -10.5});\n";
              }
              else {
                // default/traditional marker style
                $js .= "var style = new OpenLayers.Style({externalGraphic: OpenLayers.Util.getImagesLocation() + 'marker.png', graphicWidth: 21, graphicHeight: 25, graphicXOffset: -10.5, graphicYOffset: -12.5});\n";
              }
              $js .= "layer_options.styleMap = new OpenLayers.StyleMap({'default': style});\n";
            }
            $js .= "var georss = new OpenLayers.Layer.GML('{$title}', '{$url}', layer_options);\n";
            $js .= "map.addLayer(georss);\n";
            
            $js .= "selectableLayers.push(georss);\n"; // add to selectablelayers
            $js .= "georss.events.on({'featureselected': MappingKit.handleFeatureSelected, 'featureunselected': MappingKit.handleFeatureUnselected});\n";
            
            //$js .= "MappingKit.selectControls[MappingKit.selectControls.length] = selectControl;\n";
            
            if (!empty($map['timeline'])) {
              $timeline = mk_explode_options($map['timeline']);
              //$tl_layers = array_keys($timeline['layers']);
              //if (in_array($layer_id, $tl_layers)) {
              if (!empty($extras['timeline'])) {
                //$js .= "georss._timeline = " . drupal_to_js($timeline['layers'][$layer_id]) . ";\n";
                //$js .= "georss.events.register('loadend', georss, MappingKit.Timeline.onLoadEnd);\n";
                $js .= "MappingKit.Timeline.linkMapLayer('{$map['id']}', georss, " . drupal_to_js($extras['timeline']) . ");\n";
              }
            }
            break;
            
          case 'gml': // gml|url|title
            $url = mapclient_get_url($layer[1]);
            $title = js_ize(isset($layer[2]) && $layer[2] != '' ? $layer[2] : 'GML');
            
            $js .= "map.addLayer(new OpenLayers.Layer.GML('{$title}', '{$url}', layer_options));\n";
            break;
            
          case 'gpx': // gpx|url|title|marker|options
            $title = t('GPX');
            $url = mapclient_get_url($layer[1]);
            
            $js .= "layer_options.format = OpenLayers.Format.GPX;\n";
            // todo: layer_options.projection = map.displayProjection;\n?
            if (isset($layer[2]) && $layer[2] != '') {
              $title = $layer[2];
              // title is include in layer specs; do NOT use title from rss feed
              $js .= "layer_options.useFeedTitle = false;\n";
            }
            
            // default style options
            $js .= "var style_options = {};\n";
            $js .= "style_options.default = OpenLayers.Util.extend({}, OpenLayers.Feature.Vector.style['default']);\n";
            $js .= "style_options.select = OpenLayers.Util.extend({}, OpenLayers.Feature.Vector.style['select']);\n";
            
            // marker parameter?
            if (!empty($layer[3])) {
              $icon_url = mapclient_get_url($layer[3]);
              
              $js .= "style_options.default.externalGraphic = '$icon_url';\n";
              $js .= "style_options.default.graphicWidth = 16;\n";
              $js .= "style_options.default.graphicHeight = 16;\n";
              $js .= "style_options.default.graphicOpacity = 1;\n";
              
              $js .= "style_options.select.graphicWidth = 32;\n";
              $js .= "style_options.select.graphicHeight = 32;\n";
            }
            
            if (isset($layer[4])) {
              $extras = mk_explode_options($layer[4]);
              foreach ($extras as $key => $value) {
                switch ($key) {
                case 'icon':
                  if ($value[0] == '$') {
                    $value = substr($value, 1);
                    $js .= "style_options.default.externalGraphic = '\${{$value}}';\n";
                  }
                  else if ($value[0] == 'thumbnail') {
                    $value = substr($value, 1);
                    $js .= "style_options.default.externalGraphic = '\${thumbnail}';\n";
                  }
                  else {
                    $icon_url = mapclient_get_url($value, TRUE);
                    $js .= "style_options.default.externalGraphic = '$icon_url';\n";
                  }
                  break;
                case 'style':
                  $js .= mk_add_properties_js('style_options.default', $value);
                  break;
                case 'select':
                  $js .= mk_add_properties_js('style_options.select', $value);
                  break;
                default:
                  $js .= mk_add_properties_js('layer_options', $key, $value);
                  break;
                }
              }
              //drupal_set_message('extras:'. var_export($extras, TRUE));
              //$js .= mk_add_properties_js('layer_options', $extras);
            }
            $js .= "var styles = {};\n";
            $js .= "styles.default = new OpenLayers.Style(style_options.default);\n";
            $js .= "styles.select = new OpenLayers.Style(style_options.select);\n";
            $js .= "layer_options.styleMap = new OpenLayers.StyleMap(styles);\n";
            
            $title = js_ize($title); // check title for '
            $js .= "var gpx = new OpenLayers.Layer.GML('{$title}', '{$url}', layer_options);\n";
            //$js .= "gpx.events.register('loadend', gpx, MappingKit.onLoadEndGPX);\n";
            $js .= "map.addLayer(gpx);\n";
            
            $js .= "selectableLayers.push(gpx);\n"; // add to selectablelayers
            $js .= "gpx.events.on({'featureselected': MappingKit.handleFeatureSelected, 'featureunselected': MappingKit.handleFeatureUnselected});\n";
            
            break;
            
          case 'image': // image|url|title|size
            $title = t('Image');
            $url = mapclient_get_url($layer[1]);
            if (!empty($layer[2])) {
              $title = $layer[2];
            }
            $title = js_ize($title); // check title for '
            $js .= "var image = new OpenLayers.Layer.Image('{$title}', '{$url}', map_options.maxExtent, new OpenLayers.Size({$layer[3]}), layer_options);\n";
            $js .= "map.addLayer(image);\n";
            break;
            
          case 'kml': // kml|url|title|options
            $title = t('KML');
            $url = mapclient_get_url($layer[1]);
            // title
            if (isset($layer[2]) && $layer[2] != '') {
              $title = $layer[2];
            }
            // options
            $hover = 'false';
            if (isset($layer[3]) && $layer[3] != '') {
              $extras = mk_explode_options($layer[3]);
              
              $js .= "layer_options.formatOptions = {};\n";
              foreach ($extras as $key => $value) {
                switch ($key) {
                case 'kmlns':
                  // when kml doesn't have a namespace, use kmlns=*
                  $js .= "layer_options.formatOptions.$key = '$value';\n";
                  break;
                case 'extractStyles':
                case 'extractAttributes':
                  $js .= "layer_options.formatOptions.$key = $value;\n";
                  break;
                case 'hover':
                  $hover = $value;
                  break;
                case 'style':
                  $js .= "layer_options.style = OpenLayers.Util.extend({}, OpenLayers.Feature.Vector.style['default']);\n";
                  $js .= mk_add_properties_js('layer_options.style', $value);
                  break;
                case 'styleMap':
                  $js .= "var styleMap_options = {};\n";
                  $js .= mk_add_properties_js('styleMap_options', $value);
                  $js .= "layer_options.styleMap = new OpenLayers.StyleMap(styleMap_options);\n";
                  break;
                default:
                  $js .= mk_add_properties_js('layer_options', $key, $value);
                  break;
                }
              }
            }
            else {
              // defaults
              $js .= "layer_options.formatOptions = { extractStyles: true, extractAttributes: true }\n;";
            }
            $js .= "layer_options.format = OpenLayers.Format.KML;\n";
            $js .= "var kml = new OpenLayers.Layer.GML('$title', '$url', layer_options);\n";
            $js .= "map.addLayer(kml);\n";
            
            /*
            $js .= "var selectControl = new OpenLayers.Control.SelectFeature(kml, {onSelect: MappingKit.onFeatureSelect, onUnselect: MappingKit.onFeatureUnselect, hover: $hover});\n";
            $js .= "map.addControl(selectControl);\n";
            $js .= "selectControl.activate();\n";
            $js .= "MappingKit.selectControls[MappingKit.selectControls.length] = selectControl;\n";
            */
            $js .= "selectableLayers.push(kml);\n"; // add to selectablelayers
            $js .= "kml.events.on({'featureselected': MappingKit.handleFeatureSelected, 'featureunselected': MappingKit.handleFeatureUnselected});\n";
            
            break;
            
          case 'tilecache': // tilecache|url|title_in_layer_switcher|layername|options (e.g. resolutions)
            $title = js_ize(empty($layer[2]) ? 'TileCache' : $layer[2]);
            if (!empty($layer[4])) {
              $extras = mk_explode_options($layer[4]);
              //drupal_set_message('extras = '. var_export($extras, TRUE));
              foreach ($extras as $key => $value) {
                switch ($key) {
                case 'resolutions':
                  $js .= "layer_options.serverResolutions = $value;\n";
                  break;
                  
                default:
                  // do nothing
                  break;
                }
              }
            }
            $urls = explode(',', $layer[1]);
            $js .= "var lyr = new OpenLayers.Layer.TileCache('{$title}', ['" . implode("'\n,'", $urls) . "'], '{$layer[3]}', layer_options);\n";
            $js .= "map.addLayer(lyr);\n";
            break;
          
          case 'tms': // tms|url|title|options (attribution,etc?)
            $title = js_ize(isset($layer[2]) && $layer[2] != '' ? $layer[2] : 'TMS');
            $js .= "layer_options.type = 'png';\n";
            $js .= "layer_options.getURL = osm_getTileURL;\n";
            $js .= "layer_options.displayOutsideMaxExtent = true;\n";
            $js .= "layer_options.attribution = '$title';\n";
            $js .= "layer_options.transitionEffect = 'resize';\n";
            if ($sphericalMercator == 'true') {
              // wms layers have google projection
              $js .= "layer_options.displayProjection = new OpenLayers.Projection('EPSG:4326');\n";
              $js .= "layer_options.projection = new OpenLayers.Projection('EPSG:900913');\n";
            }
            if (!empty($layer[3])) {
              $extras = mk_explode_options($layer[3]);
              //drupal_set_message('extras = '. var_export($extras, TRUE));
              foreach ($extras as $key => $value) {
                switch ($key) {
                case 'attribution':
                  $js .= "layer_options.attribution = '$value';\n";
                  break;
                }
              }
            }
            $urls = explode(',', $layer[1]);
            $js .= "var lyr = new OpenLayers.Layer.TMS('{$title}', ['" . implode("'\n,'", $urls) . "'], layer_options);\n";
            $js .= "map.addLayer(lyr);\n";
            break;
            
          case 'wms': // wms|url|title|names|options
            $layer_count_wms++;
            $title = js_ize(isset($layer[2]) && $layer[2] != '' ? $layer[2] : 'WMS');
            if ($sphericalMercator == 'true') {
              // wms layers have google projection
              $js .= "layer_options.displayProjection = new OpenLayers.Projection('EPSG:4326');\n";
              $js .= "layer_options.projection = new OpenLayers.Projection('EPSG:900913');\n";
            }
            $js .= "layer_options.opacity = 1.0;\n";
            $js .= "layer_options.displayInLegend = true;\n";
            //$js .= "layer_options.transitionEffect = 'resize';\n";
            if (isset($layer[4])) {
              $extras = mk_explode_options($layer[4]);
              //drupal_set_message('wms '.$layer[2].': '. var_export($extras, TRUE));
              foreach ($extras as $key => $value) {
                switch ($key) {
                case 'legendurl':
                  $js .= "layer_options.legend = {};\n";
                  $js .= "layer_options.legend.href = '{$value}';\n";
                  break;
                case 'info_format':
                  $js .= "layer_options.info_formats = ['{$value}'];\n";
                  break;
                case 'select_params':
                  drupal_set_message('select params: ' . var_export($value, TRUE));
                  break;
                default:
                  $js .= mk_add_properties_js('layer_options', $key, $value);
                  break;
                }
              }
            }
            
            $js .= "var request_params = {layers: '{$layer[3]}', format: 'image/png', transparent: 'TRUE'};\n";
            // try to pass additional wms request options
            //$parts = parse_url($layer[1], PHP_URL_QUERY);
            //drupal_set_message(var_export($parts, TRUE));
            
            // add params to request_params
            $js .= "if (layer_options.params) { OpenLayers.Util.extend(request_params, layer_options.params); }\n";
            
            // set baselayer if not yet set and no baselayer in map
            $js .= "if (!\$defined(layer_options.isBaseLayer)) { layer_options.isBaseLayer = !\$defined(map.baseLayer); }\n";
            
            $js .= "var wms = new OpenLayers.Layer.WMS('{$title}', '{$layer[1]}', request_params, layer_options);\n";
            // gutter for demis world maps: 18
            $js .= "wms.gutter = 0;\n";
            $js .= "map.addLayer(wms);\n";
            break;
            
          case 'wfs': // wfs|service|title|typename|options
            $title = js_ize(isset($layer[2]) && $layer[2] != '' ? $layer[2] : 'WFS');
            $js .= "var wfs_params = {typename: '{$layer[3]}', maxfeatures: 100};\n";
            $js .= "var wfs_options = {extractAttributes: true};\n";
            $js .= "if (map.displayProjection) { wfs_options.projection = map.displayProjection; }\n;";
            if (isset($layer[4])) {
              $extras = mk_explode_options($layer[4]);
              //drupal_set_message('extras = '. var_export($extras, TRUE));
              foreach ($extras as $key => $value) {
                switch ($key) {
                case 'maxfeatures':
                  $js .= "wfs_params.maxfeatures = $value;\n";
                  break;
                  
                case 'extract':
                  $js .= "wfs_options.extractAttributes = $value;\n";
                  break;
                
                case 'defaultStyle':
                case 'styleMap':
                  // TODO: make independent of order
                  $js .= "var style_options = {};\n";
                  $js .= mk_add_properties_js('style_options', $value);
                  $js .= "wfs_options.styles = {'default': new OpenLayers.Style(style_options)};\n";
                  break;
                  
                case 'selectStyle':
                  // TODO: make independent of order
                  $js .= "var style_options = {};\n";
                  $js .= mk_add_properties_js('style_options', $value);
                  $js .= "wfs_options.styles.select = new OpenLayers.Style(style_options)};\n";
                  break;
                  
                case 'style':
                  $js .= "wfs_options.style = OpenLayers.Util.extend({}, OpenLayers.Feature.Vector.style['default']);\n";
                  $js .= mk_add_properties_js('wfs_options.style', $value);
                  break;
                
                case 'uniqueValueProperty':
                  $js.= "wfs_options.uniquevaluerules_property = '$value';\n";
                  break;
                  
                case 'uniqueValueFillColor':
                  $js .= "wfs_options.uniquevaluerules_symbolizers = {};\n";
                  foreach ($value as $prop_val => $color) {
                    $js .= "wfs_options.uniquevaluerules_symbolizers['$prop_val'] = {fillColor: '$color'};\n";
                  }
                  break;
                  
                case 'restrictedExtent':
                case 'restrictedextent':
                  $bounds = substr($value, 1, -1);
                  $js .= "wfs_options.restrictedExtent = OpenLayers.Bounds.fromString('$bounds');\n";
                  $js .= "if (map_options.displayProjection) { wfs_options.restrictedExtent.transform(map_options.displayProjection, map_options.projection); }\n";
                  break;
                  
                default:
                  // warning?
                  $js .= mk_add_properties_js('wfs_options', $key, $value);
                  break;
                }
              }
              $js .= "if (wfs_options.styles) { wfs_options.styleMap = new OpenLayers.StyleMap(wfs_options.styles); }\n";
              if (!empty($extras['uniqueValueProperty'])) {
                // uniquevaluerules
                $js .= "wfs_options.styleMap.addUniqueValueRules('default', wfs_options.uniquevaluerules_property, wfs_options.uniquevaluerules_symbolizers);\n";
              }
            }
            // todo: extractFeatures: false/true?
            // old style feature class? featureClass: OpenLayers.Feature.WFS
            $js .= "var wfs = new MappingKit.Layer.WFS('{$title}', '{$layer[1]}', wfs_params, wfs_options);\n";
            $js .= "map.addLayer(wfs);\n";
            
            /*
            // hover control
            // todo: what with > 1 wfs layer??
            //$js .= "var selectControl = new OpenLayers.Control.SelectFeature(wfs, {onSelect: MappingKit.wfsFeaturePopup, onUnselect: MappingKit.wfsFeatureUnPop, hover: false});\n";
            $js .= "var selectControl = new OpenLayers.Control.SelectFeature(wfs, {onSelect: MappingKit.onFeatureSelect, onUnselect: MappingKit.onFeatureUnselect, hover: false});\n";
            $js .= "map.addControl(selectControl);\n";
            $js .= "selectControl.activate();\n";
            $js .= "MappingKit.selectControls[MappingKit.selectControls.length] = selectControl;\n";
            */
            $js .= "selectableLayers.push(wfs);\n"; // add to selectablelayers
            $js .= "wfs.events.on({'featureselected': MappingKit.handleFeatureSelected, 'featureunselected': MappingKit.handleFeatureUnselected});\n";
            
            if (!empty($map['timeline'])) {
              $timeline = mk_explode_options($map['timeline']);
              $tl_layers = array_keys($timeline['layers']);
              if (in_array($layer_id, $tl_layers)) {
                //$js .= "wfs._timeline = " . drupal_to_js($timeline['layers'][$layer_id]) . ";\n";
                //$js .= "wfs.events.register('loadend', wfs, MappingKit.Timeline.onLoadEnd);\n";
                
                $js .= "MappingKit.Timeline.linkMapLayer('{$map['id']}', wfs, " . drupal_to_js($timeline['layers'][$layer_id]) . ");\n";
              }
            }
            break;
            
          default:
            $function = 'mappingkit_openlayers_finish_layer_' . $layer[0];
            if (function_exists($function)) {
              $js .= $function($layer);
            }
            else {
              drupal_set_message(t('Unknown layer type %type', array('%type' => $layer[0])), 'error');
            }
            break;
          }
        }
      }
    }
    
    // add selectControl in case of selectable layers
    $js .= "if (selectableLayers.length > 0) {\n";
    // Use 1 select control for all collected selectable layers.
    // Feature(un)selected events (or listeners) must be defined on the layers
    $js .= "var selectControl = new OpenLayers.Control.SelectFeature(
      selectableLayers, {
      clickout: true,
      multiple: false,
      hover: false
      });\n";
    $js .= "map.addControl(selectControl);\n";
    $js .= "selectControl.activate();\n";
    $js .= "MappingKit.selectControls.push(selectControl);\n";
    $js .= "};\n";
    
    // (5) controls
    if (in_array('layerswitcher', $controls)) {
      // add layer switcher
      $js .= "map.addControl(new MappingKit.Control.LayerSwitcher({title: Drupal.t('Layer switcher')}));\n";
    }
    if (in_array('scalebar', $controls)) {
      // scale line
      $scaleline_units = variable_get('mappingkit_openlayers_scaleline_units', '');
      $js .= "var scaleline_options = {};\n";
      if ($scaleline_units != '') {
        $js .= "scaleline_options.topOutUnits = '$scaleline_units';\n";
        $js .= "scaleline_options.bottomOutUnits = '';\n";
      }
      $js .= "map.addControl(new OpenLayers.Control.ScaleLine(scaleline_options));\n";
    }

    if (in_array('toolbar', $controls)) {
      // panel container with panels
      $js .= "var default_container = document.getElementById('{$map['id']}-default-panel');\n";
      $js .= "var default_panel = new OpenLayers.Control.Panel({div: default_container});\n";
      $js .= "map.addControl(default_panel);\n";
      
      // mode panel
      $js .= "var navigationControl = new OpenLayers.Control.Navigation({title: Drupal.t('Default navigation')});\n";
      $js .= "var mode_panel = new OpenLayers.Control.Panel({div: document.getElementById('{$map['id']}-mode-panel')});\n";
      $js .= "map.addControl(mode_panel);\n";
      $js .= "mode_panel.addControls([navigationControl]);\n";
      
      // action panel
      $js .= "var action_panel = new OpenLayers.Control.Panel({div: document.getElementById('{$map['id']}-action-panel')});\n";
      $js .= "map.addControl(action_panel);\n";
      
      // (5a) default panel (navigationhistory, zoomtomaxextent)
      $js .= "if (OpenLayers.Control.NavigationHistory) {\n";
        // history navigation (not available in OpenLayers 2.5)
        $js .= "nav = new OpenLayers.Control.NavigationHistory();\n";
        $js .= "map.addControl(nav);\n";
        $js .= "nav.next.title = Drupal.t('Next extent');\n";
        $js .= "nav.previous.title = Drupal.t('Previous extent');\n";
        
        //$js .= "try {\n"; // for IE?
        $js .= "default_panel.addControls([nav.previous, nav.next]);\n";
        //$js .= "} catch (e) { alert(e.message); }\n";
      $js .= "}\n";
      if ($initial_extent) {
        // control to zoom to initial extent
        $js .= "var initialextent = new OpenLayers.Bounds($initial_extent);\n";
        $js .= "if (map_options.displayProjection) { initialextent.transform(map_options.displayProjection, map_options.projection); }\n";
        $js .= "default_panel.addControls([new OpenLayers.Control.ZoomToExtent({title: Drupal.t('Zoom to the initial extent'), extent: initialextent})]);\n";
      }
      else {
        // control to zoom to max extent
        $js .= "default_panel.addControls([new OpenLayers.Control.ZoomToMaxExtent({title: Drupal.t('Zoom to the maximum extent')})]);\n";
      }
      
      // ruler control
      // todo: check for ol 2.8
      if (in_array('ruler', $controls)) {
        $js .= "mode_panel.addControls([new OpenLayers.Control.Ruler({title: Drupal.t('Measure distances'), 'div': '{$map['id']}-statusbar-text'})]);\n";
      }
      
      if (!empty($map['search'])) {
        // select-mode/clear-selection-action controls
        $search = mk_explode_options($map['search']);
        // merge with defaults
        $search = array_merge(array(), $search);
        
        // add code for search box
        $js .= "
        ";
        
        
      }
      
      if (!empty($map['select'])) {
        // select-mode/clear-selection-action controls
        $select = mk_explode_options($map['select']);
        //drupal_set_message(var_export(array('text' => $map['select'], 'options' => $select), TRUE));
        $select = array_merge(array('featurePrefix' => 'ms', 'version' => '1.1.0', 'maxFeatures' => 100), $select);
        
        $js .= "
function selectedFeaturesCallback(response) {
  MappingKit.selectedFeaturesLayer.addFeatures(response.features);
  // todo: zoom to features?
};

var layers = map.getLayersByName('{$select['layer']}');
if (layers.length > 0) {
  
  var targetLayer = layers[0];
  var selectInputId = '{$select['input']}';
  
  MappingKit.selectedFeaturesLayer = new OpenLayers.Layer.Vector('_selectedFeatures_', {
    styleMap: new OpenLayers.StyleMap({'default': {
      strokeColor: 'blue',
      strokeOpacity: 1,
      strokeWidth: 2,
      fillColor: 'blue',
      fillOpacity: 0.4,
      label: '\${{$select['attribute']}}', // name of label attribute
      graphicTitle: '\${{$select['attribute']}}', // show label in tooltip
      fontColor: '#000000',
      fontSize: '12px',
      fontFamily: 'Courier New, monospace',
      fontWeight: 'bold'
    }})
    , isBaseLayer: false
    , displayInLayerSwitcher: false
    // same visibility as wms layer?
    //, minResolution: layers[0].minResolution
    //, maxResolution: layers[0].maxResolution
  });
  map.addLayers([MappingKit.selectedFeaturesLayer]);
  
  MappingKit.selectedFeaturesControl = new OpenLayers.Control.GetFeature({
    protocol: OpenLayers.Protocol.WFS.fromWMSLayer(layers[0]
    , { featurePrefix: '{$select['featurePrefix']}',
        version: '{$select['version']}',
        maxFeatures: {$select['maxFeatures']}
      }),
    box: true,
    hover: false,
    multipleKey: 'shiftKey',
    toggleKey: 'ctrlKey'
  });
  
  // set events
  MappingKit.selectedFeaturesControl.events.register('featureselected', this, function(e) {
    // only if inRange!
    if (!(targetLayer.isBaseLayer || targetLayer.inRange)) {
      return;
    }
    var values = [];
    var features = MappingKit.selectedFeaturesLayer.features;
    var new_id = e.feature.data['{$select['attribute']}'];
    for (var i = 0; i < features.length; i++) {
      if (features[i].data['{$select['attribute']}'] == new_id) {
        // id is already selected
        return;
      }
      values.push(features[i].data['{$select['attribute']}']);
    }
    var joined_values = values.join(',');
    if (joined_values.length > 0) {
      joined_values += ',';
    }
    joined_values += new_id;
    var maxlength = $('#{$select['input']}').attr('maxlength');
    if (! maxlength) {
      maxlength = -1;
    }
    if (maxlength <= 0 || joined_values.length <= maxlength) {
      MappingKit.selectedFeaturesLayer.addFeatures([e.feature]);
      
      // update field value
      selectInputId.length > 0 && $('#{$select['input']}').val(joined_values);
    }
    else {
      // todo: warning (only 1) and/or can't select feature
      //alert('too much');
    }
  });
  
  MappingKit.selectedFeaturesControl.events.register('featureunselected', this, function(e) {
    // only if inRange!
    if (!(targetLayer.isBaseLayer || targetLayer.inRange)) {
      return;
    }
    MappingKit.selectedFeaturesLayer.removeFeatures([e.feature]);
    // update field
    var values = [];
    var features = MappingKit.selectedFeaturesLayer.features;
    for (var i = 0; i < features.length; i++) {
      values.push(features[i].data['{$select['attribute']}']);
    }
    selectInputId.length > 0 && $('#{$select['input']}').val(values.join(','));
  });
  map.addControl(MappingKit.selectedFeaturesControl);
  
  // initialize features layer
  var values_text = '{$select['input']}' != '' ? $('#{$select['input']}').val() : '';
  var values = (values_text && values_text.length) > 0 ? values_text.split(',') : [];
  if (values.length > 0) {
    // read these features from wfs layer
    var filters = [];
    for (var i = 0; i < values.length; i++) {
      filters.push( new OpenLayers.Filter.Comparison({
              type: OpenLayers.Filter.Comparison.EQUAL_TO,
              property: '{$select['attribute']}',
              value: values[i]
          }));
    }
    var filter;
    if (values.length > 1) {
      var filter = new OpenLayers.Filter.Logical({
        type: OpenLayers.Filter.Logical.OR,
        filters: filters
        });
    }
    else {
      var filter = filters[0];
    }
    MappingKit.selectedFeaturesControl.protocol.read({filter: filter, callback: selectedFeaturesCallback});
  }
}
";
        
        $js .= "mode_panel.addControls([new OpenLayers.Control.Button({type: OpenLayers.Control.TYPE_TOOL, displayClass: 'SelectButton', eventListeners: {'activate': function() { MappingKit.selectedFeaturesControl.activate(); }, 'deactivate': function() { MappingKit.selectedFeaturesControl.deactivate(); }}, title: Drupal.t('Select')})]);\n";
        $js .= "action_panel.addControls([new OpenLayers.Control.Button({displayClass: 'ClearSelectButton', title: Drupal.t('Clear current selection'), trigger: function() { MappingKit.selectedFeaturesLayer.destroyFeatures(); $('#{$select['input']}').val(''); }})]);\n";
      }
      
      // feature info
      if (in_array('info', $controls)) {
        $js .= "try {\n"; // for IE?
          $js .= "var control = new MappingKit.Control.FeatureInfo({title: Drupal.t('Feature info')});\n";
          $js .= "mode_panel.addControls([control]);\n";
        $js .= "} catch (e) { alert(e.message); }\n";
      }
      
      // control to show (save?) map context
      if (in_array('context', $controls)) {
        $js .= "var control = new MappingKit.Control.MapContext({title: Drupal.t('Get current map context')});\n";
        $js .= "action_panel.addControls([control]);\n";
      }
      
      // only wms layers can be printed
      if (module_exists('map2pdf') && user_access('print map to pdf') 
        //&& (isset($map['contexts']) || $layer_count_wms > 0)
       ) {
        $js .= "var control = new MappingKit.Control.PrintContext({title: Drupal.t('Print current map')});\n";
        $js .= "action_panel.addControls([control]);\n";
      }
      // view switcher
      if (isset($map['streetview'])) {
        $js .= "MappingKit.streetview = new MappingKit.Control.StreetView('embedded_map_main', {isDraggable: true, title: Drupal.t('Toggle street view popup')});\n";
        $js .= "action_panel.addControls([MappingKit.streetview]);\n";
      }
      // help control
      if (isset($map['help'])) {
        $js .= "var control = new MappingKit.Control.Help({title: Drupal.t('Help'), 'url': '".mapclient_get_url($map['help'])."'});\n";
        $js .= "action_panel.addControls([control]);\n";
      }
    }
    
    // (5b) other panels
    if (in_array('edit', $controls)) {
      // editing vector layer
      $title = t('Drawing layer');
      $js .= "var vlayer = new OpenLayers.Layer.Vector('$title', options);\n";
      $js .= "map.addLayer(vlayer);\n";
      // editing toolbar
      $js .= "var container = document.getElementById('{$map['id']}-edit-panel');\n";
      $js .= "var edit_panel = new OpenLayers.Control.EditingToolbar(\n";
      $js .= "    vlayer, {div: container}\n";
      $js .= ");\n";
      $js .= "edit_panel.controls[0].panel_div.title = Drupal.t('Pan');\n";
      $js .= "edit_panel.controls[1].panel_div.title = Drupal.t('Add point feature');\n";
      $js .= "edit_panel.controls[2].panel_div.title = Drupal.t('Add line feature');\n";
      $js .= "edit_panel.controls[3].panel_div.title = Drupal.t('Add polygon feature');\n";
      // modify feature with RESHAPE|DRAG (and not ROTATE|RESIZE)
      $js .= "var modifyOptions = {title: Drupal.t('Modify feature'), mode: OpenLayers.Control.ModifyFeature.RESHAPE | OpenLayers.Control.ModifyFeature.DRAG};\n";
      $js .= "modifyOptions.onDelete = function(feature) { alert('delete ' + feature.id); };\n";
      $js .= "modifyOptions.onModification = MappingKit.OpenLayers.onFeatureModification;\n";
      $js .= "edit_panel.addControls(new OpenLayers.Control.ModifyFeature(vlayer, modifyOptions));\n";
      // add edit_panel to map
      $js .= "map.addControl(edit_panel);\n";
    }
    
    if (in_array('statusbar', $controls)) {
      // (5c) controls in status bar
      // scale
      //$js .= "var elem = document.getElementById('{$map['id']}-statusbar-scale');\n";
      $js .= "map.addControl(new OpenLayers.Control.Scale('{$map['id']}-statusbar-scale'));\n";
      //$js .= "elem.className = 'statusbar statusbar-scale';\n";
      // cursor coordinates
      $js .= "var elem = document.getElementById('{$map['id']}-statusbar-mouseposition');\n";
      $js .= "map.addControl(new OpenLayers.Control.MousePosition({div: elem, numDigits: $numdigits, displayProjection: new OpenLayers.Projection('$display_projection')}));\n";
      $js .= "elem.className = 'statusbar statusbar-mouseposition';\n";
      // map projection
      $js .= "var elem = document.getElementById('{$map['id']}-statusbar-projection');\n";
      $js .= "map.addControl(new OpenLayers.Control.MapProjection({div: elem, displayProjection: new OpenLayers.Projection('$display_projection')}));\n";
      // permalink
      $js .= "map.addControl(new MappingKit.Control.Permalink('{$map['id']}-statusbar-permalink'));\n";
    }
    
    if (isset($map['overview'])) {
      // See http://trac.openlayers.org/wiki/Control/OverviewMap
      // overview=list_of_overview_layers|? + overview_layers specs
      // Separate overview map block
      
      $parts = explode('|', $map['overview']);
      if (count($parts) >= 1) {
        $layers = explode(',', $parts[0]);
        foreach ($layers as $id) {
          if (isset($map[$id])) {
            
            // Add layer to overview.layers
            $layer = explode('|', $map[$id]);
            switch ($layer[0]) {
            default:
              $function = 'mappingkit_openlayers_finish_layer_' . $layer[0];
              if (function_exists($function)) {
                $js .= $function($layer, NULL);
              }
              else {
                drupal_set_message(t('Unknown layer type %type', array('%type' => $layer[0])), 'error');
              }
              break;
            }
          }
        }
      }
      
      $js .= "var overview_options = { mapOptions: {} };\n";
      $js .= "overview_options.mapOptions.maxExtent = map.maxExtent;\n";
      $js .= "overview_options.mapOptions.projection = map.projection;\n";
      $js .= "overview_options.mapOptions.units = map.units;\n";
      
      // If no layers are given, the base layer for the main map is used.
      // mapOptions can contain: maxExtent: new OpenLayers.Bounds(...), maxResolution, projection, units

      // TODO: add wms layers
      $js .= "wms.setIsBaseLayer(true);\n"; // the overview map needs a baselayer for determining the maxextent
      $js .= "overview_options.layers = [wms];\n";
      
      $js .= "var overview = new OpenLayers.Control.OverviewMap(overview_options);\n";
      $js .= "map.addControl(overview);\n";
    }
    if (in_array('overview', $controls)) {
      // default OverviewMap control
      $js .= "var overview = new OpenLayers.Control.OverviewMap();\n";
      $js .= "map.addControl(overview);\n";
    }
    
    // initial extent?
    if (isset($initial_extent)) {
      $js .= "  var initial_bounds = new OpenLayers.Bounds($initial_extent);\n";
    }
    
    // (5d) edits
    if (isset($map['edits'])) {
      $edits = explode('|', $map['edits']);
      foreach ($edits as $edit) {
        $edit_def = explode(':', $edit);
        
        // define edits layer
        $js .= "var edits_layer = new OpenLayers.Layer.Vector('_edits_', {displayInLayerSwitcher: false});\n";
        $js .= "map.addLayers([edits_layer]);\n";
        // drag control to move point
        $js .= "var edits_drag_control = new OpenLayers.Control.DragFeature(edits_layer);\n";
        $js .= "map.addControl(edits_drag_control);\n";
        $js .= "edits_drag_control.activate();\n";
        
        $js .= "\$(document).ready(function(){
          var el_x = document.getElementById('edit-x');
          var el_y = document.getElementById('edit-y');
          var center_to_location = false;
          if (el_x && el_y) {
            if (el_x.value == '') {
              // set srid to (display)projection
              var proj = map.projection;
              if (map.displayProjection) {
                proj = map.displayProjection;
              }
              MappingKit.getGeolocation(
              function(geolocation) {
                // user coords (in EPSG:4326)
                var location = new OpenLayers.LonLat(geolocation.coords.longitude, geolocation.coords.latitude);
                // set edit fields to this location
                el_x.value = geolocation.coords.longitude;
                el_y.value = geolocation.coords.latitude;
                // transform to map projection
                if (map.displayProjection) {
                  location.transform(map.displayProjection, map.projection);
                }
                // set edit feature to this location
                var layers = map.getLayersByName('_edits_');
                layers[0].features[0].move(location);
                // zoom map to this location
                map.setCenter(location, Math.round(map.numZoomLevels/2));
              },
              function() {
                //alert('failure');
                // do nothing
              });
              /*
              if (MappingKit.settings.geolocation) {
                // center in display projection
                var display_center = new OpenLayers.LonLat(MappingKit.settings.geolocation.coords.longitude, MappingKit.settings.geolocation.coords.latitude);
                // set center of map to location
                var center_point = new OpenLayers.Geometry.Point(MappingKit.settings.geolocation.coords.longitude, MappingKit.settings.geolocation.coords.latitude);
                if (edits_layer.map.displayProjection) {
                  center_point.transform(edits_layer.map.displayProjection, edits_layer.map.projection);
                }
                var center = new OpenLayers.LonLat(center_point.x, center_point.y);
                center_to_location = true;
              }*/
              if (typeof(initial_bounds) != 'undefined') {
                // center in display projection
                var display_center = initial_bounds.getCenterLonLat();
                // convert to internal projection
                var center = new OpenLayers.LonLat(display_center.lon, display_center.lat);
                if (map.displayProjection) {
                  center.transform(map.displayProjection, map.projection);
                }
              }
              else {
                // center in internal projection
                var center = map.maxExtent.getCenterLonLat();
                // determine display_center
                var display_center = new OpenLayers.LonLat(center.lon, center.lat);
                if (map.displayProjection) {
                  display_center.transform(map.projection, map.displayProjection);
                }
              }
              // assign to display_center
              el_x.value = display_center.lon;
              el_y.value = display_center.lat;
              
              var el_srid = document.getElementById('edit-srid');
              if (el_srid) {
                el_srid.value = proj.projCode.split(':')[1];
              }
              var center_point = new OpenLayers.Geometry.Point(center.lon, center.lat);
            }
            else {
              // set center of map to location (in internal projection)
              var center_point = new OpenLayers.Geometry.Point(parseFloat(el_x.value), parseFloat(el_y.value));
              if (edits_layer.map.displayProjection) {
                center_point.transform(edits_layer.map.displayProjection, edits_layer.map.projection);
              }
              var center = new OpenLayers.LonLat(center_point.x, center_point.y);
              center_to_location = true;
            }
            edits_layer.addFeatures([new OpenLayers.Feature.Vector(center_point)]);
            // set center of map to this location
            if (typeof(initial_bounds) == 'undefined' || center_to_location) {
              if (!map.getCenter()) {
                map.setCenter(center, Math.round(map.numZoomLevels/2));
              }
            }
            
            //alert(el_x.value + ' ' + el_y.value);
          }
        });\n";
        /*
        // point control to (re)locate point
        $js .= "var edits_point_control = new OpenLayers.Control.DrawFeature(edits_layer, OpenLayers.Handler.Point);\n";
        $js .= "map.addControl(edits_point_control);\n";
        $js .= "edits_point_control.activate();\n";
        */
        
        // todo: set first point to form field values
        //$js .= "edits_layer.addFeatures([new OpenLayers.Feature.Vector(new OpenLayers.Geometry.Point(edits_layer.map.getCenter().lon, edits_layer.map.getCenter().lat))]);\n";
        
        $js .= "edits_drag_control.onDrag = function(feature, pixel){
        var el_x = document.getElementById('edit-x');
        var el_y = document.getElementById('edit-y');
        if (el_x && el_y) {
          var lonlat = feature.layer.map.getLonLatFromPixel(pixel);
          // todo: transform lonlat object?
          if (feature.layer.map.displayProjection) {
            lonlat.transform(feature.layer.map.projection, feature.layer.map.displayProjection);
          }
          el_x.value = lonlat.lon;
          el_y.value = lonlat.lat;
          //OpenLayers.Console.debug('feature', feature);
        }
        }\n";
        
        switch ($edit_def) {
        case 'point':
          // form fields in $edit_def[1]
          break;
        }
      }
    }
    
    // (6) zoom
    $js .= "if (!map.getCenter()) {\n";
    if (isset($map['zoom'])) {
      // zoom if center hasn't yet been set by permalink
      $js .= "  var center = new OpenLayers.LonLat({$center[0]}, {$center[1]});\n";
      $js .= "  if (map.displayProjection) { center.transform(map.displayProjection, map.getProjectionObject()); }\n";
      $js .= "  map.setCenter(center, $zoom);\n";
    }
    else if (isset($initial_extent)) {
      $js .= "  if (map.displayProjection) { initial_bounds.transform(map.displayProjection, map.getProjectionObject()); }\n";
      $js .= "  map.zoomToExtent(initial_bounds);\n";
    }
    else {
      // zoom to maxextent if center hasn't yet been set by permalink
      $js .= "  map.zoomToMaxExtent();\n";
    }
    $js .= "}\n";
    
    // (7) listeners, error handling
    $js .= "map.events.register('movestart', map, MappingKit.movestart);\n";
    $js .= "map.events.register('removelayer', map, MappingKit.removelayer);\n";
    
    // firebug
    $_SESSION['firebug'] && $js .= "OpenLayers.Console.debug('resolutions', map.resolutions);\n";
  }
  $js .= "}\n";
  $js .= "//-->\n";
  
  // add js mappingkit init function
  drupal_add_js($js, 'inline', 'footer');
  
  // add js onload code
  drupal_add_js("
    if (Drupal.jsEnabled) {
      // ready is ok in Firefox, but not in IE6
      \$(document).ready(MappingKit.init);
      /*
      // alternative for ready, works in IE6 also
      if (window.addEventListener) {
        window.addEventListener('load', MappingKit.init, false);
      }
      else if (window.attachEvent) {
        window.attachEvent('onload', MappingKit.init);
      }
      else {
        window.onload = MappingKit.init;
      }
      */
    }", 'inline', 'footer');
  // use onload if IE gives problems for VML
  //global $onload; $onload = "MappingKit.init()";
}

/**
 * Escape single quotes in a string with a backslash for javascript.
 */
function js_ize($str) {
  return str_replace("'", "\\'", $str);
}

/**
 * @return javascript for baselayer options
 */
function get_baselayer_options_js($baselayer_options = NULL) {
  $js = "var layer_options = {};\n";
  if (!$baselayer_options) {
    return $js;
  }
  
  foreach($baselayer_options as $option => $value) {
    switch ($option) {
      case 'opacity':
        $js .= "layer_options.opacity = $value;\n";
        break;
      case 'minzoomlevel':
        $js .= "layer_options.MIN_ZOOM_LEVEL = $value;\n";
        break;
      case 'numzoomlevels':
        $js .= "layer_options.numZoomLevels = $value;\n";
        break;
      default:
        $js .= "layer_options.$option = $value;\n";
        break;
    }
  }
  return $js;
}

/**
 *
 */
function mk_add_properties_js($obj, $key, $value = NULL) {
  $js = '';
  if (is_array($key)) {
    // key contains key/value pairs
    foreach ($key as $propname => $propvalue) {
      $js .= mk_add_properties_js($obj, $propname, $propvalue);
    }
  }
  else if (is_array($value)) {
    switch ($key) {
    case 'popupSize':
      $js .= "$obj.$key = new OpenLayers.Size();\n";
      break;
    default:
      $js .= "if (!$obj.$key) $obj.$key = {};\n";
      break;
    }
    $js .= mk_add_properties_js("$obj.$key", $value);
  }
  else if (!empty($value)) {
    switch ($key) {
    case 'icon':
      $icon_url = mapclient_get_url($value);
      $js .= "$obj.icon = new OpenLayers.Icon('{$icon_url}');\n";
      break;
    default:
      if (is_numeric($value) || $value == 'true' || $value == 'false') {
        $js .= "$obj.$key = $value;\n";
      }
      else {
        $js .= "$obj.$key = '$value';\n";
      }
      break;
    }
  }
  return $js;
}

function new_OpenLayers_Layer($layer) {
  switch ($layer[0]) {
  case 'image':
    return new OpenLayers_Image_Layer($layer);
  }
}

class OpenLayers_Layer {
  public $type;
  
  function __construct() {
  }
}

class OpenLayers_Image_Layer extends OpenLayers_Layer {
  function __construct() {
    parent::__construct();
    
    $this->type = 'image';
  }
}

/**
 * @param $layer
 *   Array of layer specification. Order: image|url|title|size.
 * @return
 *   Javascript code for wms layer
 */
function mappingkit_openlayers_finish_layer_image($layer, $js_map = 'map') {
  $js = '';
  
  // image|url|title|size
  $url = mapclient_get_url($layer[1]);
  
  $title = t('Image');
  if (!empty($layer[2])) {
    $title = $layer[2];
  }
  $title = js_ize($title); // check title for '
  
  $js .= "var image = new OpenLayers.Layer.Image('{$title}', '{$url}', map_options.maxExtent, new OpenLayers.Size({$layer[3]}), layer_options);\n";
  $js .= "{$js_map}.addLayer(image);\n";
  return $js;
}

function mappingkit_openlayers_finish_layer_gpx($layer) {
  // gpx|url|title|marker|options
  $js = '';
  
  return $js;
}

function mappingkit_openlayers_finish_layer_wms($layer, $js_map = 'map') {
  // wms|url|title|layer names|options
  $js = '';
  
  $title = js_ize(empty($layer[2]) ? 'WMS' : $layer[2]);
  
  $js .= "var layer_options = layer_options || {};\n";
  $js .= "layer_options.opacity = 1.0;\n";
  $js .= "layer_options.displayInLegend = true;\n";
  if (!empty($layer[4])) {
    $extras = mk_explode_options($layer[4]);
    foreach ($extras as $key => $value) {
      switch ($key) {
      case 'legendurl':
        $js .= "layer_options.legend = {};\n";
        $js .= "layer_options.legend.href = '{$value}';\n";
        break;
      case 'info_format':
        $js .= "layer_options.info_formats = ['{$value}'];\n";
        break;
      default:
        $js .= mk_add_properties_js('layer_options', $key, $value);
        break;
      }
    }
  }
  
  $js .= "var wms_options = {layers: '{$layer[3]}', format: 'image/png', transparent: 'TRUE'};\n";
  // try to pass additional wms request options
  //$parts = parse_url($layer[1], PHP_URL_QUERY);
  //drupal_set_message(var_export($parts, TRUE));
  
  if ($js_map == 'map') {
    $js .= "layer_options.isBaseLayer = !\$defined(map.baseLayer);\n";
  }
  $js .= "var wms = new OpenLayers.Layer.WMS('{$title}', '{$layer[1]}', wms_options, layer_options);\n";
  // gutter for demis world maps: 18
  //$js .= "wms.gutter = 0;\n";
  if (!empty($js_map)) {
    $js .= "{$js_map}.addLayer(wms);\n";
  }
  return $js;
}

/**
 * Add needed OpenLayers javascript files.
 */
function mappingkit_openlayers_add_needed_js_files($attrs, $scope = 'footer') {
  
  //drupal_set_message(var_export($attrs, TRUE));
  
  // Determine $for_flags for $attrs based on baselayers, contexts, controls, 
  // and layers.
  $for_flags = 0; // use -1 to include all
  
  // all baselayers
  $baselayers = array();
  
  // contexts
  if (isset($attrs['context']) || isset($attrs['contexts'])) {
    $for_flags |= FOR_MAPCONTEXT;
    
    // Determine (base)layer types in contexts
    $contexts = explode(',', $attrs['contexts']);
    foreach ($contexts as $context) {
      if (preg_match('/node\/(?P<nid>\d+)(\/revisions\/(?P<vid>\d+))?/', $context, $matches)) {
        // is a mapcontext node?
        $node = node_load($matches['nid'], isset($matches['vid']) ? $matches['vid'] : NULL);
        if ($node->type == 'mapcontext') {
          // baselayers
          if (isset($node->extension['baselayers'])) {
            $baselayers += get_baselayers($node->extension['baselayers']);
          }
        }
      }
    }
  }
  
  // baselayers
  if (!empty($attrs['baselayers'])) {
    $baselayers += get_baselayers($attrs['baselayers']);
  }

  foreach ($baselayers as $baselayer) {
    //$variant = $baselayer['variant'];
    //$baselayer_options = $baselayer['baselayer_options'];
    switch ($baselayer['baselayer_vendor']) {
    case 'google':
      $for_flags |= FOR_GOOGLE;
      break;
    case 've':
      $for_flags |= FOR_VE;
      break;
    case 'osm':
    case 'oam':
      $for_flags |= FOR_TMS;
      break;
    case 'yahoo':
      $for_flags |= FOR_YAHOO;
      break;
    }
  }
  
  // controls
  if (!empty($attrs['controls'])) {
    $controls = explode(',', $attrs['controls']);
    foreach ($controls as $control) {
      switch ($control) {
      case 'overview':
        $for_flags |= FOR_OVERVIEW;
        break;
      }
    }
    $for_flags |= FOR_CONTROLS | FOR_MAPCONTEXT;
  }
  else {
    $for_flags |= FOR_CONTROLS | FOR_MAPCONTEXT;
  }
  $for_flags |= FOR_HISTORY;
  $for_flags |= FOR_LAYERSWITCHER;
  $for_flags |= FOR_RULER;
  if ($for_flags & FOR_RULER) {
    $for_flags |= FOR_VECTOR; // for ruler vector
  }
  
  if (isset($attrs['edits'])) {
    $for_flags |= FOR_EDITS | FOR_VECTOR;
  }
  
  if (isset($attrs['select'])) {
    $for_flags |= FOR_VECTOR | FOR_WFS;
  }
  
  // layers
  if (!empty($attrs['layers'])) {
    $layers = explode(',', $attrs['layers']);
    foreach ($layers as $layer_id) {
      if (isset($attrs[$layer_id])) {
        // process layer definition
        $layer = explode('|', $attrs[$layer_id]);
        
        switch ($layer[0]) {
        case 'georss':
          $for_flags |= FOR_GEORSS | FOR_VECTOR;
          break;
        case 'gpx':
          $for_flags |= FOR_GPX | FOR_VECTOR;
          break;
        case 'image':
          $for_flags |= FOR_IMAGE;
          break;
        case 'kml':
          $for_flags |= FOR_KML | FOR_VECTOR;
          break;
        case 'tms':
          $for_flags |= FOR_TMS;
          break;
        case 'tilecache':
          $for_flags |= FOR_TILECACHE;
          break;
        case 'wfs':
          $for_flags |= FOR_WFS | FOR_VECTOR;
          break;
        case 'wms':
          // in core
          break;
        }
      }
    }
  }
  
  $path = variable_get('mappingkit_openlayers_path', MAPPINGKIT_DEFAULT_OPENLAYERS_PATH);
  if (! parse_url($path, PHP_URL_SCHEME)) {
    //drupal_set_message('scheme:'. var_export($scheme, TRUE));
  
    drupal_add_js($path, 'module', $scope);
    
    $jsfiles = array(
      array('file' => "OpenLayers/Util.js"), // CORE
      array('file' => "OpenLayers/BaseTypes.js"),
      array('file' => "OpenLayers/BaseTypes/Class.js"),
      array('file' => "OpenLayers/BaseTypes/Bounds.js"),
      array('file' => "OpenLayers/BaseTypes/Element.js"),
      array('file' => "OpenLayers/BaseTypes/LonLat.js"),
      array('file' => "OpenLayers/BaseTypes/Pixel.js"),
      array('file' => "OpenLayers/BaseTypes/Size.js"),
      array('file' => "OpenLayers/Console.js"),
      array('file' => "OpenLayers/Tween.js"),
      array('file' => "Rico/Corner.js", 'for' => FOR_LAYERSWITCHER | FOR_GEORSS), // georss popup
  //        "Rico/Color.js",
      //"Gears/gears_init.js",
      array('file' => "OpenLayers/Ajax.js"),
      array('file' => "OpenLayers/Events.js"),
      array('file' => "OpenLayers/Request.js"),
      array('file' => "OpenLayers/Request/XMLHttpRequest.js"),
      array('file' => "OpenLayers/Projection.js"),
      array('file' => "OpenLayers/Map.js"),
      array('file' => "OpenLayers/Layer.js"),
      array('file' => "OpenLayers/Icon.js"),
      array('file' => "OpenLayers/Marker.js"),
      array('file' => "OpenLayers/Marker/Box.js"),
      array('file' => "OpenLayers/Popup.js"),
      array('file' => "OpenLayers/Tile.js"),
      array('file' => "OpenLayers/Tile/Image.js"),
      array('file' => "OpenLayers/Tile/WFS.js", 'for' => FOR_WFS),
      array('file' => "OpenLayers/Layer/Image.js", 'for' => FOR_IMAGE),
      array('file' => "OpenLayers/Layer/SphericalMercator.js", 'for' => FOR_TMS | FOR_GOOGLE | FOR_VE | FOR_YAHOO),
      array('file' => "OpenLayers/Layer/EventPane.js", 'for' => FOR_GOOGLE | FOR_VE | FOR_YAHOO),
      array('file' => "OpenLayers/Layer/FixedZoomLevels.js", 'for' => FOR_GOOGLE | FOR_VE | FOR_YAHOO),
      array('file' => "OpenLayers/Layer/Google.js", 'for' => FOR_GOOGLE),
      array('file' => "OpenLayers/Layer/VirtualEarth.js", 'for' => FOR_VE),
      array('file' => "OpenLayers/Layer/Yahoo.js", 'for' => FOR_YAHOO),
      array('file' => "OpenLayers/Layer/HTTPRequest.js"),
      array('file' => "OpenLayers/Layer/Grid.js"),
      //"OpenLayers/Layer/MapGuide.js",
      //"OpenLayers/Layer/MapServer.js",
      //"OpenLayers/Layer/MapServer/Untiled.js",
      //"OpenLayers/Layer/KaMap.js",
      //"OpenLayers/Layer/KaMapCache.js",
      //"OpenLayers/Layer/MultiMap.js",
      array('file' => "OpenLayers/Layer/Markers.js", 'for' => FOR_GEORSS),
      //"OpenLayers/Layer/Text.js",                 
      //"OpenLayers/Layer/WorldWind.js",
      array('file' => "OpenLayers/Layer/WMS.js"),
      array('file' => "OpenLayers/Layer/WMS/Untiled.js"),
      //"OpenLayers/Layer/ArcIMS.js",
      array('file' => "OpenLayers/Layer/GeoRSS.js", 'for' => FOR_GEORSS),
      // "OpenLayers/Layer/Boxes.js",
      // "OpenLayers/Layer/XYZ.js",
      array('file' => "OpenLayers/Layer/TMS.js", 'for' => FOR_TMS),           
      array('file' => "OpenLayers/Layer/TileCache.js", 'for' => FOR_TILECACHE),
      array('file' => "OpenLayers/Popup/Anchored.js", 'for' => FOR_WFS | FOR_GEORSS | FOR_KML | FOR_GPX),
      array('file' => "OpenLayers/Popup/AnchoredBubble.js", 'for' => FOR_GEORSS),
      array('file' => "OpenLayers/Popup/Framed.js", 'for' => FOR_WFS | FOR_GEORSS | FOR_KML | FOR_GPX),
      array('file' => "OpenLayers/Popup/FramedCloud.js", 'for' => FOR_WFS | FOR_GEORSS | FOR_KML | FOR_GPX),
      array('file' => "OpenLayers/Feature.js"),
      array('file' => "OpenLayers/Feature/Vector.js", 'for' => FOR_CONTROLS | FOR_GEORSS | FOR_WFS | FOR_VECTOR), // for stylemap
  //        "OpenLayers/Feature/WFS.js",
      array('file' => "OpenLayers/Handler.js"),
      array('file' => "OpenLayers/Handler/Click.js"),
      
      array('file' => "OpenLayers/Handler/Hover.js", 'for' => FOR_CONTROLS),
      array('file' => "OpenLayers/Handler/Point.js", 'for' => FOR_CONTROLS),
      array('file' => "OpenLayers/Handler/Path.js", 'for' => FOR_CONTROLS),
      array('file' => "OpenLayers/Handler/Polygon.js", 'for' => FOR_CONTROLS),
      array('file' => "OpenLayers/Handler/Feature.js", 'for' => FOR_CONTROLS | FOR_GPX | FOR_WFS | FOR_VECTOR),
      
      array('file' => "OpenLayers/Handler/Drag.js"),
  //        "OpenLayers/Handler/RegularPolygon.js",
      array('file' => "OpenLayers/Handler/Box.js"),
      array('file' => "OpenLayers/Handler/MouseWheel.js"),
  //        "OpenLayers/Handler/Keyboard.js",
      array('file' => "OpenLayers/Control.js"),
      array('file' => "OpenLayers/Control/Attribution.js"),
      array('file' => "OpenLayers/Control/Button.js", 'for' => FOR_HISTORY),
      array('file' => "OpenLayers/Control/ZoomBox.js"),
      array('file' => "OpenLayers/Control/ZoomToMaxExtent.js"),
      array('file' => "OpenLayers/Control/DragPan.js"),
      array('file' => "OpenLayers/Control/Navigation.js"),
  //        "OpenLayers/Control/MouseDefaults.js",
      array('file' => "OpenLayers/Control/MousePosition.js"),
      array('file' => "OpenLayers/Control/OverviewMap.js", 'for' => FOR_OVERVIEW),
      array('file' => "OpenLayers/Control/KeyboardDefaults.js"),
      array('file' => "OpenLayers/Control/PanZoom.js"),
      array('file' => "OpenLayers/Control/PanZoomBar.js"),
      array('file' => "OpenLayers/Control/ArgParser.js"),
      array('file' => "OpenLayers/Control/Permalink.js"),         
      array('file' => "OpenLayers/Control/Scale.js"),
      array('file' => "OpenLayers/Control/ScaleLine.js"),
      array('file' => "OpenLayers/Control/LayerSwitcher.js", 'for' => FOR_LAYERSWITCHER),
      array('file' => "OpenLayers/Control/DrawFeature.js", 'for' => FOR_CONTROLS),
      array('file' => "OpenLayers/Control/DragFeature.js", 'for' => FOR_EDITS),
      array('file' => "OpenLayers/Control/GetFeature.js", 'for' => FOR_WFS),
  //        "OpenLayers/Control/ModifyFeature.js",
      array('file' => "OpenLayers/Control/Panel.js", 'for' => FOR_HISTORY),
      array('file' => "OpenLayers/Control/SelectFeature.js", 'for' => FOR_WFS | FOR_GPX | FOR_KML | FOR_VECTOR),
      array('file' => "OpenLayers/Control/NavigationHistory.js", 'for' => FOR_HISTORY),
      array('file' => "OpenLayers/Control/Measure.js", 'for' => FOR_RULER),
      array('file' => "OpenLayers/Geometry.js", 'for' => FOR_WFS | FOR_GPX | FOR_GEORSS | FOR_VECTOR),
  //        "OpenLayers/Geometry/Rectangle.js",
      array('file' => "OpenLayers/Geometry/Collection.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Geometry/Point.js", 'for' => FOR_WFS | FOR_GPX | FOR_GEORSS | FOR_VECTOR),
      array('file' => "OpenLayers/Geometry/MultiPoint.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Geometry/Curve.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Geometry/LineString.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Geometry/LinearRing.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Geometry/Polygon.js", 'for' => FOR_VECTOR),
  //        "OpenLayers/Geometry/MultiLineString.js",
  //        "OpenLayers/Geometry/MultiPolygon.js",
  //        "OpenLayers/Geometry/Surface.js",
      array('file' => "OpenLayers/Renderer.js", 'for' => FOR_CONTROLS | FOR_GPX | FOR_VECTOR),
      array('file' => "OpenLayers/Renderer/Elements.js", 'for' => FOR_CONTROLS | FOR_GPX | FOR_VECTOR),
      array('file' => "OpenLayers/Renderer/SVG.js", 'for' => FOR_CONTROLS | FOR_GPX),
  //        array('file' => "OpenLayers/Renderer/Canvas.js", 'for' => FOR_CONTROLS),
      array('file' => "OpenLayers/Renderer/VML.js", 'for' => FOR_CONTROLS | FOR_GPX | FOR_VECTOR), // for IE
      array('file' => "OpenLayers/Layer/Vector.js", 'for' => FOR_CONTROLS | FOR_GPX | FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Layer/Vector/RootContainer.js", 'for' => FOR_VECTOR),
      //"OpenLayers/Strategy.js",
      //"OpenLayers/Strategy/Fixed.js",
      //"OpenLayers/Strategy/Cluster.js",
      //"OpenLayers/Strategy/Paging.js",
      //"OpenLayers/Strategy/BBOX.js",
      array('file' => "OpenLayers/Protocol.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Protocol/HTTP.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Protocol/WFS.js", 'for' => FOR_WFS),
      array('file' => "OpenLayers/Protocol/WFS/v1.js", 'for' => FOR_WFS),
      array('file' => "OpenLayers/Protocol/WFS/v1_0_0.js", 'for' => FOR_WFS),
      array('file' => "OpenLayers/Protocol/WFS/v1_1_0.js", 'for' => FOR_WFS),
      //"OpenLayers/Protocol/SQL.js",
      //"OpenLayers/Protocol/SQL/Gears.js",
  //        "OpenLayers/Layer/PointTrack.js",
      array('file' => "OpenLayers/Layer/GML.js", 'for' => FOR_GPX | FOR_KML | FOR_GEORSS),
      array('file' => "OpenLayers/Style.js", 'for' => FOR_CONTROLS | FOR_GPX),
      array('file' => "OpenLayers/StyleMap.js", 'for' => FOR_CONTROLS | FOR_GPX),
      array('file' => "OpenLayers/Rule.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Filter.js", 'for' => FOR_WFS | FOR_VECTOR),
  //        "OpenLayers/Filter/FeatureId.js",
      array('file' => "OpenLayers/Filter/Logical.js", 'for' => FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Filter/Comparison.js", 'for' => FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Filter/Spatial.js", 'for' => FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Format.js", 'for' => FOR_MAPCONTEXT | FOR_GEORSS | FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Format/XML.js", 'for' => FOR_MAPCONTEXT | FOR_GEORSS | FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Format/GML.js", 'for' => FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Format/GML/Base.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Format/GML/v2.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Format/GML/v3.js", 'for' => FOR_VECTOR),
      array('file' => "OpenLayers/Format/KML.js", 'for' => FOR_KML | FOR_MAPCONTEXT),
      array('file' => "OpenLayers/Format/GeoRSS.js", 'for' => FOR_GEORSS),
      array('file' => "OpenLayers/Format/Filter.js", 'for' => FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Format/Filter/v1.js", 'for' => FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Format/Filter/v1_0_0.js", 'for' => FOR_WFS | FOR_VECTOR),
      array('file' => "OpenLayers/Format/Filter/v1_1_0.js", 'for' => FOR_WFS | FOR_VECTOR), // OL2.8
  //        "OpenLayers/Format/WFS.js",
      array('file' => "OpenLayers/Format/WFST.js"), // OL2.8
      array('file' => "OpenLayers/Format/WFST/v1.js"), // OL2.8
      array('file' => "OpenLayers/Format/WFST/v1_0_0.js"), // OL2.8
      array('file' => "OpenLayers/Format/WFST/v1_1_0.js"), // OL2.8
      array('file' => "OpenLayers/Format/WKT.js", 'for' => FOR_VECTOR), // vector on IE?
      //"OpenLayers/Format/OSM.js",
      array('file' => "OpenLayers/Format/GPX.js", 'for' => FOR_GPX),
      array('file' => "OpenLayers/Format/SLD.js", 'for' => FOR_MAPCONTEXT),
      array('file' => "OpenLayers/Format/SLD/v1.js", 'for' => FOR_MAPCONTEXT),
      array('file' => "OpenLayers/Format/SLD/v1_0_0.js", 'for' => FOR_MAPCONTEXT),
      array('file' => "OpenLayers/Format/SLD/v1.js", 'for' => FOR_MAPCONTEXT),
      //"OpenLayers/Format/Text.js",
  //        "OpenLayers/Format/JSON.js",
  //        "OpenLayers/Format/GeoJSON.js",
      array('file' => "OpenLayers/Format/WMC.js", 'for' => FOR_MAPCONTEXT),
      array('file' => "OpenLayers/Format/WMC/v1.js", 'for' => FOR_MAPCONTEXT),
      array('file' => "OpenLayers/Format/WMC/v1_0_0.js", 'for' => FOR_MAPCONTEXT),
      array('file' => "OpenLayers/Format/WMC/v1_1_0.js", 'for' => FOR_MAPCONTEXT),
      array('file' => "OpenLayers/Layer/WFS.js", 'for' => FOR_WFS),
  //        "OpenLayers/Control/MouseToolbar.js",
  //        "OpenLayers/Control/NavToolbar.js",
      array('file' => "OpenLayers/Control/PanPanel.js"),        
      array('file' => "OpenLayers/Control/Pan.js"),
      array('file' => "OpenLayers/Control/ZoomIn.js"),
      array('file' => "OpenLayers/Control/ZoomOut.js"),
      array('file' => "OpenLayers/Control/ZoomPanel.js"),
  //        "OpenLayers/Control/EditingToolbar.js",
      array('file' => "OpenLayers/Lang.js"),
      array('file' => "OpenLayers/Lang/en.js")
    ); // etc.
    
    $base_dir = dirname($path);
    if (strrpos($base_dir, '/lib') === strlen($base_dir) - 4) {
      // set singleFile to prevent OL loading separate javascript files
      drupal_add_js("var OpenLayers = { singleFile: true };\n", 'inline');
      
      // Only the necessary OpenLayers files are included one-by-one by 
      // the drupal_add_js function
      foreach ($jsfiles as $jsfile) {
        if (empty($jsfile['for']) || ($jsfile['for'] & $for_flags) != 0) {
          drupal_add_js($base_dir .'/'. $jsfile['file'], 'module', $scope);
        }
      }
      $img_path = base_path() . $base_dir . '/../img/';
    }
    else {
      $img_path = base_path() . $base_dir . '/img/';
    }
    
    // Set OpenLayers image path to prevent missing images in case of 
    // drupal js caching
    drupal_add_js("OpenLayers.ImgPath = '$img_path';\n", 'inline', $scope);
  }
  
  // add custom javascript code for this drupal module
  drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/openlayers.js', 'module', $scope);
  
  // add other (extra) custom script files?
  if ($for_flags & (FOR_MAPCONTEXT | FOR_CONTROLS)) {
    // require OpenLayers WMC files
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/MapContext.Format.js', 'extra', $scope);
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/Format/OWSContext.js', 'extra', $scope);
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/Format/OWSContext/v0_3_1.js', 'extra', $scope);
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/MapContext.Control.js', 'extra', $scope);
  }
  
  drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/Legend.js', 'extra', $scope);
  
  if ($for_flags & FOR_CONTROLS) {
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/Ruler.Control.js', 'extra', $scope);
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/PrintContext.Control.js', 'extra', $scope);
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/FeatureInfo.Control.js', 'extra', $scope);
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/ZoomToExtent.Control.js', 'extra', $scope);
  }
  
  if ($for_flags & FOR_LAYERSWITCHER) {
    // Requires OpenLayers LayerSwitcher
    drupal_add_js(drupal_get_path('module', 'mappingkit_openlayers') .'/custom/LayerSwitcher.Control.js', 'extra', $scope);
  }
}

/**
 * Get Proj.4 parameter string for a projection.
 *
 * It is determined in order: cache &gt; mappingkit_proj4_definitions &gt; mappingkit_spatial_database.
 *
 * @param $proj
 *    Projection string as EPSG followed by colon and spatial reference ID. E.g. EPSG:28992.
 * @return
 *    String with proj4 parameters.
 */
function mappingkit_get_proj4text($proj) {
  $data = array();
  
  // 1) try cache
  $cache = cache_get('mappingkit_get_proj4text');
//$cache = 0;
  if ($cache != 0) {
    $data = $cache->data;
    //drupal_set_message('(1) data = '. var_export($data, TRUE));
  }
  if (!empty($data[$proj])) {
    return $data[$proj];
  }
  
  // 2) try extra definitions
  $proj4_definitions_text = variable_get('mappingkit_proj4_definitions', '');
  $proj4_definitions_lines = explode("\n", $proj4_definitions_text);
  foreach ($proj4_definitions_lines as $line) {
    $parts = explode('=', $line, 2);
    $proj4text = trim($parts[1]);
    if (count($parts) == 2 && !empty($proj4text)) {
      $data['EPSG:' . trim($parts[0])] = $proj4text;
    }
  }
  //drupal_set_message('(2) data = '. var_export($data, TRUE));
  if (!empty($data[$proj])) {
    cache_set('mappingkit_get_proj4text', $data);
    return $data[$proj];
  }
  
  // 3) try database
  $db = variable_get('mappingkit_spatial_database', '');
  if (!empty($db)) {
    $active = db_set_active($db);
    
    // postgis: select proj4text from spatial_ref_sys where srid = 28992 and auth_name = 'EPSG'
    $parts = explode(':', $proj);
    $proj4text = db_result(db_query("select proj4text from spatial_ref_sys where srid = %d and auth_name = '%s'", $parts[1], $parts[0]));
    
    db_set_active($active);
    
    if (!empty($proj4text)) {
      $data[$proj] = $proj4text;
      //drupal_set_message('(3) data = '. var_export($data, TRUE));
      cache_set('mappingkit_get_proj4text', $data);
    }
  }
  return $data[$proj];
}

