<?php

function drupal_json($var = NULL) {
  // We are returning JavaScript, so tell the browser.
  isset($_GET['debug']) || header('Content-Type: text/javascript; charset=utf-8');
  
  if (isset($var)) {
    echo drupal_to_js($var);
  }
}

function drupal_to_js($var) {
  switch (gettype($var)) {
    case 'boolean':
      return $var ? 'true' : 'false'; // Lowercase necessary!
    case 'integer':
    case 'double':
      return $var;
    case 'resource':
    case 'string':
      return '"'. str_replace(array("\r", "\n", "<", ">", "&"),
                              array('\r', '\n', '\x3c', '\x3e', '\x26'),
                              addslashes($var)) .'"';
    case 'array':
      // Arrays in JSON can't be associative. If the array is empty or if it
      // has sequential whole number keys starting with 0, it's not associative
      // so we can go ahead and convert it as an array.
      if (empty ($var) || array_keys($var) === range(0, sizeof($var) - 1)) {
        $output = array();
        foreach ($var as $v) {
          $output[] = drupal_to_js($v);
        }
        return '[ '. implode(', ', $output) .' ]';
      }
      // Otherwise, fall through to convert the array as an object.
    case 'object':
      $output = array();
      foreach ($var as $k => $v) {
        $output[] = drupal_to_js(strval($k)) .': '. drupal_to_js($v);
      }
      return '{ '. implode(', ', $output) .' }';
    default:
      return 'null';
  }
}

function coordinate_transform($inproj, $outproj, &$points) {
  
  // assert: in/outproj start with epsg:
  $inproj = strtolower($inproj);
  $outproj = strtolower($outproj);
  if (substr($inproj, 0, 5) != 'epsg:'
    || substr($outproj, 0, 5) != 'epsg:') {
    die("Cannot reproject from $inproj to $outproj");
  }
  
  $projInObj = ms_newprojectionobj("init=". $inproj);
  $projOutObj = ms_newprojectionobj("init=". $outproj);
  $poPoint = ms_newpointobj();
  //drupal_set_message('points '.var_export($points, true));
  foreach ($points as &$point) {
    $poPoint->setXY($point['x'], $point['y']);
    //drupal_set_message(var_export($poPoint,true));
    if (MS_FAILURE == $poPoint->project($projInObj, $projOutObj)) {
      return NULL;
    }
    if ($outproj == 'epsg:4326') {
      $point['lat'] = $poPoint->y;
      $point['lon'] = $poPoint->x;
    }
    else {
      $point['x'] = $poPoint->x;
      $point['y'] = $poPoint->y;
    }
  }
  return TRUE;
}

// params: x,y,lat,lon,from,to
// x/y -> from is required, to=4326 by default
// lat/lon -> to is required, from=4326 by default

$params = $_GET;

if (isset($params['x'])) {
  if (!isset($params['from'])) {
    header();
    exit();
  }
  $from = $params['from'];
  $to = isset($params['to']) ? $params['to'] : 'EPSG:4326';
  $point = array('x' => $params['x'], 'y' => $params['y']);
}
else if (isset($params['lat'])) {
  if (!isset($params['to'])) {
    header();
    exit();
  }
  $to = $params['to'];
  $from = isset($params['from']) ? $params['from'] : 'EPSG:4326';
  $point = array('x' => $params['lon'], 'y' => $params['lat']);
}

$points = array($point);
if (coordinate_transform($from, $to, $points)) {
  drupal_json($points[0]);
}

